const HOUR_S = 3600;
const HOUR_MS = 3600000;
const DAY_MS = 1000*HOUR_S*24;
function get_time_zone()
{
    return new Date().getTimezoneOffset()*60*-1;
}
function convert_time(date)
{
    return date+get_time_zone();
}
function convert_time_ms(date,type)
{
    if(!type) return date+get_time_zone()*1000;
    return date;
}
function cloneObject(obj)
{
    var newobj = new Object();
    for (var key in obj)
    {
        newobj[key]=obj[key];
    }
    return newobj;
}
function cloneArr(arr)
{
    var newarr = [];
    for (var i = 0; i<arr.length; i++)
    {
        newarr[i]=arr[i];
    }
    return newarr;
}
function isObject(obj)
{
    return typeof obj === 'object' && obj !== null;
}

class vega_converter
{
    constructor()
    {
    }
    get_time_zone()
    {
        return new Date().getTimezoneOffset()*60*-1;
    }
    timeInUtc(s)
    {
        return s - this.get_time_zone();
    }
    intToHex(number,countByte)
    {
        var result = '';
        try
        {
            if (number<0)
            {
                var maxValue = '';
                while((maxValue.length/2)<countByte)
                {
                    maxValue+='ff';
                }
                maxValue='0x'+maxValue;
                result = this.uIntToHex(Number(maxValue)+number+1);
                while((result.length/2)<countByte)
                {
                    result+='00';
                }
                return result;
            }
            else
            {
                result = this.uIntToHex(number);
                while((result.length/2)<countByte)
                {
                    result+='00';
                }
                return result;
            }
        }
        catch (e)
        {
            console.log(e);
        }
        return result;
    }
    uIntToHex(number,countByte)
    {
        try
        {
            if (typeof number === 'string' ) 
            {
                number = parseInt(number);
            }
            if (typeof number === 'number'&&!isNaN(number)&&number>=0&&number%1 === 0)
            {
                var hex = number.toString(16);
                while(hex.length%2!==0)
                {
                    hex='0'+hex;
                }
                var bytes = this.hexToBytes(hex);
                bytes.reverse();
                hex = bytes.join('');
                if( countByte !== undefined )
                {
                    while( (hex.length/2) < countByte )
                    {
                        hex+='00';
                    }
                }
                return hex;
            }
        }
        catch (e)
        {
            console.log(e);
        }
        return '';
    }
    intToBytes(number,count)
    {
        if (count!==undefined)
        {
            var hex = '';
            if (number!==undefined)
            {
                number = parseInt(number);
                if (!isNaN(number))
                {
                    hex = number.toString(16);
                }
            }
            var hex_array = [];
            var beginNulls = (count*2) - hex.length;
            while(beginNulls>0)
            {
                hex='0'+hex;
                beginNulls--;
            }
            for (var i =0;i<count*2;i+=2)
            {
                if (hex[i]!==undefined&&hex[i+1]!==undefined)
                {
                    hex_array.push( hex.substring(i, i+2) ); 
                }
                else if (hex[i]!==undefined&&hex[i].length===1)
                {
                    var byte = "0"+hex[i];
                    hex_array.push( byte ); 
                }
                else
                {
                    hex_array.unshift('00' );
                }
            }
            hex_array.reverse();
            return hex_array.join('' );
        }
        else
        {
            return '0';
        }
    }
    parse4_20mA(value,min,max)
    {
        return min+(((max-min)*(value-4))/16);
    }
    to4_20mA(value,min,max)
    {
        return  ( ( ( ( value - min ) * 16 ) / ( max - min ) ) + 4 ) ; 
    }
    round_size(num,decimal)
    {
        if (decimal==undefined||decimal<=0)
        {
            decimal = 1;
        }
        var decimal = Math.pow(10,decimal);
        return(Math.round(num*decimal)/decimal);
    }
    round_size_str(num,decimal)
    {
        var r = this.round_size(num,decimal);
        var str = r.toString();
        var arr = str.split('.' );
        var tmp = arr[1];
        if (tmp===undefined)
        {
            tmp = '';
        }
        while(tmp.length<decimal)
        {
            tmp+='0';
        }
        return arr[0]+'.'+tmp;
    }
    hexToBytes(hex)
    {
        var bytes = [];
        for (var i = 0; i < hex.length - 1; i = i+2)
        {
           bytes.push( hex.substring(i, i+2) ); 
        }
        return bytes;
    }
    bytesToParamsSettings(bytes)
    {
        var result = new Object();
        bytes.splice(0,1);
        while(bytes.length>0)
        {
            var sizeParam = this.bytesToUint( [bytes[2]]);
            var idParamHex = this.bytesToUint( [bytes[0],bytes[1]]);
            if (isNaN(sizeParam)||isNaN(idParamHex)||idParamHex===undefined)
            {
                break;
            }
            else
            {
                bytes.splice(0,3);
            }
            var paramBytes = [];
            for(var i = 0; i<sizeParam; i++)
            {
                var paramByte = bytes[i];
                if (paramByte!==undefined)
                {
                    paramBytes.push(paramByte);
                }
                else
                {
                    break;
                }
            }
            bytes.splice(0,sizeParam);
            result[idParamHex] = 
            {
                id:idParamHex,
                size:sizeParam,
                rawData:paramBytes
            }
        }
        return result;
    }
    bytesToInt(bytes)
    {
        try
        {
            var valid_arr = isObject(bytes) && bytes.length;
            if (valid_arr)
            {
                // var countMAX = 0;
                bytes.reverse();
                var result='';
                for(var i = 0; i<bytes.length;i++)
                {
                    // if (bytes[i]=='ff' ) countMAX++;
                    result+=bytes[i]===undefined?'00':bytes[i].toString();
                }
                var resultNumber = undefined;
                // if (countMAX==bytes.length)
                // {
                //     resultNumber = undefined;
                // }
                // else
                // {
                    resultNumber = parseInt(result,16);
                    var maxVal = Math.pow(2, result.length / 2 * 8);
                    if (!isNaN(resultNumber))
                    {
                        if (resultNumber > maxVal / 2 - 1) 
                        {
                            resultNumber = resultNumber - maxVal;
                        }
                    }
                // }
                return resultNumber;
            }
            else
            {
                return NaN;
            }
        }
        catch(e)
        {
            console.log(e);
            return NaN;
        }
    }
    bytesToUint(bytes)
    {
        try
        {
            var valid_arr = isObject(bytes) && bytes.length;
            if (valid_arr)
            {
                // var countMAX = 0;
                bytes.reverse();
                var result='';
                for(var i = 0; i<bytes.length;i++)
                {
                    // if (bytes[i]=='ff' ) countMAX++;
                    result+=bytes[i]===undefined?'00':bytes[i].toString();
                }
                var resultNumber = undefined;
                // if (countMAX==bytes.length)
                // {
                //     resultNumber = undefined;
                // }
                // else
                // {
                    resultNumber = parseInt(result,16);
                // }
                return resultNumber;
            }
            else
            {
                return NaN;
            }
        }
        catch(e)
        {
            console.log(e);
            return NaN;
        }
    }
}
class VegaReport extends vega_converter 
{
    constructor(types)
    {
        super();
        this.sort = 'level_2';
        this.status = false;
        this.types = types;
        this.loader = false;
        this.queueWaiting = new Object();
        this.queueSuccess = new Object();
        this.totalCount = 0;
        this.date_range = new date_range();
        this.reload();
    }
    resetProgress()
    {
        this.status = false;
        this.loader = false;
        this.queueWaiting = new Object();
        this.queueSuccess = new Object();
        this.totalCount = 0;
    }
    set progress(val)
    {
        this._progress = val;
    }
    get firstChannel()
    {
        for(var key in this.queueWaiting)
        {
            return this.queueWaiting[key];
        }
        return new Object();
    }
    get cCount()
    {
        return Object.keys(this.queueWaiting).length;
    }
    get progress()
    {
        if (this.totalCount<=0)
        {
            return 100;
        }
        else 
        {
            var progress = 100-Math.round((100*this.cCount)/this.totalCount);
            return progress>100||isNaN(progress)?100:progress;
        }
    }
    backPage()
    {
        switch (this.step) {

            case 2:
                this.step--;
                this.selectedType = undefined;
                break;
            case 3:
                this.step--;
                this.selectedObject = undefined;
                break;
            case 4:
                this.step--;
                break;
            default:
                break;
        }
    }
    nextPage()
    {
        if (!this.checkDisabled())
        {
            this.step++;
        }
    }
    reload()
    {
        this.step = 1;
        this.selectedTypeChannel = undefined;
        this.selectedType = undefined;
        this.selectedObject = undefined;
        this.sort = 'level_2';
    }
    generation()
    {
        $('#reportProgress' ).modal('hide' );
        this.loader = false;
        this.report = new Object();
        this.report.channels = [];
        this.report.sum = 0;
        var filterPackage = undefined;
        if (this.selectedType.id==5&&this.selectedTypeChannel.id==14)
        {
            filterPackage=4;
        }
        for(var key in this.queueSuccess) 
        {
            var channel = this.queueSuccess[key];
            var info = channel.getInfo(this.date_range,filterPackage);
            this.report.channels.push(info);
            this.report.sum = this.report.sum+info.consum_sum;
            this.report.sum_rate_1 = this.report.sum_rate_1+info.consum_1;
            this.report.sum_rate_2 = this.report.sum_rate_2+info.consum_2;
            this.report.sum_rate_3 = this.report.sum_rate_3+info.consum_3;
            this.report.sum_rate_4 = this.report.sum_rate_4+info.consum_4;
        }
        this.report.sum= this.round_size(this.report.sum,4);
        this.status = true;
        this.report.info = new Object();
        this.report.info.nameFile = this.selectedType.name+'-'+this.selectedObject.name+', c '+moment.unix(this.date_range.getStart()).format('DD.MM.YY' )+' по '+moment.unix(this.date_range.getEnd()).format('DD.MM.YY' );
        switch (this.selectedType.id) {
            case 1:
                if (this.selectedTypeChannel.id==1||this.selectedTypeChannel.id==14||this.selectedTypeChannel.id==16)
                {
                    this.report.info.countCol = 11;
                }
                else
                {
                    this.report.info.countCol=10;
                }
                break;
            case 2:
                if (this.selectedTypeChannel.id==1||this.selectedTypeChannel.id==14||this.selectedTypeChannel.id==16)
                {
                    this.report.info.countCol = 8;
                }
                else if ( this.selectedTypeChannel.id==23 || this.selectedTypeChannel.id==25 )
                {
                    this.report.info.countCol = 6;
                }
                else
                {
                    this.report.info.countCol=7;
                }
                break;
            case 5:
                if (this.selectedTypeChannel.id==14||this.selectedTypeChannel.id==16)
                {
                    this.report.info.countCol = 11;
                }
                else
                {
                    this.report.info.countCol=10;
                }
                break;
            default:
                this.report.info.countCol=10;
                break;
        }
        this.report.dangers = function()
        {
            var dangers = [];
            for(var i = 0; i<this.channels.length;i++)
            {
                var channel = this.channels[i];
                for(var key in channel.dangers)
                {
                    var danger = channel.dangers[key];
                    danger.level_2 = channel.level_2;
                    danger.name = channel.name;
                    danger.desc = channel.desc;
                    dangers.push(danger);
                }
            }
            return dangers;
        }
        
    }
    startFormingReport()
    {
        this.resetProgress();
        this.queueWaiting = this.getChannels();
        this.queueSuccess = new Object();
        this.status = false;
        this.totalCount = Object.keys(this.queueWaiting).length;
        this.loader = true;
    }
    getChannels()
    {
        try 
        {
            return this.selectedObject.getChannels('type_channel',this.selectedTypeChannel.id);
        } 
        catch (e) 
        {
            return new Object();
        }
    }
    checkDisabled()
    {
        switch (this.step) 
        {
            case 1:
                if (this.selectedTypeChannel)
                {
                    return false;
                }
                break;
            case 2:
                if (this.selectedType)
                {
                    return false;
                }
                break;
            case 3:
                if (this.selectedObject)
                {
                    return false;
                }
                break;
            case 4:
                    return false;
                break;
            default:
                break;
        }
        return true;
    }
    getText()
    {
        switch (this.step) {
            case 1:
                    return "Выберите тип устройств для которых Вы собираетесь выполнить запрос формирования отчета";
                break;
            case 2:
                    return "Выберите вид отчета, который вы собираетесь сформировать";
                break;
            case 3:
                    return "Выберите объект";
                break;
            default:
                    return "";
                break;
        }
    }
}
class command_list
 {
     constructor()
    {
       this.get_users=false;
       this.manage_users=false;
       this.delete_users=false;
       this.get_device_appdata=false;
       this.get_data=false;
       this.send_data=false;
       this.manage_device_appdata=false;
       this.delete_device_appdata=false;
       this.get_gateways=false;
       this.manage_gateways=false;
       this.delete_gateways=false;
       this.get_devices=false;
       this.manage_devices=false;
       this.delete_devices=false;
       this.get_coverage_map=false;
       this.get_device_downlink_queue=false;
       this.manage_device_downlink_queue=false;
    }
    selectAll()
    {
        var valid = new valid_data();
        for (var key in this)
        {
            if (valid.isBool(this[key]))
            {
                this[key]=true;
            }
        }
    }
    unselectAll()
    {
        var valid = new valid_data();
        for (var key in this)
        {
            if (valid.isBool(this[key]))
            {
                this[key]=false;
            }
        }
    }
    set_patterns(pattern)
    {
        var valid = new valid_data();
        for (var key in pattern)
        {
            if (pattern[key]!=undefined&&valid.isBool(pattern[key])&&this[key]!=undefined&&valid.isBool(this[key]))
            {
                this[key]=pattern[key];
            }
        }
    }
    set_command_list_array(c_list)
    {
        var valid = new valid_data();
        if (c_list!==undefined&&c_list.length>0)
        {
            for (var i = 0 ; i<c_list.length;i++)
            {
                if (this[c_list[i]]!==undefined&&valid.isBool(this[c_list[i]]))
                {
                    this[c_list[i]]=true;
                }
            }  
        }
    }
    get_command_list_array()
    {
        var valid = new valid_data();
        var temp_array = new Array();
        for (var key in this)
        {
            if (valid.isBool(this[key]))
            {
                temp_array.push(key);
            }
        }
        return temp_array;
    }
 }
class sticky
{
    constructor()
    {
        this.general_info = true;
        this.summary_info = true;
        this.chart_consum = false;
        this.chart_consum_new = false;
        this.chart_consum1 = false;
        this.chart_consum2 = false;
        this.chart_consum4 = false;
        this.chart_temperature = false;
        this.chart_conn = false;
        this.danger=false;
        this.chart_consum_danger=false;
        this.data = false;
        this.data_event = false;
        this.data_energy = false;
        this.data_energy_archive = false;
        this.data_slices = false;
        this.data_journals = false;
        this.data_history = false;
        this.data_instant = false;
        this.data_instant2 = false;
        this.data_info = false;
        this.data_info2 = false;
        this.data_jp = false;
        this.data_rl = false;
        this.menu=true;
        this.map=true;
    }
    toogle(str)
    {
        this[str]=this[str]?false:true;
    }
}

class report_consum
{
    constructor()
    {
        this.level_1 = new Array();
        this.level_2 = new Array();
        this.type_channel = [''];
        this.device = [];
        this.channel = [];
        this.date_range = new date_range();
        this.interval;
        this.good_devices = new Array();
    }
    check_all_channel_report()
    {
        for (var i=0;i<this.channel.length;i++)
        {
            this.channel[i].tick=true;
        }
    }
    none_all_channel_report()
    {
        for (var i=0;i<this.channel.length;i++)
        {
            this.channel[i].tick=false;
        }
    }
    get()
    {
        return this.channel;
    }
    search_device()
    {
    }
}
class valid_data 
{
    constructor()
    {
        
    }
    valid(str)
    {
        if (str!==undefined&&str.toString().length>0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    name(str)
    {
        if (this.valid(str)&&str.length<=25)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    comment(str)
    {
        if (this.valid(str)&&str.length<=200)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    address(str)
    {
        if (this.valid(str))
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    isSMS(str)
    {
         if (this.valid(str)&&str.length<=70)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    isMessenger(str)
    {
         if (this.valid(str)&&str.length<=1000)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    isInt(str)
    {
        if (this.isNumber(str)&&str % 1 === 0)
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    devEui(str)
    {
        return (this.valid(str)&&str.length===16);
    }
    isNumber(str)
    {
        if (str!==undefined&&!isNaN(str)&&str!=="")
        {
            return true;
        }
        else
        {
            return false;
        }
    }
    isBool(str)
    {
        if (typeof str==="boolean")
        {
            return true;
        }
        else return false;
    }
    isDate(str)
    {
        if (str!==undefined)
        {
            try
            {
                var date = Math.abs(new Date(str).getTime());
                if (date>0&&this.isInt(date))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch(err)
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }
}
class edit_device
{
    constructor(type)
    {
        this.device = new device();
        this.level_1 = new Object();
        this.level_2;
        this.num_channel;
        this.status;
        this.color = new Object();
        this.info_channel;
        this.init_value;
        this.kt;
        this.name;
        this.type_channel = new Object();
        this.division;
        this.history;
        this.calib_interval;
        this.ia;
        this.model;
        this.serial;
        this.personal_num;
        this.type_meter = 0;
        this.title = 'Создание нового устройства';
        this._init_status=false;
        this.type=type;
        this.security_name;
        this.hall_1_name;
        this.hall_2_name;
        this.water;
        this.rason;
        this.min_t=-55;
        this.max_t=100;
        this.vega_valid =  new valid_data();
        
       
        this.instrument;
        this.instrument2;
        this.unit;
        this.min_v;
        this.max_v;
        this.min_normal_v;
        this.max_normal_v;
        this.security_name_2;
        this.sensor_name_1;
        this.sensor_name_2;
        
        this.provider;
        this.typeCargo = '1';
        this.cargo;
        
        this.name_sensor_in_1;
        this.name_sensor_in_2;
        this.name_sensor_out_1;
        this.name_sensor_out_2;
        
        
        this.state_sensor_0;
        this.state_sensor_1;
        this.telephones='';
        this.emails='';
        this.telegram_chats = '';
        this.message_sms = '';
        this.message_messenger = '';
        this.sms=false;
        this.email=false;
        this.telegram=false;
        this.enable_danger = false;
        this.voice = false;
        this.voice_message = '';
        this.count_rate;
        this.count_phase = 1;
        this.display_power_profile = true;
        this.relay = false;
    }
    valid_enable_danger()
    {
        return this.vega_valid.isBool(this.enable_danger);
    }
    valid_sms()
    {
        return this.vega_valid.isBool(this.sms);
    }
    valid_email()
    {
        return this.vega_valid.isBool(this.email);
    }
    valid_telegram()
    {
        return this.vega_valid.isBool(this.telegram);
    }
    valid_voice()
    {
        return this.vega_valid.isBool(this.voice);
    }
    valid_message_sms()
    {
        return this.vega_valid.isSMS(this.message_sms);
    }
    valid_message_messenger()
    {
        return this.vega_valid.isMessenger(this.message_messenger);
    }
    valid_voice_message()
    {
        return this.vega_valid.comment(this.voice_message);
    }
    valid_telephones()
    {
        return true;
    }
    valid_emails()
    {
        return true;
    }
    valid_telegram_chats()
    {
        return true;
    }
    valid_kt()
    {
        return this.vega_valid.isNumber(this.kt);
    }
    valid_count_rate()
    {
        return this.vega_valid.isInt(this.count_rate)&&this.count_rate>=1&&this.count_rate<=4;
    }
    valid_count_phase()
    {
        return this.vega_valid.isInt(this.count_phase)&&(this.count_phase===1||this.count_phase===3);
    }
    valid_display_power_profile()
    {
        return this.vega_valid.isBool(this.display_power_profile);
    }
    valid_relay()
    {
        return this.vega_valid.isBool(this.relay);
    }
    valid_instrument()
    {
        return this.vega_valid.name(this.instrument);
    }
    valid_instrument2()
    {
        return this.vega_valid.name(this.instrument2);
    }
    valid_sensor_name_2()
    {
        return this.vega_valid.name(this.sensor_name_2);
    }
    valid_name_sensor_in_1()
    {
        return this.vega_valid.name(this.name_sensor_in_1);
    }
    valid_name_sensor_in_2()
    {
        return this.vega_valid.name(this.name_sensor_in_2);
    }
    valid_name_sensor_out_1()
    {
        return this.vega_valid.name(this.name_sensor_out_1);
    }
    valid_name_sensor_out_2()
    {
        return this.vega_valid.name(this.name_sensor_out_2);
    }
    
    
    valid_sensor_name_1()
    {
        return this.vega_valid.name(this.sensor_name_1);
    }
    valid_security_name_2()
    {
        return this.vega_valid.name(this.security_name_2);
        
    }
    valid_typeCargo()
    {
        return this.typeCargo?true:false;
    }
    valid_provider()
    {
        return this.vega_valid.name(this.provider);
    }
    valid_cargo()
    {
        return this.vega_valid.name(this.cargo);
    }
    
    valid_max_normal_v()
    {
         return this.vega_valid.isNumber(this.max_normal_v);
    }
    valid_min_normal_v()
    {
         return this.vega_valid.isNumber(this.min_normal_v);
    }
    valid_range_normal_value()
    {
        if(this.version == 1)
        {
            return this.valid_max_v()&&this.valid_min_v();
        }
        else 
        {
            if (this.valid_max_normal_v()&&this.valid_min_normal_v()&&this.valid_max_v()&&this.valid_min_v())
            {
                var min = parseFloat(this.min_v);
                var max = parseFloat(this.max_v);
                var min_n = parseFloat(this.min_normal_v);
                var max_n = parseFloat(this.max_normal_v);
                if (max_n>min_n&&max>=max_n&&min<=min_n)
                {
                    return true;
                }
            }
            return false;
        }
    }
    valid_range_value()
    {
        if (this.valid_max_v()&&this.valid_min_v())
        {
            var min = parseFloat(this.min_v);
            var max = parseFloat(this.max_v);
            if (max>min)
            {
                return true;
            }
        }
        return false;
    }
    valid_max_v()
    {
         return this.vega_valid.isNumber(this.max_v);
    }
    valid_min_v()
    {
         return this.vega_valid.isNumber(this.min_v);
    }
    valid_unit()
    {
        return this.vega_valid.name(this.unit);
    }
    valid_devEui()
    {
        return this.vega_valid.devEui(this.device.devEui);
    }
    valid_personal_num()
    {
        return this.vega_valid.name(this.personal_num);
    }
    valid_comment()
    {
        return this.vega_valid.comment(this.info_channel)
    }
    valid_type_channel()
    {
        return this.type_channel&&this.vega_valid.isInt(this.type_channel.id);
    }
    valid_division()
    {
        return this.vega_valid.isNumber(this.division);
    }
    valid_init_value()
    {
        return this.vega_valid.isNumber(this.init_value);
    }
    valid_last_date()
    {
        return this.vega_valid.isDate(this.last_date)
    }
    valid_name_level_1()
    {
        return this.vega_valid.name(this.level_1.name_level_1);
    }
    
    valid_security_name()
    {
        return this.vega_valid.name(this.security_name);
    }
    valid_hall_1_name()
    {
        return this.vega_valid.name(this.hall_1_name);
    }
    valid_hall_2_name()
    {
        return this.vega_valid.name(this.hall_2_name);
    }
    valid_water()
    {
       return this.water==1||this.water==2?true:false;
    }
    valid_name()
    {
        return this.vega_valid.name(this.name)
    }
    valid_name_level_2()
    {
        return this.vega_valid.name(this.level_2);
    }
    valid_address_level_1()
    {
        return this.vega_valid.address(this.level_1.address_level_1);
    }
    valid_num_channel()
    {
        return this.vega_valid.isInt(this.num_channel);
    }
    valid_status()
    {
        return this.vega_valid.isInt(this.status);
    }
    valid_color()
    {
        return this.vega_valid.isInt(this.color.id);
    }
    valid_ia()
    {
        return this.vega_valid.isInt(this.ia);
    }
    valid_calib_interval()
    {
        return this.vega_valid.isInt(this.calib_interval);
    }
    valid_1()
    {
        if (this.type_channel.type==1)
        {
             try
            {
                this.init_value = this.init_value.replace(/,/,'.' );
                this.division = this.division.replace(/,/,'.' );
                if (this.type_channel.id==1) 
                {
                    this.kt = this.kt.toString().replace(/,/,'.' );
                }
                else
                {
                    this.kt=1;
                }
            }
            catch(err)
            {
                return false;
            }
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_division())
            {
                return false;
            }
            if (!this.valid_init_value())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            return true;
        }
        else if (this.type_channel.type==2)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_4()
    {
        if (this.type_channel.type==3)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_security_name())
            {
                return false;
            }
            if (!this.valid_hall_1_name())
            {
                return false;
            }
            if (!this.valid_hall_2_name())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_td11rev2()
    {
        if (this.type_channel.type==3)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_security_name())
            {
                return false;
            }
            if (!this.valid_hall_1_name())
            {
                return false;
            }
            if (!this.valid_hall_2_name())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
     valid_6()
    {
        if (this.type_channel.type==6&&this.valid_smart())
        {   
            if (!this.valid_instrument())
            {
                return false;
            } 
            if( this.version == 1 )
            {
                if (!this.valid_instrument2())
                {
                    return false;
                } 
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_smart()
    {
        if (this.type_channel.type==6)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            } 
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_tl11()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }   
        if (!this.valid_provider())
        {
            return false;
        }
        if (!this.valid_cargo())
        {
            return false;
        }
        if (!this.valid_typeCargo())
        {
            return false;
        }
        if (!this.valid_personal_num())
        {
            return false;
        }
        return true;
    }

    valid_td12()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }   
        if (!this.valid_provider())
        {
            return false;
        }
        if (!this.valid_cargo())
        {
            return false;
        }
        if (!this.valid_hall_1_name())
        {
            return false;
        }
        if (!this.valid_typeCargo())
        {
            return false;
        }
        if (!this.valid_personal_num())
        {
            return false;
        }
        return true;
    }
    valid_5()
    {
        if (this.type_channel.type==5)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }    
            if (!this.valid_instrument())
            {
                return false;
            }
            if (!this.valid_sensor_name_2())
            {
                return false;
            }
            if (!this.valid_sensor_name_1())
            {
                return false;
            }
            if (!this.valid_security_name_2())
            {
                return false;
            }
            if (!this.valid_security_name())
            {
                return false;
            } 
            if (!this.valid_range_normal_value())
            {
                return false;
            }
            if (!this.valid_range_value())
            {
                return false;
            }
            if (!this.valid_unit())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    basicValid()
    {
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        
        return true;
    }
    valid_si13_merc()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        if (this.type_channel.type==9)
        {
            if (!this.valid_count_phase())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            if (!this.valid_ia())
            {
                return false;
            }
            if (!this.valid_kt()) 
            {
                return false;
            }
            return true;
        }
        else if (this.type_channel.type==1)
        {
            try
            {
                this.init_value = this.init_value.replace(/,/,'.' );
                this.division = this.division.replace(/,/,'.' );
                if (this.type_channel.id==1) 
                {
                    this.kt = this.kt.toString().replace(/,/,'.' );
                }
                else
                {
                    this.kt=1;
                }
            }
            catch(err)
            {
                return false;
            }
            
            if (!this.valid_division())
            {
                return false;
            }
            if (!this.valid_init_value())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
           
            if (!this.valid_num_channel())
            {
                return false;
            }
            
            if (!this.valid_calib_interval())
            {
                return false;
            }
            return true;
        }
        else if (this.type_channel.type==2)
        {
            return true;
        }
        else if (this.type_channel.type==22)
        {
            if (!this.valid_ia())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_ue()
    {
        if (this.type_channel.type==7)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_count_rate())
            {
                return false;
            }
            if (!this.valid_count_phase())
            {
                return false;
            }
            if (!this.valid_display_power_profile())
            {
                return false;
            }
            if (!this.valid_relay())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            if (!this.valid_ia())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_ue2()
    {
        if (this.type_channel.type==7)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_count_rate())
            {
                return false;
            }
            if (!this.valid_count_phase())
            {
                return false;
            }
            if (!this.valid_display_power_profile())
            {
                return false;
            }
            if (!this.valid_relay())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            if (!this.valid_ia())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_ug()
    {
        if (this.type_channel.type==8)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            
            if (!this.valid_name_sensor_in_1())
            {
                return false;
            }
            if (!this.valid_name_sensor_in_2())
            {
                return false;
            }
            if (!this.valid_name_sensor_out_1())
            {
                return false;
            }
            if (!this.valid_name_sensor_out_2())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_gm_1()
    {
        if (this.type_channel.type==8)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_lm1()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        if (!this.valid_security_name())
        {
            return false;
        }
        return true;
    }
    valid_src()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        return true;
    }
    valid_9()
    {
        if (this.type_channel.type==4)
        {
            if (!this.valid_devEui())
            {
                return false;
            }
            if (!this.valid_comment())
            {
                return false;
            }
            if (!this.valid_type_channel())
            {
                return false;
            }
            if (!this.valid_name_level_1())
            {
                return false;
            } 
            if (!this.valid_name())
            {
                return false;
            }
            if (!this.valid_name_level_2())
            {
                return false;
            }
            if (!this.valid_address_level_1())
            {
                return false;
            }
            if (!this.valid_num_channel())
            {
                return false;
            }
            if (!this.valid_color())
            {
                return false;
            }
            if (!this.valid_status())
            {
                return false;
            }
            if (!this.valid_water())
            {
                return false;
            }
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            return true;
        }
        else
        {
            return false;
        }
    }
    valid_hs()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        if (!this.valid_security_name_2())
        {
            return false;
        }
        if (!this.valid_security_name())
        {
            return false;
        } 
        return true;
    }
    valid_um()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        
        return true;
    }
    valid_mbus()
    {
        if (!this.valid_devEui())
        {
            return false;
        }
        if (!this.valid_comment())
        {
            return false;
        }
        if (!this.valid_type_channel())
        {
            return false;
        }
        if (!this.valid_name_level_1())
        {
            return false;
        } 
        if (!this.valid_name())
        {
            return false;
        }
        if (!this.valid_name_level_2())
        {
            return false;
        }
        if (!this.valid_address_level_1())
        {
            return false;
        }
        if (!this.valid_num_channel())
        {
            return false;
        }
        if (!this.valid_color())
        {
            return false;
        }
        if (!this.valid_status())
        {
            return false;
        }
        if ( this.type_channel.type == 20 )
        {
            if (!this.valid_last_date())
            {
                return false;
            }
            if (!this.valid_calib_interval())
            {
                return false;
            }
            if (!this.valid_ia())
            {
                return false;
            }
        }
        else if( this.type_channel.type != 2 )
        {
            return false;
        }
        return true;
    }
    valid()
    {
        var dt = parseInt(this.device.device_type);
        switch (dt) {
            case 1:
                return this.valid_1();
                break;
            case 2:
                return this.valid_1();
                break;
            case 3:
                return this.valid_si13_merc();
                break;
            case 4:
                if ( this.version == 1 )
                {
                    return this.valid_td11rev2();
                }
                else if ( this.version == 0 )
                {
                    return this.valid_4();
                }
                else
                {
                    return false;
                }
                break;
            case 5:
                return this.valid_5();
                break;  
            case 6:
                return this.valid_6();
                break;
            case 7:
                return this.valid_smart();
                break;
            case 8:
                return this.valid_smart();
                break;
            case 9:
                return this.valid_9();
                break;
            case 10:
                return this.valid_smart();
                break;
            case 11:
                return this.valid_1();
                break;
            case 12:
                return this.valid_ue();
                break;
            case 13:
                return this.valid_ug();
                break;
            case 14:
                return this.valid_lm1();
                break;
            case 15:
                if ( this.version == 1 )
                {
                    return this.valid_tl11();
                }
                else if ( this.version == 0 )
                {
                    return this.valid_td12();
                }
                else
                {
                    return false;
                }
                
                break;  
            case 17:
                return this.valid_gm_1();
                break;
            case 18:
                return this.valid_1();
                break;
            case 20:
                return this.valid_mbus();
                break;
            case 21:
                return this.valid_mbus();
                break;
            case 23:
                return this.valid_hs();
                break;
            case 24:
                return this.valid_ue2();
                break;
            case 25:
                return this.valid_um();
                break;
            case 26:
                return this.valid_src();
                break;
            default:
                return false;
                break;
        }
    }
    refresh()
    {
        try
        {
            if (!this.level_1.name_level_1.name_level_1||this.level_1.name_level_1.name_level_1===undefined)
            {
                return true;
            }
            else
            {
               if (this.level_1.name_level_1.address_level_1&&this.level_1.name_level_1.address_level_1!==undefined)
                {
                    this.level_1.address_level_1=this.level_1.name_level_1.address_level_1;
                }
                this.level_1.name_level_1=this.level_1.name_level_1.name_level_1;
            }
        }
        catch (err)
        {
            return false;
        }
        finally
        {
            
        }
    }
    init()
    {
        if (!this._init_status)
        {
            this.device = new device();
            this.level_1.address_level_1 = '';
            this.level_1.name_level_1 = '';
            this.level_2 = '';
            this.num_channel = '';
            this.status = '';
            this.color = '';
            this.info_channel = '';
            this.init_value = '';
            this.kt = 1;
            this.name = '';
            this.type_channel = '';
            this.division = '';
            this.history = '';
            this.calib_interval='';
            this.ia='';
            this.model='';
            this.serial='';
            this.personal_num='';
            this.type_meter = 0;
            this._init_status=true
        }
        return true;
    }
}
class type_device
{
    constructor(arr_type)
    {
        this.list = arr_type;
    }
    checkMultipleVersions (id)
    {
        var type = this.get_type_device ( id );
        if ( isObject(type) && isObject(type.versions) && isObject(type.versions.list) )
        {
            return Object.keys(type.versions.list).length > 1 ;
        }
        return false;
    }
    get_key_first_settings_type_device ( id )
    {
        var type = this.get_type_device ( id );
        if ( isObject(type) && isObject(type.versions) )
        {
            return type.versions.first;
        }
        return undefined;
    }
    get_key_default_settings_type_device ( id, appEui )
    {
        var type = this.get_type_device ( id );
        if ( isObject(type) && isObject(type.versions) )
        {
            var defaultVersion = type.versions.default;
            
            if( isObject(type.versions.list) && appEui !== undefined )
            {
                var countMatches = 0;
                var versionId = undefined;
                for( var key in type.versions.list )
                {
                    var version = type.versions.list[key];
                    if( isObject(version) && isObject(version.appEui) )
                    {
                        if ( version.appEui.indexOf( appEui ) > -1 )
                        {
                            countMatches++;
                            versionId = version.id;
                        }
                    }
                }
                if( countMatches === 1 ) defaultVersion = versionId;
            }
            return defaultVersion;
        }
        return undefined;
    }
    get_settings_type_device ( id, version )
    {
        if(version !== undefined && version !== '' )
        {
            version = version.toString();
        }
        var type = this.get_type_device ( id );
        if ( isObject(type) && isObject(type.versions) && isObject(type.versions.list) )
        {
            if ( version === undefined || version === '' ) version = this.get_key_first_settings_type_device ( id );
            var selectedVersion = type.versions.list[version];
            if(selectedVersion == undefined && Object.keys(type.versions.list).length == 1)
            {
                //Костыль против бага что старая версия не почистилась по какой то причине
                //Был электросчетчик, удалили, создали новое устройство с новым типом, версия старая
                version = this.get_key_first_settings_type_device ( id );  
                selectedVersion = type.versions.list[version];
            }
            var settings = isObject(selectedVersion) ? type.versions.list[version].settings : [];
            var validSettings = isObject(settings) && settings.length > 0;
            if ( validSettings ) return settings;
        }
        return [];
    }
    get_type_device ( id )
    {
        try
        {
            for (var i =0;i<this.list.length;i++)
           {
               if (this.list[i].id==id)
               {
                   return this.list[i];
               }
           }
           return false;
        }
        catch (err)
        {
            return false;
        }
    }
    get_type_device_on_appEui(appEui)
    {
        try
        {
            for (var i = 0; i < this.list.length; i++ )
            {
                var versions = this.list[i].versions;
                if( isObject(versions) && isObject(versions.list) )
                {
                    for ( var key in versions.list )
                    {
                        var version = versions.list[key];
                        if( isObject(version) && isObject(version.appEui) )
                        {
                            if ( version.appEui.indexOf( appEui ) > -1 )
                            {
                                return this.list[i];
                            }
                        }
                    }
                }
            }
            return false;
        }
        catch (err)
        {
            return false;
        }
    }
}
class type_channels
{
    constructor(arr_type)
    {
        this.list = arr_type;
    }
    get_type_channel( id )
    {
        try
        {
            for (var i =0;i<this.list.length;i++)
           {
               if (this.list[i].id==id)
               {
                   return this.list[i];
               }
           }
           return false;
        }
        catch (err)
        {
            return false;
        }
    };
}
class status_channel_list
{
    constructor(list)
    {
        this.list = list;
    }
    get_status( id )
   {
       try
       {
            for (var i =0;i<this.list.length;i++)
            {
                if (this.list[i].id==id)
                {
                    return this.list[i].name;
                }
            }
            return "-";
       }
        catch (err)
        {
            return false;
        }
   };
}
class color_list
{
    constructor(list)
    {
        this.list = list;
    }
    get_color( id )
    {
        try
        {
         for (var i =0;i<this.list.length;i++)
         {
             if (this.list[i].id==id)
             {
                 return this.list[i].hex;
             }
         }
         return "transparent";
        }
         catch (err)
         {
             return false;
         }
    };
    get_full_color( id )
    {
         try
        {
         for (var i =0;i<this.list.length;i++)
         {
             if (this.list[i].id==id)
             {
                 return this.list[i];
             }
         }
         return "transparent";
        }
         catch (err)
         {
             return false;
         }
    }
}
class Channel
{
    constructor(object)
    {
       var valid = new valid_data();
       if (!object)
       {
           object = new Object();
       }
       this.address_level_1 =object.address_level_1;
       this.name_level_1 =object.name_level_1;
       this.level_2 =object.level_2;
       this.name =object.name;
       this.num_channel =object.num_channel;
       this.status =object.status;
       this.color =object.color;
       this.info_channel =object.info_channel;
       this.init_value = object.init_value;
       this.kt =valid.isNumber(object.kt)?object.kt:1;
       this.last_date =object.last_date;
       this.date_installation = object.date_installation;
       this.date_shipment = object.date_shipment;
       this.calib_interval=object.calib_interval;
       this.ia=object.ia;
       this.model=object.model;
       this.serial=object.serial;
       this.personal_num=object.personal_num;
       this.type_meter = object.type_meter;
       this.type_channel =object.type_channel;
       this.history =object.history;
       this.temp_history = new Object();
       this.division =object.division;
       this.statistics=new Statistic();
       this.danger=false;
       this.security_name=object.security_name;
       this.hall_1_name=object.hall_1_name;
       this.hall_2_name=object.hall_2_name;
       this.min_t=object.min_t!==undefined?object.min_t:-55;
       this.max_t=object.max_t!==undefined?object.max_t:100;
       this.water=object.water;
       this.instrument=object.instrument;
       this.instrument2=object.instrument2;
       this.unit=object.unit;
       this.min_v=object.min_v;
       this.max_v=object.max_v;
       this.min_normal_v=object.min_normal_v;
       this.max_normal_v=object.max_normal_v;
       this.security_name_2=object.security_name_2;
       this.sensor_name_1=object.sensor_name_1;
       this.sensor_name_2=object.sensor_name_2;
       
       this.provider=object.provider;
       this.typeCargo=object.typeCargo;
       this.cargo=object.cargo;
       
       this.name_sensor_in_1=object.name_sensor_in_1;
       this.name_sensor_in_2=object.name_sensor_in_2;
       this.name_sensor_out_1=object.name_sensor_out_1;
       this.name_sensor_out_2=object.name_sensor_out_2;
       
       this.state_sensor_0=object.state_sensor_0;
       this.state_sensor_1=object.state_sensor_1;
       this.telephones = object.telephones;
       this.emails = object.emails;
       this.telegram_chats = object.telegram_chats;
       this.sms = object.sms;
       this.email = object.email;
       this.telegram = object.telegram;
       this.enable_danger = object.enable_danger;
       this.message_sms = object.message_sms;
       this.message_messenger = object.message_messenger;
       this.voice = object.voice;
       this.voice_message = object.voice_message;
       this.count_rate=object.count_rate;
       this.count_phase = object.count_phase;
       this.display_power_profile = object.display_power_profile;
       this.relay = object.relay;
    }
    clear_history()
    {
    }
    getInfo(dateRange,filterPackage)
    {
        var my_converter = new vega_converter();
        var _items_counter = new Object();
        var dangers = new Object();
        var result = new Object();
        var version = this._self.version;
        var si_i = this.type_channel>=1&&this.type_channel<=6&&this._self.device_type!=3;
        var si13_pulse = this.type_channel>=1&&this.type_channel<=6&&this._self.device_type==3&&version==0; 
        var si13_pulse_rev2 = this.type_channel>=1&&this.type_channel<=6&&this._self.device_type==3&&version==1; 
        var mbus_danger = this.type_channel == 6 && ( this._self.device_type == 20 );
        var si13_m = this.type_channel==16&&version==0; 
        var si13_m_rev2 = this.type_channel==16&&version==1; 
        var mbus = this.type_channel==20;
        var sve1 = this.type_channel==8; 
        var td11 = this.type_channel==7&&version==0; 
        var td11rev2 = this.type_channel==7&&version==1;
        var tp11 = this.type_channel==9&&version==0; 
        var tp11rev2 = this.type_channel==9&&version==1; 
        
        var ue = this.type_channel==14; 
        var ug = this.type_channel==15;
        var gm1 = ug && this._self.device_type == 17;
        var smart = this.type_channel==10||this.type_channel==11||this.type_channel==12||this.type_channel==13;
        var hs = this.type_channel==23;
        var um = this.type_channel==25;
        var ss = this.type_channel==13;
        var lm1 = this.type_channel==17;
        var src = this.type_channel==26;
        var tl11 = this.type_channel==18;
        
        var kt = this.kt!==undefined?this.kt:1;
        var init_value = this.init_value!==undefined?this.init_value:0;
        var division = this.division!==undefined?this.division:1;
        var min_v = parseFloat(this.min_v);
        var max_v = parseFloat(this.max_v);
        result.desc = new Object();
        result.count_all = 0;
        result.count_danger = 0;
        result.last_ts = 0;
        result.last_time = 0;
        result.meterReadings = {
            summary:
                    {
                        begin:
                        {
                            time:99999999999999,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        end:
                        {
                            time:0,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        consum:function(num)
                        {
                            return my_converter.round_size(this.end.value-this.begin.value,num);
                        }
                    },
            rate_1:
                    {
                        begin:
                        {
                            time:99999999999999,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        end:
                        {
                            time:0,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        consum:function(num)
                        {
                            return my_converter.round_size(this.end.value-this.begin.value,num);
                        }
                    },
            rate_2:
                    {
                        begin:
                        {
                            time:99999999999999,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        end:
                        {
                            time:0,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        consum:function(num)
                        {
                            return my_converter.round_size(this.end.value-this.begin.value,num);
                        }
                    },
            rate_3:
                    {
                        begin:
                        {
                            time:99999999999999,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        end:
                        {
                            time:0,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        consum:function(num)
                        {
                            return my_converter.round_size(this.end.value-this.begin.value,num);
                        }
                    },
            rate_4:
                    {
                        begin:
                        {
                            time:99999999999999,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        end:
                        {
                            time:0,
                            value:0,
                            getValue:function(num)
                            {
                                return this.value?my_converter.round_size(this.value,num):this.value;
                            }
                        },
                        consum:function(num)
                        {
                            return this.end.value-this.begin.value;
                        }
                    },
            set_value:function(cell,time,value,fcnt)
            {
                //функция приема значения
                if (time<this[cell].begin.time)
                {
                    this[cell].begin.time = time;
                    this[cell].begin.value = value;
                    this[cell].begin.fcnt = fcnt;
                }
                if (time>this[cell].end.time)
                {
                    this[cell].end.time = time;
                    this[cell].end.value = value;
                    this[cell].begin.fcnt = fcnt;
                }
            },
            converTimeToMS:function()
            {
                this.rate_1.begin.time = this.rate_1.begin.time*1000;
                this.rate_1.end.time = this.rate_1.end.time*1000;
                this.rate_2.begin.time = this.rate_2.begin.time*1000;
                this.rate_2.end.time = this.rate_2.end.time*1000;
                this.rate_3.begin.time = this.rate_3.begin.time*1000;
                this.rate_3.end.time = this.rate_3.end.time*1000;
                this.rate_4.begin.time = this.rate_4.begin.time*1000;
                this.rate_4.end.time = this.rate_4.end.time*1000;
                this.summary.begin.time = this.summary.begin.time*1000;
                this.summary.end.time = this.summary.end.time*1000;
            }
        }
        for (var i = 0; i< this.history.length;i++)
        {
            var cell_history = this.history[i];
            var type_package = cell_history.type;
            var vTs = cell_history.ts!==undefined && typeof cell_history.ts === 'number' && cell_history.ts > 0;
            var vTime = cell_history.time!==undefined && typeof cell_history.time === 'number' && cell_history.time > 0;
            var correctTimeForConsum = vTime && cell_history.time >= dateRange.getStart() && cell_history.time <= (dateRange.getEnd()+HOUR_S);
            var correctTsForConsum = vTs && cell_history.ts >= ( dateRange.getStart() * 1000 ) && cell_history.ts <= ( ( dateRange.getEnd() + HOUR_S ) * 1000 );
            if (si13_m)
            {
                var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                // var correctTime = validTime&&cell_history.ts>=(dateRange.getStart()*1000)&&cell_history.ts<=(dateRange.getEnd()*1000);
                // var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                if (validTime&&!_items_counter[cell_history.ts])
                {
                    result.count_all++;
                    if (type_package == 4)
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                            result.last_sensor=cell_history.sensor_rate_sum*kt;
                        }
                        if (correctTsForConsum&&!_items_counter[cell_history.ts])
                        {
                            _items_counter[cell_history.ts]=true;
                            result.meterReadings.set_value('summary',moment(cell_history.ts).unix(),cell_history.sensor_rate_sum*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_1',moment(cell_history.ts).unix(),cell_history.sensor_rate_1*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_2',moment(cell_history.ts).unix(),cell_history.sensor_rate_2*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_3',moment(cell_history.ts).unix(),cell_history.sensor_rate_3*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_4',moment(cell_history.ts).unix(),cell_history.sensor_rate_4*kt,cell_history.fcnt);
                        }
                    }
                }
            }
            else if (si13_m_rev2)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                // var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                // var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                if (validTime&&!_items_counter[cell_history.time])
                {
                    result.count_all++;
                    if (type_package == 4)
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.sensor_rate_sum*kt;
                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            var timeMS = cell_history.time*1000;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.sensor_rate_sum*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_1',cell_history.time,cell_history.sensor_rate_1*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_2',cell_history.time,cell_history.sensor_rate_2*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_3',cell_history.time,cell_history.sensor_rate_3*kt,cell_history.fcnt);
                            result.meterReadings.set_value('rate_4',cell_history.time,cell_history.sensor_rate_4*kt,cell_history.fcnt);
                        }
                    }
                }
            }
            if (mbus)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                // var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                // var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                if (validTime&&!_items_counter[cell_history.ts])
                {
                    if (type_package == 1)
                    {
                        result.count_all++;
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.sensor_rate_sum;
                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.sensor_rate_sum,cell_history.fcnt);
                        }
                    }
                }
            }
            else if (si13_pulse)
            {
                result.count_all++;
                if (type_package == 1)
                {
                    var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                    // var correctTime = validTime&&cell_history.ts>=dateRange.getStart()*1000&&cell_history.ts<=dateRange.getEnd()*1000;
                    if (validTime&&!_items_counter[cell_history.ts])
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                            result.last_sensor=cell_history.get_value(init_value,division,kt);

                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.ts])
                        {
                            _items_counter[cell_history.ts]=true;
                            result.meterReadings.set_value('summary',moment(cell_history.ts).unix(),cell_history.get_value(init_value,division,kt),cell_history.fcnt);
                        }
                    }
                }
                else if (type_package == 2)
                {
                    var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                    var correctTime = validTime&&cell_history.ts>=dateRange.getStart()*1000&&cell_history.ts<=dateRange.getEnd()*1000;
                    // var validSensor = cell_history.sensor!==undefined&&typeof cell_history.sensor === 'number';
                    var validNumChannel = (cell_history.num_channel+6) == this.num_channel;
                    if (correctTime&&!dangers[cell_history.ts]&&validNumChannel)
                    {
                        cell_history.primaryDanger = 'a0';
                        cell_history.dateTime = cell_history.ts;
                        dangers[cell_history.ts] = cell_history;
                        result.count_danger++;
                    }
                }
            }
            else if (si13_pulse_rev2)
            {
                result.count_all++;
                if (type_package == 1)
                {
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    // var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                    if (validTime&&!_items_counter[cell_history.time])
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.get_value(init_value,division,kt);

                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.get_value(init_value,division,kt),cell_history.fcnt);
                        }
                    }
                }
                else if (type_package == 2)
                {
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                    // var validSensor = cell_history.sensor!==undefined&&typeof cell_history.sensor === 'number';
                    var validNumChannel = (cell_history.num_channel+6) == this.num_channel;
                    if (correctTime&&!dangers[cell_history.time]&&validNumChannel)
                    {
                        cell_history.primaryDanger = 'a0';
                        cell_history.dateTime = cell_history.time*1000;
                        dangers[cell_history.time] = cell_history;
                        result.count_danger++;
                    }
                }
            }
            else if (mbus_danger)
            {
                result.count_all++;
                if (type_package == 5)
                {
                    var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                    var correctTime = validTime&&cell_history.ts>=dateRange.getStart()*1000&&cell_history.ts<=dateRange.getEnd()*1000;
                    if (correctTime&&!dangers[cell_history.ts]&&(cell_history.num_channel+10)==this.num_channel)
                    {
                        cell_history.primaryDanger = 'a0';
                        cell_history.dateTime = cell_history.ts;
                        dangers[cell_history.ts] = cell_history;
                        result.count_danger++;
                    }
                }
            }
            else if (si_i)
            {
                result.count_all++;
                if (type_package == 1)
                {
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    // var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                    if (validTime&&!_items_counter[cell_history.time])
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.get_value(init_value,division,kt);

                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.get_value(init_value,division,kt),cell_history.fcnt);
                        }
                    }
                }
                else if (type_package == 2)
                {
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                    var timeMs = cell_history.time * 1000;
                    if (correctTime&&!dangers[timeMs]&&cell_history.num_channel==this.num_channel)
                    {
                        cell_history.primaryDanger = 'a0';
                        cell_history.dateTime = timeMs;
                        dangers[timeMs] = cell_history;
                        result.count_danger++;
                    }
                }
            }
            else if (sve1)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                if (validTime&&!_items_counter[cell_history.time])
                {
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.sensorKB;
                        }
                        if(correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.sensorKB,cell_history.fcnt);
                        }
                        if (correctTime)
                        {
                            cell_history.primaryDanger = '';
                            var checkEvent = false;
                            if (cell_history.state_display)
                            {
                                cell_history.primaryDanger=cell_history.primaryDanger+'a';
                                checkEvent = true;
                            }
                            if (cell_history.leaking)
                            {
                                cell_history.primaryDanger=cell_history.primaryDanger+'b';
                                checkEvent = true;
                            }
                            if (cell_history.breakthrough)
                            {
                                cell_history.primaryDanger=cell_history.primaryDanger+'c';
                                checkEvent = true;
                            }
                            if (cell_history.hall_1)
                            {
                                cell_history.primaryDanger=cell_history.primaryDanger+'d';
                                checkEvent = true;
                            }
                            if (checkEvent)
                            {
                                result.count_danger++;
                                cell_history.dateTime = cell_history.time*1000;
                                dangers[cell_history.time*1000] = cell_history;
                            }
                        }
                    }
                }
            }
            else if (td11)
            {
                var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                var correctTime = validTime&&cell_history.ts>=(dateRange.getStart()*1000)&&cell_history.ts<=(dateRange.getEnd()*1000);
                if (validTime&&!_items_counter[cell_history.ts])
                {
                    _items_counter[cell_history.ts] = true;
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                            result.last_sensor=cell_history.temperature;
                        }
                        if ( correctTime )
                        {
                            //var checkEvent = cell_history.reason!==0;
                            //var checkTemperature = t<=t_min||t>=t_max;

                            //if ((checkEvent||checkTemperature)&&!_dangers[cell_history.ts])

                            var checkEvent = cell_history.reason!==0;
                            var t = parseFloat(cell_history.temperature);
                            var t_max = this.max_t;
                            var t_min = this.min_t;
                            var checkTemperature = t<=t_min||t>=t_max;
                            if (checkTemperature)
                            {
                                cell_history.primaryDanger = 'a';
                                cell_history.dangerTemperature = true;
                            }
                            if (checkEvent)
                            {
                                cell_history.primaryDanger = cell_history.primaryDanger?cell_history.primaryDanger+cell_history.reason:cell_history.reason;
                                cell_history.dangerEvent = true;
                            }
                            if ((checkEvent||checkTemperature)&&!dangers[cell_history.ts])
                            {
                                result.count_danger++;
                                cell_history.dateTime = cell_history.ts;
                                dangers[cell_history.ts] = cell_history;
                            }
                        }
                    }
                }
            }
            else if (td11rev2)
            {
                var validTime = cell_history.time !== undefined && cell_history.time > 0;
                var correctTime = validTime && cell_history.time >= dateRange.getStart() && cell_history.time <= dateRange.getEnd();
                if (validTime&&!_items_counter[cell_history.ts]&&correctTime)
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all ++;
                    if ( type_package == 1 )
                    {
                        if ( cell_history.time > result.last_time )
                        {
                            result.last_time = cell_history.time;
                            result.last_sensor = cell_history.temperature;
                        }
                        var checkEvent = cell_history.reason !== 0;
                        var t = parseFloat( cell_history.temperature );
                        if ( cell_history.limit_exceeded )
                        {
                            cell_history.primaryDanger = 'a';
                            cell_history.dangerTemperature = true;
                        }
                        if ( checkEvent )
                        {
                            cell_history.primaryDanger = cell_history.primaryDanger ? cell_history.primaryDanger + cell_history.reason : cell_history.reason;
                            cell_history.dangerEvent = true;
                        }
                        if ( ( checkEvent || cell_history.limit_exceeded ) && !dangers[cell_history.time] )
                        {
                            result.count_danger++;
                            cell_history.dateTime = cell_history.time * 1000;
                            dangers[cell_history.time] = cell_history;
                        }
                    }
                }
            }
            else if (tl11)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                if (validTime&&!_items_counter[cell_history.ts])
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all++;
                    var checkEvent = cell_history.reason!=='00';
                    var t = parseFloat(cell_history.temperature);
                    var t2 = parseFloat(cell_history.temperature_2);
                    var t_max = this.max_t;
                    var t_min = this.min_t;
                    var checkTemperature = t<=t_min||t>=t_max;
                    var checkTemperature2 = t2<=t_min||t2>=t_max;
                    
                    if (cell_history.time>result.last_time)
                    {
                        result.last_lime=cell_history.time;
                        result.last_sensor=cell_history.temperature;
                        result.last_sensor_2=cell_history.temperature2;
                    }
                    if (checkTemperature||checkTemperature2)
                    {
                        cell_history.primaryDanger = 'a';
                        cell_history.dangerTemperature = true;
                    }
                    if (checkEvent)
                    {
                        cell_history.primaryDanger = cell_history.primaryDanger?cell_history.primaryDanger+cell_history.reason:cell_history.reason;
                        cell_history.dangerEvent = true;
                    }
                    if ((checkEvent||checkTemperature)&&!dangers[cell_history.time])
                    {
                        result.count_danger++;
                        cell_history.dateTime = cell_history.time*1000;
                        dangers[cell_history.time] = cell_history;
                    }
                    result.desc.hall_1_name=this.hall_1_name;
                }
            }
            else if (tp11)
            {
                var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                var correctTime = validTime&&cell_history.ts>=(dateRange.getStart()*1000)&&cell_history.ts<=(dateRange.getEnd()*1000);
                var sensor = my_converter.parse4_20mA(cell_history.sensorTP,min_v,max_v);
                if (validTime&&!_items_counter[cell_history.ts]&&correctTime)
                {
                    _items_counter[cell_history.ts]=true;
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                            if (cell_history.sensorTP==0)
                            {
                                result.last_sensor=undefined;
                            }
                            else
                            {
                                result.last_sensor=sensor;
                            }
                        }
                    }
                    cell_history.primaryDanger = '';
                    if (cell_history.sensor_danger_1)
                    {
                        cell_history.primaryDanger=cell_history.primaryDanger+'a';
                    }
                    else if (cell_history.sensor_danger_2)
                    {
                        cell_history.primaryDanger=cell_history.primaryDanger+'b';
                    }
                    else
                    {
                        cell_history.dangerEvent = false;
                    }
                    cell_history.dangerEvent = cell_history.sensor_danger_1||cell_history.sensor_danger_2?true:false;
                    cell_history.security_name = this.security_name;
                    cell_history.security_name_2 = this.security_name_2;
                    var min = parseFloat(cell_history.min_normal_v);
                    var max = parseFloat(cell_history.max_normal_v);
                    var min_v = parseFloat(cell_history.min_v);
                    var max_v = parseFloat(cell_history.max_v);
                    if (cell_history.sensorTP===0)
                    {
                        cell_history.dangerData = true;
                        cell_history.primaryDanger=cell_history.primaryDanger+1;
                    }
                    else if (cell_history.sensorTP<=25&&cell_history.sensorTP>=2)
                    {
                        if (sensor<=min||sensor>=max)
                        {
                            cell_history.dangerData = true;
                            cell_history.primaryDanger=cell_history.primaryDanger+1;
                        }
                    }
                    else
                    {
                       cell_history.primaryDanger=cell_history.primaryDanger+1;
                       cell_history.dangerData = true;
                    }
                    if (cell_history.dangerData||cell_history.dangerEvent)
                    {
                        if (cell_history.dangerData&&cell_history.dangerEvent)
                        {
                            cell_history.primaryDanger='0'+cell_history.primaryDanger;
                        }
                        result.count_danger++;
                        cell_history.dateTime = cell_history.ts;
                        dangers[cell_history.ts] = cell_history;
                    }
                    //есть еще и второй
                }
            }
            else if ( tp11rev2 )
            {
                var validTime = cell_history.time !== undefined && cell_history.time > 0;
                var correctTime = validTime && cell_history.time >= dateRange.getStart() && cell_history.time <= dateRange.getEnd();
                var sensor = my_converter.parse4_20mA( cell_history.sensorTP, min_v, max_v );
                cell_history.primaryDanger = '';
                cell_history.dangerEvent = false;
                cell_history.dangerData = false;
                
                if (validTime&&!_items_counter[cell_history.time]&&correctTime)
                {
                    if ( type_package == 1 )
                    {
                        result.count_all ++;
                        _items_counter[cell_history.time] = true;
                        cell_history.dateTime = cell_history.time * 1000;
                        cell_history.security_name = this.security_name;
                        cell_history.security_name_2 = this.security_name_2;
                        if ( cell_history.time > result.last_time )
                        {
                            result.last_ts = cell_history.ts;
                            result.last_time = cell_history.time;
                            if ( cell_history.sensorTP == 0 )
                            {
                                result.last_sensor = undefined;
                            }
                            else
                            {
                                result.last_sensor = sensor;
                            }
                        }
                        
                        if( cell_history.reason != 0 && cell_history.reason != 5)
                        {
                            cell_history.primaryDanger = cell_history.reason;
                            cell_history.dangerEvent = true;
                            dangers[cell_history.time] = cell_history;
                            result.count_danger++;
                        }
                    }
                }
            }
            else if (ue)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                if (validTime&&!_items_counter[cell_history.time]&&(filterPackage===undefined||filterPackage==type_package))
                {
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.sensor_rate_sum;
                            result.last_kt=cell_history.kt;
                        }
                        if ( correctTimeForConsum && !_items_counter[cell_history.time] )
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.sensor_rate_sum,cell_history.fcnt);
                        }
                        if (correctTime)
                        {
                            cell_history.primaryDanger = '';
                            if (cell_history.event!==undefined&&cell_history.event!=1&&cell_history.event!=19)
                            {
                                result.count_danger++;
                                cell_history.primaryDanger = cell_history.event;
                                cell_history.dateTime = cell_history.time*1000;
                                dangers[cell_history.time*1000] = cell_history;
                            }
                        }
                    } 
                    else if (type_package == 4)
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.sensor_rate_sum;
                            result.last_kt=cell_history.kt;
                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.sensor_rate_sum,cell_history.fcnt);
                            result.meterReadings.set_value('rate_1',cell_history.time,cell_history.sensor_rate_1,cell_history.fcnt);
                            result.meterReadings.set_value('rate_2',cell_history.time,cell_history.sensor_rate_2,cell_history.fcnt);
                            result.meterReadings.set_value('rate_3',cell_history.time,cell_history.sensor_rate_3,cell_history.fcnt);
                            result.meterReadings.set_value('rate_4',cell_history.time,cell_history.sensor_rate_4,cell_history.fcnt);
                        }
                    }
                }
            }
            else if (ug)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=dateRange.getStart()&&cell_history.time<=dateRange.getEnd();
                if (validTime&&!_items_counter[cell_history.time])
                {
                    result.count_all++;
                    if (cell_history.port == 2)
                    {
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                            result.last_sensor=cell_history.sensor_rate_sum;
                        }
                        if (correctTimeForConsum&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            result.meterReadings.set_value('summary',cell_history.time,cell_history.sensor_rate_sum,cell_history.fcnt);
                        }
                        if (correctTime)
                        {
                            if(gm1)
                            {
                                if(cell_history.reason!=1)
                                {
                                    cell_history.primaryDanger = cell_history.reason;
                                    result.count_danger++;
                                    cell_history.dateTime = cell_history.time*1000;
                                    dangers[cell_history.ts] = cell_history;
                                }
                            }
                            else
                            {
                                if (cell_history.reason)
                                {
                                    cell_history.primaryDanger = cell_history.reason;
                                    result.count_danger++;
                                    cell_history.dateTime = cell_history.time*1000;
                                    dangers[cell_history.ts] = cell_history;
                                }
                            }
                        }
                    }
                }
                result.desc.name_sensor_in_1=this.name_sensor_in_1;
                result.desc.name_sensor_in_2=this.name_sensor_in_2;
                result.desc.name_sensor_out_1=this.name_sensor_out_1;
                result.desc.name_sensor_out_2=this.name_sensor_out_2;
            }
            else if (smart&&cell_history.version == 0)
            {
                var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                var correctTime = validTime&&cell_history.ts>=(dateRange.getStart()*1000)&&cell_history.ts<=(dateRange.getEnd()*1000);
                if (validTime&&!_items_counter[cell_history.ts]&&correctTime)
                {
                    _items_counter[cell_history.ts] = true;
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                        }
                        if (cell_history.reason)
                        {
                            cell_history.primaryDanger = cell_history.reason;
                            result.count_danger++;
                            cell_history.dateTime = cell_history.ts;
                            dangers[cell_history.ts] = cell_history;
                        }
                    }
                }
            }
            else if (smart&&cell_history.version >= 1&&!ss)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=(dateRange.getStart())&&cell_history.time<=(dateRange.getEnd());
                if (validTime&&!_items_counter[cell_history.time]&&correctTime)
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                        }
                        if (cell_history.time>result.last_time)
                        {
                            result.last_time=cell_history.time;
                        }
                        if (cell_history.reason)
                        {
                            cell_history.primaryDanger = cell_history.reason;
                            result.count_danger++;
                            cell_history.dateTime = cell_history.time * 1000;
                            dangers[cell_history.time * 1000] = cell_history;
                        }
                    }
                }
            }
            else if ( hs )
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=(dateRange.getStart())&&cell_history.time<=(dateRange.getEnd());
                if (validTime&&!_items_counter[cell_history.time]&&correctTime)
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all++;
                    if (cell_history.ts>result.last_ts)
                    {
                        result.last_ts=cell_history.ts;
                    }
                    if (cell_history.time>result.last_time)
                    {
                        result.last_time = cell_history.time;
                        result.last_damp = cell_history.damp;
                        result.last_angle = cell_history.angle;
                        result.last_temperature = cell_history.temperature;
                    }
                    if ( cell_history.reason > 1 && correctTime )
                    {
                        cell_history.primaryDanger = cell_history.reason;
                        result.count_danger++;
                        cell_history.dateTime = cell_history.time * 1000;
                        dangers[cell_history.time * 1000] = cell_history;
                    }
                }
            }
            else if ( um )
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=(dateRange.getStart())&&cell_history.time<=(dateRange.getEnd());
                if (validTime&&!_items_counter[cell_history.time]&&correctTime)
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all++;
                    if (cell_history.ts>result.last_ts)
                    {
                        result.last_ts=cell_history.ts;
                    }
                    if (cell_history.time>result.last_time)
                    {
                        result.last_time = cell_history.time;
                        result.last_damp = cell_history.damp;
                        result.last_angle = cell_history.angle;

                        result.last_lux = cell_history.lux;
                        result.last_dB = cell_history.dB;
                        result.last_CO2 = cell_history.CO2;

                        result.last_temperature = cell_history.temperature;
                    }
                    if ( cell_history.reason > 1 && correctTime )
                    {
                        cell_history.primaryDanger = cell_history.reason;
                        result.count_danger++;
                        cell_history.dateTime = cell_history.time * 1000;
                        dangers[cell_history.time * 1000] = cell_history;
                    }
                }
            }
            else if (lm1)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=(dateRange.getStart())&&cell_history.time<=(dateRange.getEnd());
                if (validTime&&!_items_counter[cell_history.time])
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all++;
                    if (type_package == 1)
                    {
                        if (cell_history.ts>result.last_ts)
                        {
                            result.last_ts=cell_history.ts;
                        }
                        if (cell_history.alarm)
                        {
                            cell_history.security_name = this.security_name;
                            cell_history.primaryDanger = cell_history.alarm;
                            result.count_danger++;
                            cell_history.dateTime = cell_history.time*1000;
                            dangers[cell_history.time] = cell_history;
                        }
                    }
                }
            }
            else if (src)
            {
                var validTime = cell_history.time!==undefined&&cell_history.time>0;
                var correctTime = validTime&&cell_history.time>=(dateRange.getStart())&&cell_history.time<=(dateRange.getEnd());
                if (validTime&&!_items_counter[cell_history.time]&&correctTime)
                {
                    _items_counter[cell_history.time] = true;
                    result.count_all++;
                    if (cell_history.ts>result.last_ts)
                    {
                        result.last_ts=cell_history.ts;
                    }
                    if (cell_history.time>result.last_time)
                    {
                        result.last_reason = cell_history.reason;
                        result.last_time = cell_history.time;
                        result.last_status_state = cell_history.status_state;
                        result.last_angle = cell_history.angle;
                        result.last_led_duty = cell_history.led_duty;
                        result.last_state_tamper = cell_history.state_tamper;
                        result.last_charge = cell_history.charge;
                        result.last_nomenal_voltage = cell_history.nomenal_voltage;

                        result.last_total_power_summary = cell_history.total_power_summary;
                        result.last_lat = cell_history.lat;
                        result.last_lng = cell_history.lng;

                        result.last_temperature = cell_history.temperature;
                    }
                    if ( cell_history.reason > 1 && correctTime )
                    {
                        cell_history.primaryDanger = cell_history.reason;
                        result.count_danger++;
                        cell_history.dateTime = cell_history.time * 1000;
                        dangers[cell_history.time * 1000] = cell_history;
                    }
                }
            }
        }
        if (si_i||sve1||ue||ug||td11rev2||mbus||si13_m_rev2||si13_pulse_rev2||si13_pulse||si13_m)
        {
            result.meterReadings.converTimeToMS();
        }
        if (si_i||sve1||ue||ug||td11rev2||mbus||si13_m_rev2||si13_pulse_rev2||hs||um)
        {
            result.last_datetime = result.last_time*1000;
        }
        else
        {
            result.last_datetime = result.last_ts;
        }
        if (ue&&result.last_kt!=65535)
        {
            result.kt = result.last_kt;
        }
        else
        {
            result.kt = kt;
        }
        if(mbus)
        {
            this.serial = this.ia;
        }
        result.security_name = this.security_name;
        result.security_name_2 = this.security_name_2;
        result.dangers = dangers;
        result.water = this.water;
        result.unit = this.unit;
        result.status = this.status;
        result.last_sensor = result.last_sensor?my_converter.round_size(result.last_sensor,4):undefined;
        result.consum_sum = result.meterReadings.summary.consum(4);
        result.consum_1 = result.meterReadings.rate_1.consum(4);
        result.consum_2 = result.meterReadings.rate_2.consum(4);
        result.consum_3 = result.meterReadings.rate_3.consum(4);
        result.consum_4 = result.meterReadings.rate_4.consum(4);
        result.beginReading_sum = result.meterReadings.summary.begin.getValue(4);
        result.endReading_sum = result.meterReadings.summary.end.getValue(4);
        result.beginDate_sum = result.meterReadings.summary.begin.time;
        result.endDate_sum = result.meterReadings.summary.end.time;
        result.model = this.model;
        result.serial = this.serial;
        result.name = this.name;
        result.level_2 = this.level_2;
        result.personal_num = this.personal_num;
        result.type_meter = this.type_meter;
        result.unit = this.unit;
        return result;
    }
    edit(object,dev)
    {
       var valid = new valid_data();
       this.address_level_1 =object.address_level_1;
       this.name_level_1 =object.name_level_1;
       this.level_2 =object.level_2;
       this.name =object.name;
       this.status =object.status;
       this.color =object.color;
       this.info_channel =object.info_channel;
       this.init_value =object.init_value;
       this.kt =valid.isNumber(object.kt)?object.kt:1;
       this.last_date =object.last_date;
       this.date_installation = object.date_installation;
       this.date_shipment = object.date_shipment;
       this.calib_interval=object.calib_interval;
       this.ia=object.ia;
       this.model=object.model;
       this.serial=object.serial;
       this.personal_num=object.personal_num;
       this.type_meter = valid.isNumber(object.type_meter)?object.type_meter:0;
       this.type_channel =object.type_channel;
       this.division =object.division;
       this.security_name=object.security_name;
       this.hall_1_name=object.hall_1_name;
       this.hall_2_name=object.hall_2_name;
       this.min_t=object.min_t!==undefined?object.min_t:-55;
       this.max_t=object.max_t!==undefined?object.max_t:100;
       this.water=object.water;  
       this.instrument=object.instrument;
       this.instrument2=object.instrument2;
       this.unit=object.unit;
       this.min_v=object.min_v;
       this.max_v=object.max_v;
       this.min_normal_v=object.min_normal_v;
       this.max_normal_v=object.max_normal_v;
       this.security_name_2=object.security_name_2;
       this.sensor_name_1=object.sensor_name_1;
       this.sensor_name_2=object.sensor_name_2;
       
       this.provider=object.provider;
       this.typeCargo=object.typeCargo;
       this.cargo=object.cargo;
       
       this.name_sensor_in_1=object.name_sensor_in_1;
       this.name_sensor_in_2=object.name_sensor_in_2;
       this.name_sensor_out_1=object.name_sensor_out_1;
       this.name_sensor_out_2=object.name_sensor_out_2;
       
       this.state_sensor_0=object.state_sensor_0;
       this.state_sensor_1=object.state_sensor_1;
       this.telephones = object.telephones;
       this.emails = object.emails;
       this.telegram_chats = object.telegram_chats;
       this.sms = object.sms;
       this.email = object.email;
       this.telegram = object.telegram;
       this.enable_danger = object.enable_danger;
       this.message_sms = object.message_sms;
       this.message_messenger = object.message_messenger;
       this.voice = object.voice;
       this.voice_message = object.voice_message;
       this.count_rate=object.count_rate;
       this.count_phase = object.count_phase;
       this.display_power_profile = object.display_power_profile;
       this.relay = object.relay;
       this.version = object.version;
       if(isObject(dev)) this._self = dev;
       this._self.version = object.version;
    }
}
class date_range
{
    //требуется moment js
    constructor()
    {
        this.start;
        this.end;
    }
    setStart(val)
    {
        this.start = val;
    }
    setEnd(val)
    {
        this.end = val;
    }
    getStart()
    {
        return this.start;
    }
    getEnd()
    {
        return this.end;
    }
    getStartUTC()
    {
        return this.start + moment().utcOffset()*60;
    }
    getEndUTC()
    {
        return this.end + moment().utcOffset()*60;
    }
    compare_unix(date)
    {  
        return (date>=this.start&&date<=this.end)?true:false;  
    }
    compare(date,type)
    {
        if(type==2) return (date>=(this.getStartUTC()*1000)&&date<=((this.getEndUTC()+HOUR_S)*1000))?true:false; 
        return (date>=(this.start*1000)&&date<=(this.end*1000))?true:false;  
    }
    
    compare_unixUTC(date)
    {
        return (date>=this.getStartUTC()&&date<=this.getEndUTC())?true:false;  
    }
    compareUTC(date)
    {
        return (date>=(this.getStartUTC()*1000)&&date<=(this.getEndUTC()*1000))?true:false;  
    }
    
    compare_full(history,date_range)
    {
        return (history.time>=date_range.start&&history.time<=date_range.end)?true:false;  
    }
}
class Statistic
{
    constructor()
    {
       this.sum_sensor='-';
       this.chart  = new Array();
       this.chart_temperature  = new Array();
       this.chart_temperature_new = new Array();
       this.chart_sensor_new = new Array();

       this.chart_damp_new = new Array();
       this.chart_lux_new = new Array();
       this.chart_dB_new = new Array();
       this.chart_CO2_new = new Array();

       this.rssi  = new Array();
       this.snr  = new Array();
       this.date  = new Array();  
       this.consumed = undefined;
       this.avr_snr=0;
       this.avr_rssi=0;
       this.fcnt=0;
       this.last_time =0;
       this.last_chart =0;
       this.last_charge = 0;
       this.last_fcnt =0;
       this.last_sensor =0;
       this.start_sensor=0;
       this.end_sensor=0;
       this.arr_date  = new Array();
       this.arr_rssi = [];
       this.connect = [];
       this.arr_snr = [];
       this.type_channel =0;
       this.arr_value  = new Array();
       this.arr_value_4  = new Array();
       this.count_danger=0;
       this.count_danger_full=0; 
       this.last_sensorKB;
       this.last_state_display;
       this.last_leaking;
       this.last_breakthrough;
    }
}
class Logs
{
    constructor()
    {
        this.list = new Array();
        this.full_list = new Array();
    }
    set_log(event,device,mess)
    {
        var newlog={
            mess:mess,
            event:event,
            device:device,
            time:new Date().getTime(),
            time_ru:moment().format('LTS' ).replace(',',' ' )
        };
        if (this.list.length>=10)
        {
            this.list.splice(0,1);
        }
        this.list.push(newlog);
        this.full_list.push(newlog);
    }
    
}
class Devices 
{
    constructor()
    {
        this.devices_list = new Array();
        this.full_devices_list = new Array();
        this.select_channel_temp=new Object();
        this.select_device_devEui='';//Выбранное устройство
        this.select_device_channel='';//Выбранный канал
        this.all_channel_list = new Array();
        this.group_list=new Object();
    }
    selectGroup(nameGroup)
    {
        for(var i = 0;i<this.group_list.length;i++)
        {
            var group = this.group_list[i];
            if (group.title == nameGroup)
            {
                group.toggle = true;
            }
            else
            {
                group.toggle = false;
            }
        }
    }
    countSelectedGroup()
    {
        var count = 0;
        for(var i = 0;i<this.group_list.length;i++)
        {
            var group = this.group_list[i];
            if (group.toggle)count++;
        }
        return count;
    }
    unselectAllGroup()
    {
        for(var i = 0;i<this.group_list.length;i++)
        {
            var group = this.group_list[i];
            group.toggle = false;
        }
    }
    groupingDevice()
    {
        var oldList = this.group_list;
        var oldGrups = new Object();
        for (var i = 0; i<oldList.length;i++)
        {
            var group = oldList[i].title;
            oldGrups['group_'+group] = oldList[i];
        }
        var groups = new Object();
        for(var i = 0; i<this.full_devices_list.length;i++)
        {
            var dev = this.full_devices_list[i];
            if ( isObject(dev) )
            {
                var group = dev.group!==undefined&&dev.group!==''?dev.group:'Other';
                var id = 0;
                if (!groups['group_'+group])
                {
                    groups['group_'+group] = {
                        devices:[],
                        id:id,
                        toggle:oldGrups['group_'+group]!==undefined?oldGrups['group_'+group].toggle:false,
                        title:group 
                    };
                    id++;
                }
                groups['group_'+group].devices.push(dev);
            }
        }
        var arr = [];
        for(var key in groups)
        {
            if (key=='group_Other'&&arr[0])
            {
                var first=arr[0];
                arr[0]=groups[key];
                arr.push(first);
            }
            else
            {
                arr.push(groups[key]);
            }
        }
        if (arr.length>0&& isObject(arr[0]) &&arr[0].title == 'Other' )
        {
            var temp = arr[0];
            arr.splice(0,1);
            arr.push(temp);
        }
        this.group_list = arr;
    }
    getPresenceTypeChannel(type)
    {
        for(var i = 0; i<this.devices_list.length;i++)
        {
            var dev = this.devices_list[i];
            if ( dev && isObject(dev.channels) )
            {
                for(var key in dev.channels)
                {
                    var channel = dev.channels[key];
                    if (channel.type_channel == type)
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    }
    
    getPointsMapsSelectedDevice(mode)
    {
        var temp = new Object();
        var channel = this.select_channel_temp;
        if (channel.devEui&&channel.name&&channel.num_channel!==undefined)
        {
            var statistics = channel.statistics;
            if (mode=='last' )
            {
                if (statistics.last_coordinate.time)
                {
                    var time = statistics.last_coordinate.time;
                    var lat = statistics.last_coordinate.lat;
                    var lng = statistics.last_coordinate.lng;
                    temp[time] = {
                        lat:lat,
                        lng:lng,
                        message:moment.unix(time).format('LLL' )
                    }
                }
            }
            else if (mode=='all' )
            {
                if (statistics.arr_valid_coordinates)
                {
                    for(var key in statistics.arr_valid_coordinates)
                    {
                        var location = statistics.arr_valid_coordinates[key];
                        if (!location) continue;
                        var time = location.time;
                        var lat = location.lat;
                        var lng = location.lng;
                        temp[time] = {
                            lat:lat,
                            lng:lng,
                            message:moment.unix(time).format('LLL' )
                        }
                    }
                }
                
            }
            else if (mode=='track' )
            {
                var latlngs = [];
                var latlngs_arr = [];
                var paths = new Object();
                if (statistics.arr_valid_coordinates)
                {
                    for(var key in statistics.arr_valid_coordinates)
                    {
                        var location = statistics.arr_valid_coordinates[key];
                        if (!location) continue;
                        var time = location.time;
                        var lat = location.lat;
                        var lng = location.lng;
                        if (lat&&lng)
                        {
                            latlngs_arr.push( [lat,lng]);
                            latlngs.push({lat:lat,lng: lng});
                            paths[key] =
                            {
                                weight: 2,
                                color: '#0bbce2',
                                latlngs: {
                                        lat:lat,
                                        lng:lng
                                },
                                radius: 3,
                                type: 'circle',
                                opacity:1,
                                stroke:false,
                                fillOpacity:1,
                            }
                            paths[key+'_'] =
                            {
                                weight: 10,
                                color: '#0bbce2',
                                latlngs: {
                                        lat:lat,
                                        lng:lng
                                },
                                radius: 10,
                                type: 'circle',
                                message:moment.unix(time).format('LLL' ),
                                stroke:false,
                                fillOpacity:0
                            }
                        }
                    }
                    var decorations = new Object();
                    decorations[0] = 
                    {
                        coordinates:latlngs_arr,
                        patterns: [
                            {
                                offset: '10%',
                                repeat: 150,
                                symbol: L.Symbol.arrowHead({
                                    pixelSize: 5,
                                    polygon: false, 
                                    pathOptions: {
                                        stroke: true,
                                        className:'cursorOne',
                                        color:'#0bbce2'
                                    }
                                })
                            }
                        ]
                    }
                    paths[0] = {
                        type: "polyline",
                        weight:3,
                        color:'#0bbce2',
                        latlngs:latlngs,
                        stroke: true,
                        lineCap:'round',
                        lineJoin:'round',
                        zIndexOffset:0,
                        bubblingMouseEvents:false,
                        interactive:false,
                        className:'cursorOne',
                        fillOpacity:0.2,
                        opacity:0.8
                    };
                    temp = 
                    {
                        decorations:decorations,
                        paths:paths,
                        coordinates:latlngs
                    }
                }
                
            }
            //расписать другие режимы 
        }
        return temp;
    }
    delete_device(devEui)
    {
        for (var i = 0; i<this.devices_list.length;i++)
        {
            if (this.devices_list[i].devEui==devEui)
            {
                 this.devices_list.splice(i,1);
            }
        }
        return false;
    }
    reload_all_channel_list()
    {
        var temp = new Array();
        for (var i =0; i<this.devices_list.length;i++)
        {
            for (var j=0;j<this.devices_list[i].channels.length;j++)
            {
                if (this.devices_list[i].channels[j].name)
                {
                    var tempObj = new Object();
                    tempObj.devEui = this.devices_list[i].devEui;
                    tempObj.channel=this.devices_list[i].channels[j];
                    temp.push(tempObj);
                }
            }
        }
        this.all_channel_list=temp;
    }
    get_editDevice(device,num,color,dev_type)
    {
        var valid = new valid_data();
        var editDevice = new edit_device();
        var channel = this.get_channel(device.devEui,num);
        editDevice.name=channel.name;
        editDevice.device = device;
        editDevice.version = device.version;
        editDevice.level_1.address_level_1 = channel.address_level_1;
        editDevice.level_1.name_level_1 = channel.name_level_1;
        editDevice.level_2 = channel.level_2;
        editDevice.name = channel.name;
        editDevice.num_channel = channel.num_channel;
        editDevice.status = channel.status;
        editDevice.color = color;
        editDevice.info_channel = channel.info_channel;
        editDevice.init_value = channel.init_value;
        editDevice.kt = valid.isNumber(channel.kt)?channel.kt:1;
        editDevice.calib_interval = channel.calib_interval;
        editDevice.ia = channel.ia;
        editDevice.model = channel.model;
        editDevice.serial = channel.serial;
        editDevice.personal_num = channel.personal_num;
        editDevice.type_meter = valid.isNumber(channel.type_meter)?channel.type_meter:0;
        editDevice.division = channel.division;
        editDevice.type_channel = dev_type;
        editDevice.last_date = new Date(moment(channel.last_date).format('YYYY-MM-DD' ));
        editDevice.date_installation = new Date(moment(channel.date_installation).format('YYYY-MM-DD' ));
        editDevice.date_shipment = new Date(moment(channel.date_shipment).format('YYYY-MM-DD' ));
        editDevice.history = channel.history;
        editDevice.security_name = channel.security_name;
        editDevice.hall_1_name = channel.hall_1_name;
        editDevice.hall_2_name = channel.hall_2_name;
        editDevice.min_t = channel.min_t;
        editDevice.max_t = channel.max_t;
        editDevice.min_t=channel.min_t!==undefined?channel.min_t:-55;
        editDevice.max_t=channel.max_t!==undefined?channel.max_t:100;
        editDevice.water=channel.water;
        editDevice.instrument = channel.instrument;
        editDevice.instrument2 = channel.instrument2;
        editDevice.unit = channel.unit;
        editDevice.min_v = channel.min_v;
        editDevice.max_v = channel.max_v;
        editDevice.min_normal_v = channel.min_normal_v;
        editDevice.max_normal_v = channel.max_normal_v;
        editDevice.security_name_2 = channel.security_name_2;
        editDevice.sensor_name_1 = channel.sensor_name_1;
        editDevice.sensor_name_2 = channel.sensor_name_2;
        
        editDevice.provider=channel.provider;
        editDevice.typeCargo=channel.typeCargo;
        editDevice.cargo=channel.cargo;
        
        editDevice.name_sensor_in_1=channel.name_sensor_in_1;
        editDevice.name_sensor_in_2=channel.name_sensor_in_2;
        editDevice.name_sensor_out_1=channel.name_sensor_out_1;
        editDevice.name_sensor_out_2=channel.name_sensor_out_2;
        
        editDevice.state_sensor_0 = channel.state_sensor_0;
        editDevice.state_sensor_1 = channel.state_sensor_1;
        editDevice.telephones = channel.telephones;
        editDevice.emails = channel.emails;
        editDevice.telegram_chats = channel.telegram_chats;
        editDevice.sms = channel.sms;
        editDevice.email = channel.email;
        editDevice.telegram = channel.telegram;
        editDevice.enable_danger = channel.enable_danger;
        editDevice.message_sms = channel.message_sms;
        editDevice.message_messenger = channel.message_messenger;
        editDevice.voice = channel.voice;
        editDevice.voice_message = channel.voice_message;
        editDevice.count_rate=channel.count_rate;
        editDevice.count_phase = channel.count_phase;
        editDevice.display_power_profile = channel.display_power_profile;
        editDevice.relay = channel.relay;
        return editDevice;
    }

    add_history_multi(json,index,date,newData)
    {
        var dev = this.devices_list[index];
        var my_converter = new vega_converter();
        if (dev)
        {
            var channel1 = dev.channels[6]?dev.channels[6]:new Channel();
            var channel2 = dev.channels[7]?dev.channels[7]:new Channel();
            var channel3 = dev.channels[10]?dev.channels[10]:new Channel();
            var channel4 = dev.channels[11]?dev.channels[11]:new Channel();
            var ts = json.ts;
            var date_installation1 = channel1.date_installation?parseInt(moment(channel1.date_installation).startOf('day' ).format('x' )):0;
            var date_installation2 = channel2.date_installation?parseInt(moment(channel2.date_installation).startOf('day' ).format('x' )):0;
            var validDateInit1 = !date_installation1||ts>date_installation1;
            var validDateInit2 = !date_installation2||ts>date_installation2;
            if (dev.device_type==3)
            {
                if (newData.type_package==4)
                {
                    var res  = new cell_history(json,newData,1,dev.device_type,dev.version);
                    for (var key in this.devices_list[index].channels)
                    {
                        var ia;
                        var channel = this.devices_list[index].channels[key];
                        if (res.address!==undefined)
                        {
                            ia = parseInt(res.address,16);
                            if (!isNaN(ia)&&ia==channel.ia)
                            {
                                channel.history.push(res);
                            }
                        }
                    }
                }
                else if (newData.type_package==6)
                {
                    var res  = new cell_history(json,newData,1,dev.device_type,dev.version);
                    for (var key in this.devices_list[index].channels)
                    {
                        var ia;
                        var channel = this.devices_list[index].channels[key];
                        if (res.address!==undefined)
                        {
                            ia = parseInt(res.address,16);
                            if (!isNaN(ia)&&ia==channel.ia)
                            {
                                channel.history.push(res);
                            }
                        }
                    }
                }
                else if (newData.type_package==1)
                {
                    var res1  = new cell_history(json,newData,7,dev.device_type,dev.version);
                    var res2  = new cell_history(json,newData,8,dev.device_type,dev.version);
                    if (validDateInit1)
                    {
                        if (!channel1.temp_history) channel1.temp_history = new Object();
                        if (channel1.temp_history[res1.ts]===undefined||!channel1.temp_history[res1.ts])
                        {
                           channel1.temp_history[res1.ts]=true;
                           channel1.history.push(res1);
                        }
                        else
                        {
                            if (channel1.type_channel==6)
                            {
                                channel1.history.push(res1);
                            }
                        }
                    }
                    if (validDateInit2)
                    {
                        if (!channel2.temp_history) channel2.temp_history = new Object();
                        if (channel2.temp_history[res2.ts]===undefined||!channel2.temp_history[res2.ts])
                        {
                           channel2.temp_history[res2.ts]=true;
                           channel2.history.push(res2);
                        }
                        else
                        {
                            if (channel2.type_channel==6)
                            {
                                channel2.history.push(res2);
                            }
                        }
                    }                 
                }
                else if (newData.type_package==2)
                {
                    var res1  = new cell_history(json,newData,7,dev.device_type,dev.version);
                    var res2  = new cell_history(json,newData,8,dev.device_type,dev.version);
                    if (channel1.temp_history[res1.ts]===undefined||!channel1.temp_history[res1.ts]&&validDateInit1)
                    {
                       channel1.temp_history[res1.ts]=true;
                       channel1.history.push(res1);
                    }
                    else
                    {
                        if (res1.num_channel==1)
                        {
                            channel1.history.push(res1);
                        }
                    }
                    if (channel2.temp_history[res2.ts]===undefined||!channel2.temp_history[res2.ts]&&validDateInit2)
                    {
                       channel2.temp_history[res2.ts]=true;
                       channel2.history.push(res2);
                    }
                    else
                    {
                        if (res2.num_channel==2)
                        {
                            channel2.history.push(res2);
                        }
                    }
                    if (res1.ts>=dev.last_data_ts&&res1.ts.toString().length<=13)
                    {
                        dev.last_data_ts=res1.ts;
                    }
                }
            }
            else if (dev.device_type==20 || dev.device_type==21)
            {
                if (newData.type_package==1)
                {
                    var res  = new cell_history(json,newData,1,dev.device_type,dev.version);
                    for (var key in this.devices_list[index].channels)
                    {
                        var ia;
                        var channel = this.devices_list[index].channels[key];
                        if (res.address!==undefined)
                        {
                            ia = parseInt(res.address,16);
                            if (!isNaN(ia)&&ia==channel.ia)
                            {
                                channel.history.push(res);
                            }
                        }
                    }
                }
                else if (newData.type_package==5)
                {
                    var res1  = new cell_history(json,newData,11,dev.device_type,dev.version);
                    var res2  = new cell_history(json,newData,12,dev.device_type,dev.version);
                    if (channel3.temp_history[res1.ts]===undefined||!channel3.temp_history[res1.ts])
                    {
                       channel3.temp_history[res1.ts]=true;
                       channel3.history.push(res1);
                    }
                    else
                    {
                        if (res1.num_channel==1)
                        {
                            channel3.history.push(res1);
                        }
                    }
                    if (channel4.temp_history[res2.ts]===undefined||!channel4.temp_history[res2.ts])
                    {
                       channel4.temp_history[res2.ts]=true;
                       channel4.history.push(res2);
                    }
                    else
                    {
                        if (res2.num_channel==2)
                        {
                            channel4.history.push(res2);
                        }
                    }
                    if (res1.ts>=dev.last_data_ts&&res1.ts.toString().length<=13)
                    {
                        dev.last_data_ts=res1.ts;
                    }
                }
            }
            else
            {
                if (newData.type_package==1)
                {
                    for (var key in this.devices_list[index].channels)
                    {
                        try
                        {
                            var res  = new cell_history(json,newData,(parseInt(key)+1),this.devices_list[index].device_type,this.devices_list[index].version);
                            if (res)
                            {
                                var channel = this.devices_list[index].channels[key];
                                if (res.time!=undefined)
                                {
                                    var validDateInit = !channel.date_installation||res.time>moment(channel.date_installation).startOf('day' ).unix();
                                    if (validDateInit)
                                    {
                                        if (channel.temp_history === undefined) channel.temp_history = new Object();
                                        if (channel.temp_history[res.time]===undefined||!channel.temp_history[res.time])
                                        {
                                           channel.temp_history[res.time]=true;
                                           channel.history.push(res);
                                        }
                                        else
                                        {
                                            if (channel.type_channel==6)
                                            {
                                                channel.history.push(res);
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    channel.history.push(res);
                                }
                                if (res.ts>=this.devices_list[index].last_data_ts&&res.ts.toString().length<=13)
                                {
                                    this.devices_list[index].last_data_ts=res.ts;
                                    if (res.charge)
                                    {
                                        this.devices_list[index].last_data_charge = res.charge;
                                        channel.last_data_charge = res.charge;
                                    }
                                }
                            }
                        }
                        catch(err)
                        {
                            console.log(err);
                        }
                    }
                    return newData;
                }
                else if (newData.type_package==2)
                {
                    for (var key in this.devices_list[index].channels)
                    {
                        try
                        {
                           var res  = new cell_history(json,newData,(parseInt(key)+1),this.devices_list[index].device_type,this.devices_list[index].version);
                            if (res)
                            {
                                if (this.devices_list[index].channels[key].temp_history === undefined) this.devices_list[index].channels[key].temp_history = {};
                                if (res.time!==undefined)
                                {
                                    if (this.devices_list[index].channels[key].temp_history[res.time]===undefined||!this.devices_list[index].channels[key].temp_history[res.time])
                                    {
                                        var si12 = dev.device_type==2;
                                        if(si12)
                                        {
                                            var dangerChannel = this.devices_list[index].channels[key].type_channel == 6;
                                            if(dangerChannel && this.devices_list[index].channels[key].num_channel == res.num_channel )
                                            {
                                                this.devices_list[index].channels[key].temp_history[res.time]=true;
                                                this.devices_list[index].channels[key].history.push(res);
                                            }
                                        }
                                        else
                                        {
                                            this.devices_list[index].channels[key].temp_history[res.time]=true;
                                            this.devices_list[index].channels[key].history.push(res);
                                        }
                                    }
                                }
                                else
                                {
                                    var num = parseInt(key);
                                    var validNum = !isNaN(num)&&num+1==newData.num_channel;
                                    if (validNum)
                                    {
                                        this.devices_list[index].channels[key].history.push(res);
                                    }
                                }
                                if (res.ts>=this.devices_list[index].last_data_ts&&res.ts.toString().length<=13)
                                {
                                    this.devices_list[index].last_data_ts=res.ts;
                                    if (res.charge)
                                    {
                                        this.devices_list[index].last_data_charge = res.charge;
                                        this.devices_list[index].channels[key].last_data_charge = res.charge;
                                    }
                                }
                            }
                        }
                        catch(err)
                        {
                            console.log(err);
                        }
                    }
                    return newData;
                }
                else if (newData.type_package==3)
                {
        //            for (var key in newData.archive)
        //            {
        //                newData.sensors['sensor_'+(newData.num_channel)]=newData.archive[key][0];
        //                newData.time=newData.archive[key][1];
        //                try
        //                {
        //                    var res  = new cell_history(json,newData,newData.num_channel,this.devices_list[index].device_type);
        //                    if (res)
        //                    {
        //                        if (res.time!=undefined)
        //                        {
        //                            if (this.devices_list[index].channels[newData.num_channel].temp_history===undefined)
        //                            {
        //                                this.devices_list[index].channels[newData.num_channel].temp_history = new Object();
        //                            }
        //                            if (this.devices_list[index].channels[newData.num_channel].temp_history[res.time]===undefined||!this.devices_list[index].channels[newData.num_channel].temp_history[res.time])
        //                            {
        //                               this.devices_list[index].channels[newData.num_channel].temp_history[res.time]=true;
        //                               this.devices_list[index].channels[newData.num_channel].history.push(res);
        //                            }
        //                        }
        //                        else
        //                        {
        //                            this.devices_list[index].channels[newData.num_channel].history.push(res);
        //                        }
        //                    }
        //                }
        //                   catch(err)
        //                {
        //                    console.log(err);
        //                }
        //            }
        //            return newData;
                    return false;
                }
                else
                {
                    return false;
                }
            }
        }
        else
        {
            return false;
        }
        
    }
    add_history_multi_new(json,index,date,newData)
    {
        var ts = json.ts;
        for (var key in this.devices_list[index].channels)
        {
            var channel = this.devices_list[index].channels[key];
            var date_installation = channel.date_installation?parseInt(moment(channel.date_installation).startOf('day' ).format('x' )):0;
            var validDateInit = !date_installation||ts>date_installation;
            if( validDateInit )
            {
                var res  = new cell_history(json,newData,(parseInt(key)+1),this.devices_list[index].device_type,this.devices_list[index].version);
                if (res.ts>=this.devices_list[index].last_data_ts&&res.ts.toString().length<=13)
                {
                    this.devices_list[index].last_data_ts=res.ts;
                    if (res.charge)
                    {
                        this.devices_list[index].last_data_charge = res.charge;
                    }
                }
                channel.history.push(res);
            }
        }
    }
    add_history_single(json,index,date,newData)
    {
        var channel = this.devices_list[index].channels[0];
        var res  = new cell_history(json,newData,1,this.devices_list[index].device_type,this.devices_list[index].version);
        if (res.ts>=this.devices_list[index].last_data_ts&&res.ts.toString().length<=13)
        {
            this.devices_list[index].last_data_ts=res.ts;
            if (res.charge)
            {
                this.devices_list[index].last_data_charge = res.charge;
            }
        }
        this.devices_list[index].channels[0].history.push(res);
        var port5 = json.port === 5;
        var type8 = newData.type_package == 8;
        var type13 = newData.type_package == 13;
        var type14 = newData.type_package == 14;
        var type15 = newData.type_package == 15;
        var type33 = newData.type_package == 33;
        if(port5 && this.devices_list[index].device_type == 24)
        {
            if(type8)
            {
                var validTariffSchedule = isObject(channel.tariff_schedule)&&isObject(channel.tariff_schedule.request);
                if(validTariffSchedule)
                {
                    if(channel.tariff_schedule.request.uuid == res.UUID)
                    {
                        channel.tariff_schedule.response = {
                            result: res,
                            reset: function()
                            {
                                this.edit = cloneObject(this.result);
                            } 
                        };
                        channel.tariff_schedule.response.reset();
                    }
                }
            }
            if(type13||type14)
            {
                var validIndication = isObject(channel.indication)&&isObject(channel.indication.request);
                if( validIndication && channel.indication.request.uuid == res.UUID )
                {
                    if(!isObject(channel.indication.response)) 
                    {
                        channel.indication.response = {
                            block1:{},
                            block2:{},
                            reset: function()
                            {
                                var block1 = cloneObject(this.block1);
                                var block2 = cloneObject(this.block2);
                                this.edit = {};
                                var edit = this.edit;
                                for(var key in block1)
                                {
                                    if( block1[key] !== undefined ) edit[key] = block1[key];
                                }
                                for(var key in block2)
                                {
                                    if( block2[key] !== undefined ) edit[key] = block2[key];
                                }
                            }
                        };
                    }
                    if(type13)
                    {
                        channel.indication.response.block1 = res;
                    }
                    else if(type14)
                    {
                        channel.indication.response.block2 = res;
                    }
                    channel.indication.response.reset();
                }
            }
            if(type15||type33)
            {
                var validSpecialDays = isObject(channel.special_days)&&isObject(channel.special_days.request);
                if( validSpecialDays && channel.special_days.request.uuid == res.UUID )
                {
                    if(!isObject(channel.special_days.response)) 
                    {
                        channel.special_days.response = {
                            block1:{},
                            block2:{},
                            reset: function()
                            {
                                var block1 = cloneObject(this.block1);
                                var block2 = cloneObject(this.block2);
                                this.edit = {};
                                var edit = this.edit;
                                for(var key in block1)
                                {
                                    if( block1[key] !== undefined ) edit[key] = block1[key];
                                }
                                for(var key in block2)
                                {
                                    if( block2[key] !== undefined ) edit[key] = block2[key];
                                }
                            }
                        };
                    }
                    if(type15)
                    {
                        channel.special_days.response.block1 = res;
                    }
                    else if(type33)
                    {
                        channel.special_days.response.block2 = res;
                    }
                    channel.special_days.response.reset();
                }
            }
        } 
    }
    add_history(json,index,date)
    {   
        var device = this.devices_list[index];
        if (json.ts>=device.last_data_ts&&json.ts.toString().length<=13)
        {
            this.devices_list[index].last_data_ts=json.ts;
        }
        if (device&&device.device_type)
        {
            var newData = new data_lora(device.device_type,json.data,json.port,device.version);
            if (newData.validParse)
            {
                switch (newData.device_type) {
                    case 1:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 2:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 3:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 4:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 5:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 9:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 6:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 7:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 8:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 10:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 11:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 12:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 13:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 14:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 15:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 17:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 18:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 20:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 21:
                        this.add_history_multi_new(json,index,date,newData);
                        break;
                    case 23:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 24:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 25:
                        this.add_history_single(json,index,date,newData);
                        break;
                    case 26:
                        this.add_history_single(json,index,date,newData);
                        break;
                    default:
                        return false;
                        break;
                }
            }
            else
            {
                return false;
            }
            return newData;
        }
        else
        {
            return false;
        }
    };
    channels_search(num_channel,devEui,level_1,level_2)
    {
        for (var i=0;i< this.devices_list.length;i++)
        {
            if (this.devices_list[i].devEui==devEui)
            {
                for (var j=0;j< this.devices_list[i].channels.length;j++)
                {
                    if (this.devices_list[i].channels[j].address_level_1==level_1.address_level_1&&this.devices_list[i].channels[j].name_level_1==level_1.name_level_1&&this.devices_list[i].channels[j].level_2==level_2&&this.devices_list[i].channels[j].num_channel==num_channel)
                    {
                        return true;
                    }
                }
            }
        }
        return false;
    };
    device_search(devEui,level_1,level_2)
    {
        for (var i=0;i< this.devices_list.length;i++)
        {
            for (var j=0;j< this.devices_list[i].channels.length;j++)
            {
                if (this.devices_list[i].channels[j].name_level_1==level_1.name_level_1&&this.devices_list[i].channels[j].address_level_1==level_1.address_level_1&&this.devices_list[i].channels[j].level_2==level_2)
                {
                    return true;
                }
            }
        }
        return false;
    };
    level_2_search(level_2,level_1)
    {
        for (var i=0;i< this.devices_list.length;i++)
        {
            for (var j=0;j< this.devices_list[i].channels.length;j++)
            {
                if (this.devices_list[i].channels[j].name_level_1==level_1.name_level_1&&this.devices_list[i].channels[j].address_level_1==level_1.address_level_1&&this.devices_list[i].channels[j].level_2==level_2)
                {
                    return this.devices_list[i].channels[j];
                }
            }
        }
        return false;
    };
    level_1_search(name,address)
    {
        for (var i=0;i< this.devices_list.length;i++)
        {
            for (var j=0;j< this.devices_list[i].channels.length;j++)
            {
                if (this.devices_list[i].channels[j].name_level_1==name&&this.devices_list[i].channels[j].address_level_1==address)
                {
                    return this.devices_list[i].channels[j];
                }
            }
        }
        return false;
    };
    search_channel_device_list(num,devEui)
    {
        num--;
        for (var i=0;i< this.devices_list.length;i++)
        {
                if (devEui==this.devices_list[i].devEui&&this.devices_list[i].channels[num]!=undefined&&this.devices_list[i].channels[num].status!=undefined&&this.devices_list[i].channels[num].status)
                {
                   return true;
                }
        }
        return false;
    };
    select_channel(devEui,num_channel)
    {
        try
        {
            for (var i=0;i< this.devices_list.length;i++)
            {
                if (this.devices_list[i].devEui==devEui)
                {
                    for (var j=0;j< this.devices_list[i].channels.length;j++)
                    {
                        if (this.devices_list[i].channels[j].num_channel==num_channel)
                        {
                            this.select_device_devEui=devEui;
                            this.select_device_channel=num_channel;
                            this.select_channel_temp = this.devices_list[i].channels[j];
                            return true;
                        }
                    }
                }
            }
            return false;
        }
        catch(err)
        {
            return false; 
        }
    }
    get_active_channel(devEui)
   {
       var channels = this.get_device(devEui).channels;
       var resArr = [];
       if (channels)
       {
            for (var i =0;i<channels.length;i++)
            {
                if (channels[i].status!=undefined&&channels[i].status==1)
                {
                    resArr.push(channels[i].name);
                }
            }
       }
       return resArr;
   };
    get_device(devEui)
   {
       for (var i = 0; i<this.devices_list.length;i++)
       {
           if (this.devices_list[i].devEui==devEui)
           {
               return this.devices_list[i];
           }
       }
       return false;
   }
   reconnect_device(device)
   {
       for (var i = 0; i<this.devices_list.length;i++)
       {
           if (this.devices_list[i].devEui==device.devEui)
           {
               this.devices_list[i]=Object.assign(this.devices_list[i],device);
               return true;
           }
       }
       return false;
   }
   clean_tick_device_original()
   {
       for (var i = 0; i<this.full_devices_list.length;i++)
       {
            this.full_devices_list[i].tick=false;
       }
   }
   get_device_original(devEui)
   {
       for (var i = 0; i<this.full_devices_list.length;i++)
       {
           if (this.full_devices_list[i].devEui==devEui)
           {
               return this.full_devices_list[i];
           }
       }
       return undefined;
   }
   get_count_channel()
   {
       var count_channel=0;
       for (var i = 0; i< this.devices_list.length;i++)
       {
           for (var j = 0 ; j<this.devices_list[i].channels.length;j++)
           {
               if (this.devices_list[i].channels[j].type_channel != undefined && this.devices_list[i].channels[j].num_channel !=undefined)
               {
                   count_channel++
               }
           }
       }
       return count_channel;
   }
   get_channel(devEui,channel)
   {
       try
       {
            for (var i = 0; i<this.devices_list.length;i++)
            {
                if (this.devices_list[i].devEui==devEui)
                {
                     for (var j = 0; j<this.devices_list[i].channels.length;j++)
                     {
                         if (this.devices_list[i].channels[j].num_channel==channel)
                         {
                             return this.devices_list[i].channels[j];
                         }
                     }
                }
            }
        }
        catch (err)
        {
            return [];
        }
        return new Object();
   }
   get_select_channel()
   {
       try
       {
            for (var i = 0; i<this.devices_list.length;i++)
            {
                if (this.devices_list[i].devEui==this.select_device_devEui)
                {
                     for (var j = 0; j<this.devices_list[i].channels.length;j++)
                     {
                         if (this.devices_list[i].channels[j].num_channel==this.select_device_channel)
                         {
                             return this.devices_list[i].channels[j];
                         }
                     }
                }
            }
        }
        catch (err)
        {
            return [];
        }
   }
   get_history(channel,date)
   {
       try
       {
        var history = channel.history;
        var temp = new Array();
        var dev = this.get_device(channel.devEui);
        var dt = 0;
        if (dev) dt = dev.device_type;
        for (var i = 0; i <history.length;i++)
        {
            var validTs = history[i].ts !== undefined && history[i].ts>date.getStart()*1000&&history[i].ts<date.getEnd()*1000;
            var validTime = history[i].time !== undefined && history[i].time>=date.getStart()&&history[i].time<=date.getEnd();
            var ss = channel.type_channel==13;
            var smart_type_channel = (channel.type_channel==10||channel.type_channel==11||channel.type_channel==12||ss)&&dev.version == 0;
            var smartRev2_type_channel = (channel.type_channel==10||channel.type_channel==11||channel.type_channel==12)&&dev.version >= 1;
            var hs = channel.type_channel==23;
            var um = channel.type_channel==25;
            var si13 = dt == 3;
            var si13_modBus = si13&&channel.type_channel==22&&dev.version > 0;
            var si13_m = si13&&channel.type_channel==16&&dev.version == 0;
            var si13_m_rev2 = si13&&channel.type_channel==16&&dev.version == 1;
            var si13_danger = si13 && channel.type_channel==6&&dev.version == 0;
            var si13_danger_rev2 = si13 && channel.type_channel==6&&dev.version == 1;
            var si21 = dt == 11;
            var si22 = dt == 18;
            var si21_22rev2 = ( si21 || si22 ) && dev.version > 0;
            if(si13)
            {
                if(si13_m&&validTs)
                {
                    temp.push(history[i]);
                }
                else if (si13_m_rev2&&validTime)
                {
                    temp.push(history[i]);
                } 
                else if(si13_danger&&validTs)
                {
                    temp.push(history[i]);
                }
                else if(si13_danger_rev2&&validTime)
                {
                    temp.push(history[i]);
                }
                else if(si13_modBus && validTime)
                {
                    temp.push(history[i]);
                }
                else if (validTs)
                {
                    temp.push(history[i]);
                }
            }
            else if (channel.type_channel==7||channel.type_channel==9||smart_type_channel)
            {
                if (history[i].ts>date.getStart()*1000&&history[i].ts<date.getEnd()*1000)
                {
                    temp.push(history[i]);
                }
            }
            else if ( channel.type_channel==6 && ( dt == 20 || dt == 21 ) )
            {
                if (history[i].ts>date.getStart()*1000&&history[i].ts<date.getEnd()*1000)
                {
                    temp.push(history[i]);
                } 
            }
            else if ((channel.type_channel==8||channel.type_channel==6||smartRev2_type_channel || hs || um) && !si13)
            {
                if( si21_22rev2 && ( validTs || validTime ) )
                {
                    temp.push(history[i]);
                } 
                else if (history[i].time>=date.getStart()&&history[i].time<=date.getEnd())
                {
                    temp.push(history[i]);
                } 
            }
            else
            {  
                if ( dt == 20 || dt == 21 )
                {
                    if (history[i].ts>date.getStart()*1000&&history[i].ts<date.getEnd()*1000)
                    {
                        temp.push(history[i]);
                    }
                }
                else if (dt == 12 )
                {
                    var validTime = history[i].time>=date.getStart()&&history[i].time<=date.getEnd();
                    var validDate1 = history[i].date_1&&history[i].date_1>date.getStart()&&history[i].date_1<date.getEnd();
                    var validDate1 = history[i].date_2&&history[i].date_2>date.getStart()&&history[i].date_2<date.getEnd();
                    if (validTime||validDate1||validDate1)
                    {
                        if (history[i].type!=5||(validDate1&&history[i].type==5))
                        {
                            temp.push(history[i]);
                        }
                    }
                }
                // else if ( dt == 24 )
                // {
                //     var validTime = history[i].time>date.getStart()&&history[i].time<date.getEnd();
                //     var validDate1 = history[i].date_1&&history[i].date_1>date.getStart()&&history[i].date_1<date.getEnd();
                //     var validDate1 = history[i].date_2&&history[i].date_2>date.getStart()&&history[i].date_2<date.getEnd();
                //     if (validTime||validDate1||validDate1||history[i].time==undefined)
                //     {
                //         if (history[i].type!=5||(validDate1&&history[i].type==5))
                //         {
                //             temp.push(history[i]);
                //         }
                //     }
                // }
                else if (history[i].time>=date.getStart()&&history[i].time<=date.getEnd())
                {
                    temp.push(history[i]);
                } 
            }
        }
        return temp;
       }
       catch(err)
       {
           return []
       }
   }

   get_history_select_channel(date)
   {
        var channel = this.get_select_channel()
        return this.get_history(channel,date);    
   }
   get_history_last(history)
   {
       try
       {
            var temp = new Object();
            temp.time = 0;
            temp.index = 0;
            for (var i = 0;i<history.length ;i++)
            {
                if (history[i].time>temp.time) 
                {
                    temp.time = history[i].time;
                    temp.index = i;
                }
            }
            return history[temp.index];
        }
        catch (err)
        {
            return new cell_history();
        }
   }
   //получение статистики
   get_statistics(date)
   {
       this.reload_all_channel_list();
       try
        { 
            var device = this.get_device(this.select_device_devEui);
            var channel = this.get_channel(this.select_device_devEui,this.select_device_channel);
            var minDate = 99999999999999;
            var maxDate = 0;
            var minS = undefined;
            var maxS = undefined;
            var fcnt = 0;
            var snr = new Object();
            snr.val = 0;
            snr.count = 0;
            var rssi = new Object();
            rssi.val = 0;
            rssi.count=0;
            var _dangers = new Object();
            var connect = [];
            var arr_date = [];
            var arr_rssi = [];
            var arr_snr = [];
            var tempArr  = new Array();
            var arr_value = [];
            var arr_value_4 = [];
            var temp = new Object();
            var my_converter = new vega_converter();
            channel.statistics.last_coordinate = {
                time:0
            };
            channel.statistics.arr_valid_coordinates = [];
            channel.statistics.count = 0;
            channel.statistics.countPackage_type_1 = 0;
            channel.statistics.countPackage_type_2 = 0;
            channel.statistics.countPackage_type_3 = 0;
            channel.statistics.countPackage_type_4 = 0;
            channel.statistics.countPackage_type_5 = 0;
            channel.statistics.countPackage_type_6 = 0;
            channel.statistics.count_1 = 0;
            channel.statistics.count_all = 0;
            channel.statistics.count_full = 0;
            channel.statistics.count_danger = 0;
            channel.statistics.count_danger_full = 0;
            channel.statistics.chart = new Array();
            channel.statistics.chart_consumption = new Array();
            channel.statistics.chart_t = new Array();
            channel.statistics.arr_value_consum = new Array();
            channel.statistics.arr_value_consum_v4 = new Array();
            channel.statistics.arr_value_events = new Array();
            channel.statistics.chart_temperature = new Array();
            channel.statistics.chart_temperature_new = new Array();
            channel.statistics.chart_sensor_new = new Array();
            channel.statistics.last_ts=0;
            channel.statistics.last_time=0;
            channel.statistics.arr_power_profile = [];
            var valid = new valid_data();
            var kt = this.select_channel_temp.kt;
            kt = valid.isNumber(kt)?kt:1;
            var arr_ts_list = [];
            var summ_t=0;
            var count_t=0;
            var _items = new Object();
            var _items_1 = new Object();
            var _items_counter_all = new Object();
            var _items_counter = new Object();
            var _items_counter_power = new Object();
            var _items_counter_2 = new Object();
            var _items_counter_p_2 = new Object();
            var _items_all = new Object();
            var last_type_1 = {ts:null,time:null,value:null};
            var last_type_5 = {ts:null,time:null,value:null};
            var last_sensor_out_1 = {ts:null,time:null,value:null};
            var last_sensor_out_2 = {ts:null,time:null,value:null};
            var last_state_energy = {ts:null,time:null,value:null};
            
            var last_charge = {ts:null,time:null,value:null};
            var last_charge = {ts:null,time:null,value:null};
            var summ_sensorTP = 0;
            var count_sensorTP = 0;
            var si11 = device.device_type == 1;
            var si21 = device.device_type == 11;
            var si22 = device.device_type == 18;
            var si12 = device.device_type == 2;
            var impulse = channel.type_channel >= 1 && channel.type_channel<=5;
            var channelDanger = channel.type_channel==6;
            var si13 = device.device_type == 3 && device.version == 0;
            var si13rev2 = device.device_type == 3 && device.version == 1;
            var smart = (device.device_type == 10 || device.device_type == 8 || device.device_type == 7 || device.device_type == 6)&&device.version == 0;
            var hs = device.device_type == 23;
            var um = device.device_type == 25;
            var smartRev2 = ( device.device_type == 8 || device.device_type == 7 || device.device_type == 6)&&device.version >= 1;
            var td11 =  device.device_type == 4 && device.version == 0;
            var td11rev2 =  device.device_type == 4 && device.version == 1;
            var tp11 = device.device_type == 5 && device.version == 0;
            var tp11rev2 = device.device_type == 5 && device.version == 1;
            var mbus1 = device.device_type == 20;
            var mbus2 = device.device_type == 21;
            var mbus = mbus1 || mbus2;
            var gm2 = device.device_type == 13;
            var gm1 = device.device_type == 17;
            var sve = device.device_type == 9;
            var tl11 = device.device_type == 15;
            var lm11 = device.device_type == 14;
            var src = device.device_type == 26;
            var ue = device.device_type == 12;
            var ue_spbzip = device.device_type == 24 ;

            var tz = get_time_zone();
            for (var i = 0; i< channel.history.length;i++)
            {
                var cell_history = channel.history[i];
                if( device.version != channel.history[i].version ) continue;
                var validHistory = channel.history[i] !== undefined;
                if (td11rev2&&validHistory)
                {
                    tz = get_time_zone() * 1000;
                    // td11
                    var checkEvent = cell_history.reason!==0;
                    var port = cell_history.port;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var validType = cell_history.type == 1;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    if ( port == 2 )
                    {
                        if ( correctTime && validType )
                        {
                            var timeMs = cell_history.time * 1000;
                            var _item = [cell_history.time,1,0];
                            if (cell_history.time > channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_charge=cell_history.charge;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_fcnt=cell_history.fcnt; 
                                channel.statistics.last_hall_1 = cell_history.hall_1;  
                                channel.statistics.last_hall_2 = cell_history.hall_2;  
                                channel.statistics.last_state_security = cell_history.state_security;  
                                channel.statistics.last_state_tamper = cell_history.state_tamper;  
                                channel.statistics.last_limit_exceeded = cell_history.limit_exceeded;
                                channel.statistics.last_min_temperature = cell_history.min_temperature;
                                channel.statistics.last_max_temperature = cell_history.max_temperature;
                            }
                            if ((checkEvent||cell_history.limit_exceeded)&&!_dangers[timeMs])
                            {
                                _dangers[timeMs]=true;
                                channel.statistics.count_danger++;
                                if (checkEvent)
                                {
                                    _item[2] = 1;
                                    _item[3] = cell_history.reason;
                                }
                                if (cell_history.limit_exceeded)
                                {
                                    _item[2] = 2;
                                    _item[3] = cell_history.reason;
                                }
                                if (cell_history.limit_exceeded&&checkEvent)
                                {
                                    _item[2] = 3;
                                    _item[3] = cell_history.reason;
                                }
                            }
                            if (!_items[timeMs])
                            {
                                if (!isNaN(cell_history.temperature))
                                {
                                    summ_t += cell_history.temperature;
                                    count_t++;
                                }
                                channel.statistics.count++;
                                _items[timeMs]=true;
                                channel.statistics.chart_temperature_new.push( [timeMs+tz,cell_history.temperature,cell_history.min_temperature,cell_history.max_temperature, cell_history.limit_exceeded]);
                                arr_value_4.push(_item);
                                channel.statistics.chart_temperature.push( [timeMs, cell_history.temperature] );
                            }
                        }
                    }
                }
                else if ( td11 )
                {
                    //td11
                    var checkEvent = cell_history.reason!==0;
                    var t = parseFloat(cell_history.temperature);
                    var _item = [cell_history.ts,1,0];
                    var t_max = channel.max_t;
                    var t_min = channel.min_t;
                    var checkTemperature = t<=t_min||t>=t_max;
                    var validTs = cell_history.ts!==undefined&&cell_history.ts>0;
                    var validType = cell_history.type == 1;
                    var correctTs = validTs&&cell_history.ts>=(date.getStart()*1000)&&cell_history.ts<=(date.getEnd()*1000);
                    var port = cell_history.port;
                    if ( correctTs && validType &&  port == 2)
                    {
                        if ( cell_history.ts > channel.statistics.last_ts )
                        {
                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_time=cell_history.time;
                            channel.statistics.last_fcnt=cell_history.fcnt; 
                            channel.statistics.last_hall_1 = cell_history.hall_1;  
                            channel.statistics.last_hall_2 = cell_history.hall_2;  
                            channel.statistics.last_state_security = cell_history.state_security;  
                            channel.statistics.last_state_tamper = cell_history.state_tamper;  
                            channel.statistics.last_limit_exceeded = cell_history.limit_exceeded;
                            channel.statistics.last_min_temperature = cell_history.min_temperature;
                            channel.statistics.last_max_temperature = cell_history.max_temperature;
                            channel.statistics.last_ts=cell_history.ts;
                        }
                        if ((checkEvent||checkTemperature)&&!_dangers[cell_history.ts])
                        {
                            _dangers[cell_history.ts]=true;
                            channel.statistics.count_danger++;
                            if (checkEvent)
                            {
                                _item[2] = 1;
                                _item[3] = cell_history.reason;
                            }
                            if (checkTemperature)
                            {
                                _item[2] = 2;
                                _item[3] = cell_history.reason;
                            }
                            if (checkTemperature&&checkEvent)
                            {
                                _item[2] = 3;
                                _item[3] = cell_history.reason;
                            }
                        }
                        if (!_items[cell_history.ts])
                        {
                            channel.statistics.count++;
                            _items[cell_history.ts]=true;
                            arr_value_4.push(_item);
                            if (!isNaN(cell_history.temperature))
                            {
                                summ_t += cell_history.temperature;
                                count_t++;
                            }
                        }
                    }
                }
                else if ( sve )
                {
                    //sve
                    var port = cell_history.port;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var validType = cell_history.type == 1;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    if( validType && validTime )
                    {
                        if (correctTime)
                        {
                            var _item = [cell_history.time,1,0];
                            var checkEvent = cell_history.state_display||cell_history.leaking||cell_history.breakthrough||cell_history.hall_1;
                            if (checkEvent&&!_dangers[cell_history.time])
                            {
                                _dangers[cell_history.time]=true;
                                channel.statistics.count_danger++;
                                _item[2] = 1;
                                _item[3] = 'danger';
                            }
                            if (!_items[cell_history.time])
                            {
                                channel.statistics.count++;
                                _items[cell_history.time]=true;
                                arr_value_4.push(_item);
                            }
                        }
                        if (cell_history.time>channel.statistics.last_time)
                        {
                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_time=cell_history.time;
                            channel.statistics.last_fcnt=cell_history.fcnt; 
                            channel.statistics.last_hall_1 = cell_history.hall_1;  
                            channel.statistics.last_sensorKB = cell_history.sensorKB;
                            channel.statistics.last_state_display=cell_history.state_display;
                            channel.statistics.last_leaking = cell_history.leaking;
                            channel.statistics.last_breakthrough = cell_history.breakthrough;
                        }
                    }
                }
                else if (tl11)
                {
                    //tl11
                    var port = cell_history.port;
                    if (cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd()&&port==2)
                    {
                        var timeMs = cell_history.time*1000;
                        var _item = [timeMs,1,0];
                        channel.statistics.count_all++;
                        _items_all[timeMs]=true;
                        var t_max = channel.max_t;
                        var t_min = channel.min_t;
                        var t = parseFloat(cell_history.temperature);
                        var t2 = parseFloat(cell_history.temperature_2);
                        
                        var checkTemperature_1 = t<=t_min||t>=t_max; //5
                        var checkTemperature_2 = t2<=t_min||t2>=t_max; //6
                        var normReason = cell_history.reason=='00';
                        
                        if (checkTemperature_1||checkTemperature_2||!normReason)
                        {
                            _item[1]=parseInt(cell_history.reason);
                            _item[2]=1;
                            
                            
                            var event = new Object();
                            event.time = cell_history.time;
                            event.reason = cell_history.reason;
                            event.deviation = '';
                            if (checkTemperature_1)
                            {
                                event.deviation+='Температура 1'+', ';
                            }
                            if (checkTemperature_2)
                            {
                                event.deviation+='Температура 2'+', ';
                            }
                            if (!normReason)
                            {
                                event.deviation+='Незапланированное событие'+', ';
                            }
                            if (event.deviation.length>0)
                            {
                                event.deviation = event.deviation.slice(0,event.deviation.length-2);
                            }
                            event.temperature_1 = t;
                            event.temperature_2 = t2;
                            event.fcnt = cell_history.fcnt;
                            event.get_time = cell_history.get_time;
                            channel.statistics.arr_value_events.push(event);
                        }
                        
                        if (cell_history.time>channel.statistics.last_time)
                        {
                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_temperature_2=cell_history.temperature_2;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_time=cell_history.time;
                            channel.statistics.last_ts=cell_history.ts;
                            channel.statistics.last_fcnt=cell_history.fcnt;
                            channel.statistics.last_hall_1 = cell_history.hall_1;  
                            channel.statistics.last_hall_2 = cell_history.hall_2;  
                            channel.statistics.last_state_tamper = cell_history.state_tamper;  
                        }
                        channel.statistics.chart_t.push( [timeMs,cell_history.temperature,cell_history.temperature_2]);
                        arr_value_4.push(_item);
                    }
                }
                else if (gm2)
                {
                    //УГ
                    var port = cell_history.port;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                    if (port == 2)
                    {
                        if (correctTime&&!_items_counter[cell_history.time])
                        {
                            var _item = [cell_history.time,1,0];
                            _items_counter[cell_history.time]=true;
                            var reason = cell_history.reason;
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_charge=cell_history.charge;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_ts=cell_history.ts;
                                channel.statistics.last_fcnt=cell_history.fcnt;
                                channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;
                                channel.statistics.last_sensor_in_1=cell_history.sensor_in_1;
                                channel.statistics.last_sensor_in_2=cell_history.sensor_in_2;
                                channel.statistics.last_sensor_out_1=cell_history.sensor_out_1;
                                channel.statistics.last_sensor_out_2=cell_history.sensor_out_2;
                                channel.statistics.last_hall_1 = cell_history.hall_1;  
                                channel.statistics.last_state_tamper = cell_history.state_tamper;  
                            }
                            if (cell_history.time<minDate)
                            {
                                minDate = cell_history.time;
                                minS = cell_history.sensor_rate_sum;
                            }
                            if (cell_history.time>maxDate)
                            {
                                maxDate = cell_history.time;
                                maxS = cell_history.sensor_rate_sum;
                            }
                            if (reason)
                            {
                                reason = parseInt(reason);
                                if (!isNaN(reason)&&reason>0)
                                {
                                    _item[2]=1;
                                    _item[1] = reason;
                                }
                            }
                            arr_value_4.push(_item);
                            channel.statistics.chart.push( [(cell_history.time+get_time_zone())*1000,cell_history.sensor_rate_sum,reason]);
                            channel.statistics.chart_consumption.push( [(cell_history.time+get_time_zone())*1000,cell_history.sensor_rate_sum,reason,cell_history.fcnt]);
                            
                            channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                        }
                        else if(validTime&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            channel.statistics.chart_consumption.push( [(cell_history.time+get_time_zone())*1000,cell_history.sensor_rate_sum,reason,cell_history.fcnt]);
                        }
                    }
                }
                else if (gm1)
                {
                    //GM-1
                    var port = cell_history.port;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                    var tz = get_time_zone() * 1000;
                    if (port == 2)
                    {
                        var _item = [cell_history.time,1,0];
                        var reason = cell_history.reason;
                        if (cell_history.ts>channel.statistics.last_ts)
                        {
                            channel.statistics.last_ts=cell_history.ts;
                        }
                        if(validTime)
                        {
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_charge=cell_history.charge;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_fcnt=cell_history.fcnt;
                                channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;
                            }
                        }
                        if (correctTime&&!_items_counter[cell_history.time])
                        {
                            var timeMS = cell_history.time * 1000;
                            var time = timeMS + tz;
                            _items_counter[cell_history.time]=true;
                            if (cell_history.time<minDate)
                            {
                                minDate = cell_history.time;
                                minS = cell_history.sensor_rate_sum;
                            }
                            if (cell_history.time>maxDate)
                            {
                                maxDate = cell_history.time;
                                maxS = cell_history.sensor_rate_sum;
                            }
                            if (reason)
                            {
                                reason = parseInt(reason);
                                if (!isNaN(reason)&&reason>0)
                                {
                                    _item[2]=1;
                                    _item[1] = reason;
                                }
                            }
                            arr_value_4.push(_item);
                            channel.statistics.chart.push( [time,cell_history.sensor_rate_sum,reason] );
                            channel.statistics.chart_consumption.push( [time,cell_history.sensor_rate_sum,reason,cell_history.fcnt] );
                            channel.statistics.chart_temperature.push( [timeMS,cell_history.temperature]);
                        }
                        else if(validTime&&!_items_counter[cell_history.time])
                        {
                            _items_counter[cell_history.time]=true;
                            channel.statistics.chart_consumption.push( [time,cell_history.sensor_rate_sum,reason,cell_history.fcnt] );
                        }
                    }
                }
                else if ( lm11 )
                {
                    //lm11
                    var type_package = cell_history.type;
                    var port = cell_history.port;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    var _item_counter = [cell_history.time,0,0];
                    if (port == 2 && type_package==1)
                    {
                        if (cell_history.time>=channel.statistics.last_time)
                        {
                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_ts=cell_history.ts;
                            channel.statistics.last_time=cell_history.time;
                            channel.statistics.last_fcnt=cell_history.fcnt;

                            channel.statistics.last_in_move=cell_history.in_move;
                            channel.statistics.last_speed=cell_history.speed;
                            channel.statistics.last_sat_visible=cell_history.sat_visible;
                            channel.statistics.last_sat_used=cell_history.sat_used;
                            channel.statistics.last_alarm=cell_history.alarm;
                            channel.statistics.last_angle=cell_history.angle;
                            
                            channel.statistics.last_lat=cell_history.lat;
                            channel.statistics.last_lng=cell_history.lng;
                        }
                        if (correctTime&&!_items_counter[cell_history.time])
                        {
                            if (cell_history.time>=channel.statistics.last_coordinate.time&&cell_history.coord_status)
                            {
                                channel.statistics.last_coordinate.ts = cell_history.ts;
                                channel.statistics.last_coordinate.time = cell_history.time;
                                channel.statistics.last_coordinate.temperature=cell_history.temperature;
                                channel.statistics.last_coordinate.charge=cell_history.charge;
                                channel.statistics.last_coordinate.fcnt=cell_history.fcnt;
                                channel.statistics.last_coordinate.in_move=cell_history.in_move;
                                channel.statistics.last_coordinate.speed=cell_history.speed;
                                channel.statistics.last_coordinate.sat_visible=cell_history.sat_visible;
                                channel.statistics.last_coordinate.sat_used=cell_history.sat_used;
                                channel.statistics.last_coordinate.alarm=cell_history.alarm;
                                channel.statistics.last_coordinate.lat=cell_history.lat;
                                channel.statistics.last_coordinate.lng=cell_history.lng;
                            }
                            var _item = [cell_history.time,1,0];
                            _items_counter[cell_history.time]=true;
                            
                            if (cell_history.time<minDate)
                            {
                                minDate = cell_history.time;
                            }
                            if (cell_history.time>maxDate)
                            {
                                maxDate = cell_history.time;
                            }
                            if (cell_history.alarm)
                            {
                                _item[2]=1;
                                _item[1] = 1;
                            }
                            if (cell_history.lat&&cell_history.lng) channel.statistics.arr_valid_coordinates[cell_history.time]=cell_history;
                            arr_value_4.push(_item);
                            channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                        }
                    }
                }
                else if ( src )
                {
                    //src
                    var type_package = cell_history.type;
                    var port = cell_history.port;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    var _item_counter = [cell_history.time,0,0];
                    if (port == 2)
                    {
                        if (cell_history.time>=channel.statistics.last_time)
                        {
                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_ts=cell_history.ts;
                            channel.statistics.last_time=cell_history.time;
                            channel.statistics.last_fcnt=cell_history.fcnt;
                            channel.statistics.last_status_state=cell_history.status_state;
                            channel.statistics.last_state_tamper=cell_history.state_tamper;
                            channel.statistics.last_led_duty=cell_history.led_duty;
                            channel.statistics.last_nomenal_voltage=cell_history.nomenal_voltage;
                            channel.statistics.last_reason=cell_history.reason;
                            channel.statistics.last_angle=cell_history.angle;
                            channel.statistics.last_nomenal_voltage=cell_history.nomenal_voltage;
                            channel.statistics.last_total_power_summary=cell_history.total_power_summary;
                            channel.statistics.last_lat=cell_history.lat;
                            channel.statistics.last_lng=cell_history.lng;
                        }
                        if (correctTime&&!_items_counter[cell_history.time])
                        {
                            if (cell_history.time>=channel.statistics.last_coordinate.time)
                            {
                                channel.statistics.last_coordinate.temperature=cell_history.temperature;
                                channel.statistics.last_coordinate.charge=cell_history.charge;
                                channel.statistics.last_coordinate.ts=cell_history.ts;
                                channel.statistics.last_coordinate.time=cell_history.time;
                                channel.statistics.last_coordinate.fcnt=cell_history.fcnt;
                                channel.statistics.last_coordinate.status_state=cell_history.status_state;
                                channel.statistics.last_coordinate.state_tamper=cell_history.state_tamper;
                                channel.statistics.last_coordinate.led_duty=cell_history.led_duty;
                                channel.statistics.last_coordinate.nomenal_voltage=cell_history.nomenal_voltage;
                                channel.statistics.last_coordinate.reason=cell_history.reason;
                                channel.statistics.last_coordinate.angle=cell_history.angle;
                                channel.statistics.last_coordinate.nomenal_voltage=cell_history.nomenal_voltage;
                                channel.statistics.last_coordinate.total_power_summary=cell_history.total_power_summary;
                                channel.statistics.last_coordinate.lat=cell_history.lat;
                                channel.statistics.last_coordinate.lng=cell_history.lng;
                            }
                            var _item = [cell_history.time,1,0];
                            _items_counter[cell_history.time]=true;
                            
                            if (cell_history.time<minDate)
                            {
                                minDate = cell_history.time;
                            }
                            if (cell_history.time>maxDate)
                            {
                                maxDate = cell_history.time;
                            }
                            if (cell_history.alarm)
                            {
                                _item[2]=1;
                                _item[1] = 1;
                            }
                            if (cell_history.lat&&cell_history.lng) channel.statistics.arr_valid_coordinates[cell_history.time]=cell_history;
                            arr_value_4.push(_item);
                            channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                        }
                    }
                }
                else if (ue)
                {
                    //ue
                    var type_package = cell_history.type;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var validDate1 = cell_history.date_1!==undefined&&cell_history.date_1>0;
                    var validDate2 = cell_history.date_2!==undefined&&cell_history.date_2>0;
                    var correctTime = validTime&&(cell_history.time)>=date.getStartUTC()&&(cell_history.time)<=date.getEndUTC();
                    var correctDate1 = validDate1&&(cell_history.date_1)>date.getStartUTC()&&(cell_history.date_1)<date.getEndUTC();
                    var correctDate2 = validDate2&&(cell_history.date_2)>date.getStartUTC()&&(cell_history.date_2)<date.getEndUTC();
                    var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                    var _item_counter = [cell_history.time,0,0];
                    
                    channel.statistics.count_full++;
                    if (type_package == 1)
                    {
                        if (correctTime&&!_items_counter[cell_history.time])
                        {
                            //Эту запись можно рассматривать как нормальный первый пакет
                            //фиксация
                            channel.statistics.count_all++;
                            channel.statistics.count_1++;
                            channel.statistics.countPackage_type_1++;
                            var t = parseFloat(cell_history.temperature);
                            if (!isNaN(t))
                            {
                                summ_t += t;
                                count_t++;
                            }
                            
                            _items_counter[cell_history.time]=true;
                            //-----------------------------------------//
                            if (cell_history.time<minDate&&validSensor)
                            {
                                minDate = cell_history.time;
                                minS = cell_history.sensor_rate_sum;
                            }
                            if (cell_history.time>maxDate&&validSensor)
                            {
                                maxDate = cell_history.time;
                                maxS = cell_history.sensor_rate_sum;
                            }
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_time=cell_history.time;
                                if (validSensor)channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;
                                channel.statistics.last_release_date=cell_history.release_date;
                                channel.statistics.last_release_date_rus=moment.unix(channel.statistics.last_release_date).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_count_phase_1=cell_history.count_phase;
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_state_relay=cell_history.state_3;
                                channel.statistics.last_state_KK=cell_history.state_1;
                                channel.statistics.last_state_K=cell_history.state_2;
                                channel.statistics.last_relay=cell_history.relay_state;
                                channel.statistics.last_event=cell_history.event;
                            }
                            //ГРАФИК ТЕМПЕРАТУР
                            if (!_items_counter_all[cell_history.time])
                            {
                                channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                                _items_counter_all[cell_history.time]=true;
                            }
                            if (validSensor)
                            {
                                //ГРАФИК ПОКАЗАНИЙ
                                channel.statistics.chart.push( [cell_history.time*1000,cell_history.sensor_rate_sum,cell_history.event,cell_history.fcnt]);

                                channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,cell_history.event,cell_history.fcnt] );

                                channel.statistics.arr_value_consum.push( [cell_history.time*1000,undefined,undefined,undefined,undefined,cell_history.sensor_rate_sum,undefined,undefined]);
                            }
                            if (cell_history.event==19)
                            {
                                _item_counter[1] = cell_history.event;
                                _item_counter[2] = 2;
                            }
                            else if (cell_history.event!=1)
                            {
                                _item_counter[1] = cell_history.event;
                                _item_counter[2] = 3;
                                channel.statistics.count_danger++;
                            }
                            //ГРАФИК СОБЫТИЙ
                            arr_value_4.push(_item_counter);
                        }
                        else if(validTime&&!_items_counter_all[cell_history.time]&&validSensor)
                        {
                            _items_counter_all[cell_history.time]=true;
                            channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,cell_history.event,cell_history.fcnt] );
                        }
                    }
                    else if (type_package==4)
                    {
                        if (correctTime&&!_items_counter_2[cell_history.time])
                        {
                            channel.statistics.countPackage_type_4++;
                            _items_counter_2[cell_history.time]=true;
                            if (validSensor)
                            {
                                if (cell_history.time>channel.statistics.last_time)
                                {
                                    channel.statistics.last_time=cell_history.time;
                                    channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;

                                }
                                var timeMS = cell_history.time*1000;
                                var sensor_rate_1 = cell_history.sensor_rate_1;
                                var sensor_rate_2 = cell_history.sensor_rate_2;
                                var sensor_rate_3 = cell_history.sensor_rate_3;
                                var sensor_rate_4 = cell_history.sensor_rate_4;
                                var sensor_rate_sum = cell_history.sensor_rate_sum;
                                var count_rate_active = cell_history.count_rate_active;
                                var rate_active = cell_history.rate_active;
                                var uuid = cell_history.UUID;
                                if (cell_history.time<minDate)
                                {
                                    minDate = cell_history.time;
                                    minS = cell_history.sensor_rate_sum;
                                }
                                if (cell_history.time>maxDate)
                                {
                                    maxDate = cell_history.time;
                                    maxS = cell_history.sensor_rate_sum;
                                }
                                if (!_items_counter_all[cell_history.time])
                                {
                                    channel.statistics.chart.push( [cell_history.time*1000,cell_history.sensor_rate_sum,0,cell_history.fcnt] );

                                    channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,0,cell_history.fcnt] );

                                    _items_counter_all[cell_history.time]=true;
                                }
                                channel.statistics.arr_value_consum.push( [timeMS,sensor_rate_1,sensor_rate_2,sensor_rate_3,sensor_rate_4,sensor_rate_sum,count_rate_active,rate_active]);
                                if (uuid===undefined||uuid==='0000' )
                                {
                                
                                    channel.statistics.arr_value_consum_v4.push( [timeMS,sensor_rate_1,sensor_rate_2,sensor_rate_3,sensor_rate_4,sensor_rate_sum,count_rate_active,rate_active]);
                                }
                            }
                        }
                        else if(validTime&&!_items_counter_all[cell_history.time]&&validSensor)
                        {
                            _items_counter_all[cell_history.time]=true;
                            channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,0,cell_history.fcnt] );
                        }
                    }
                    else if (type_package==5)
                    {
                        var history_p5 = cell_history;
                        var uuid = cell_history.UUID;
                        if ((correctDate1)&&(uuid===undefined||uuid==='0000' )&&!_items_counter_power[cell_history.date_1])
                        {
                            channel.statistics.countPackage_type_5++;
                            _items_counter_power[cell_history.date_1]= true;
                            channel.statistics.arr_power_profile.push(history_p5);
                        }
                    }
                    else if (type_package==2)
                    {
                        if (correctTime&&!_items_counter_p_2[cell_history.time])
                        {
                            _items_counter_p_2[cell_history.time] = true;
                            channel.statistics.countPackage_type_2++;
                        }
                    }
                }
                else if (ue_spbzip)
                {
                    //ue
                    var type_package = cell_history.type;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var validDate1 = cell_history.date_1!==undefined&&cell_history.date_1>0;
                    var validDate2 = cell_history.date_2!==undefined&&cell_history.date_2>0;
                    var correctTime = validTime&&(cell_history.time)>=date.getStartUTC()&&(cell_history.time)<=date.getEndUTC();
                    var correctDate1 = validDate1&&(cell_history.date_1)>date.getStartUTC()&&(cell_history.date_1)<date.getEndUTC();
                    var correctDate2 = validDate2&&(cell_history.date_2)>date.getStartUTC()&&(cell_history.date_2)<date.getEndUTC();
                    var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                    var _item_counter = [cell_history.time,0,0];
                    
                    if( channel.statistics['countPackage_type_'+type_package] === undefined ) channel.statistics['countPackage_type_'+type_package] = 0;
                    channel.statistics['countPackage_type_'+type_package]++;

                    channel.statistics.count_full++;
                    if (type_package == 1)
                    {
                        if (correctTime&&!_items_counter[cell_history.time])
                        {
                            //Эту запись можно рассматривать как нормальный первый пакет
                            //фиксация
                            channel.statistics.count_all++;
                            channel.statistics.count_1++;
                            // channel.statistics.countPackage_type_1++;
                            var t = parseFloat(cell_history.temperature);
                            if (!isNaN(t))
                            {
                                summ_t += t;
                                count_t++;
                            }
                            
                            _items_counter[cell_history.time]=true;
                            //-----------------------------------------//
                            if (cell_history.time<minDate&&validSensor)
                            {
                                minDate = cell_history.time;
                                minS = cell_history.sensor_rate_sum;
                            }
                            if (cell_history.time>maxDate&&validSensor)
                            {
                                maxDate = cell_history.time;
                                maxS = cell_history.sensor_rate_sum;
                            }
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_time=cell_history.time;
                                if (validSensor)channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;
                                channel.statistics.last_release_date=cell_history.release_date;
                                channel.statistics.last_release_date_rus=moment.unix(channel.statistics.last_release_date).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_count_phase_1=cell_history.count_phase;
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_state_relay=cell_history.state_3;
                                channel.statistics.last_state_KK=cell_history.state_1;
                                channel.statistics.last_state_K=cell_history.state_2;
                                channel.statistics.last_relay=cell_history.relay_state;
                                channel.statistics.last_event=cell_history.event;
                            }
                            //ГРАФИК ТЕМПЕРАТУР
                            if (!_items_counter_all[cell_history.time])
                            {
                                channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                                _items_counter_all[cell_history.time]=true;
                            }
                            if (validSensor)
                            {
                                //ГРАФИК ПОКАЗАНИЙ
                                channel.statistics.chart.push( [cell_history.time*1000,cell_history.sensor_rate_sum,cell_history.event,cell_history.fcnt]);

                                channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,cell_history.event,cell_history.fcnt] );

                                channel.statistics.arr_value_consum.push( [cell_history.time*1000,undefined,undefined,undefined,undefined,cell_history.sensor_rate_sum,undefined,undefined]);
                            }
                            if (cell_history.event==19)
                            {
                                _item_counter[1] = cell_history.event;
                                _item_counter[2] = 2;
                            }
                            else if (cell_history.event!=1)
                            {
                                _item_counter[1] = cell_history.event;
                                _item_counter[2] = 3;
                                channel.statistics.count_danger++;
                            }
                            //ГРАФИК СОБЫТИЙ
                            arr_value_4.push(_item_counter);
                        }
                        else if(validTime&&!_items_counter_all[cell_history.time]&&validSensor)
                        {
                            _items_counter_all[cell_history.time]=true;
                            channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,cell_history.event,cell_history.fcnt] );
                        }
                    }
                    else if (type_package==4)
                    {
                        if (correctTime&&!_items_counter_2[cell_history.time])
                        {
                            // channel.statistics.countPackage_type_4++;
                            _items_counter_2[cell_history.time]=true;
                            if (validSensor)
                            {
                                if (cell_history.time>channel.statistics.last_time)
                                {
                                    channel.statistics.last_time=cell_history.time;
                                    channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;

                                }
                                var timeMS = cell_history.time*1000;
                                var sensor_rate_1 = cell_history.sensor_rate_1;
                                var sensor_rate_2 = cell_history.sensor_rate_2;
                                var sensor_rate_3 = cell_history.sensor_rate_3;
                                var sensor_rate_4 = cell_history.sensor_rate_4;
                                var sensor_rate_sum = cell_history.sensor_rate_sum;
                                var count_rate_active = cell_history.count_rate_active;
                                var rate_active = cell_history.rate_active;
                                var uuid = cell_history.UUID;
                                if (cell_history.time<minDate)
                                {
                                    minDate = cell_history.time;
                                    minS = cell_history.sensor_rate_sum;
                                }
                                if (cell_history.time>maxDate)
                                {
                                    maxDate = cell_history.time;
                                    maxS = cell_history.sensor_rate_sum;
                                }
                                if (!_items_counter_all[cell_history.time])
                                {
                                    channel.statistics.chart.push( [cell_history.time*1000,cell_history.sensor_rate_sum,0,cell_history.fcnt]);
                                    
                                    channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,0,cell_history.fcnt] );

                                    _items_counter_all[cell_history.time]=true;
                                }
                                channel.statistics.arr_value_consum.push( [timeMS,sensor_rate_1,sensor_rate_2,sensor_rate_3,sensor_rate_4,sensor_rate_sum,count_rate_active,rate_active]);
                                // if (uuid===undefined||uuid==='0000' )
                                // {
                                
                                    channel.statistics.arr_value_consum_v4.push( [timeMS,sensor_rate_1,sensor_rate_2,sensor_rate_3,sensor_rate_4,sensor_rate_sum,count_rate_active,rate_active]);
                                // }
                            }
                        }
                        else if(validTime&&!_items_counter_all[cell_history.time]&&validSensor)
                        {
                            _items_counter_all[cell_history.time]=true;
                            channel.statistics.chart_consumption.push( [cell_history.time*1000,cell_history.sensor_rate_sum,0,cell_history.fcnt] );
                        }
                    }
                    else if (type_package==5)
                    {
                        var history_p5 = cell_history;
                        var uuid = cell_history.UUID;
                        if ((correctDate1)&&(uuid===undefined||uuid==='0000' )&&!_items_counter_power[cell_history.date_1])
                        {
                            // channel.statistics.countPackage_type_5++;
                            _items_counter_power[cell_history.date_1]= true;
                            channel.statistics.arr_power_profile.push(history_p5);
                        }
                    }
                    else if (type_package==2)
                    {
                        if (correctTime&&!_items_counter_p_2[cell_history.time])
                        {
                            _items_counter_p_2[cell_history.time] = true;
                            // channel.statistics.countPackage_type_2++;
                        }
                    }
                }
                else if (mbus)
                {
                    //mbus
                    var type_package = cell_history.type;

                    var validTime = cell_history.time !== undefined && cell_history.time > 0;
                    var validTs = cell_history.ts !== undefined && cell_history.ts > 0;
                    var correctTime = validTime && cell_history.time >= date.getStart() && cell_history.time <= date.getEnd();
                    var correctTs = validTs && cell_history.ts >= ( date.getStart() * 1000 ) && cell_history.ts <= ( date.getEnd() * 1000 );
                    var validSensor = cell_history.sensor_rate_sum !== undefined && typeof cell_history.sensor_rate_sum === 'number';
                    var danger = cell_history.sensor_11;
                    var danger2 = cell_history.sensor_12;
                    var _item_counter = [cell_history.ts,0,0];
                    var timeMs = cell_history.time * 1000; 
                    channel.statistics.count_full++;

                    if ( type_package == 1 )
                    {
                        _item_counter[0] = timeMs;
                        var last = cell_history.time > channel.statistics.last_time;
                        
                        if ( typeof channel.statistics.countPackage_type_1 !== 'number' ) channel.statistics.countPackage_type_1 = 0;
                        channel.statistics.countPackage_type_1++;

                        if ( last )
                        {
                            channel.statistics.last_ts = cell_history.ts;
                            channel.statistics.last_time = cell_history.time;
                            channel.statistics.last_temperature = cell_history.temperature;
                            channel.statistics.last_temperature_2 = cell_history.temperature_2;
                            channel.statistics.last_charge = cell_history.charge;
                            channel.statistics.last_sensor_rate_sum = cell_history.sensor_rate_sum;
                            channel.statistics.last_total_heat_carrier_volume = cell_history.total_heat_carrier_volume;
                            channel.statistics.last_working_time_h = cell_history.working_time_h;
                            channel.statistics.last_current_coolant_flow = cell_history.current_coolant_flow;
                            channel.statistics.last_ts_rus = moment( cell_history.ts ).format( 'DD.MM.YY HH:mm' );
                            channel.statistics.last_time_rus = moment( timeMs ).format( 'DD.MM.YY HH:mm' );
                            channel.statistics.last_fcnt = cell_history.fcnt;
                            channel.statistics.last_cell_history = cell_history
                        }

                        if ( correctTime && !_items_counter[timeMs] )
                        {
                            _items_counter[timeMs] = true;
                            if ( cell_history.time < minDate )
                            {
                                minDate = cell_history.time;
                                minS = cell_history.sensor_rate_sum;
                            }
                            if ( cell_history.time > maxDate )
                            {
                                maxDate = cell_history.time;
                                maxS = cell_history.sensor_rate_sum;
                            }
                            if (validSensor)
                            {
                                channel.statistics.chart.push( [timeMs,cell_history.sensor_rate_sum,0] );
                                channel.statistics.chart_consumption.push( [timeMs,cell_history.sensor_rate_sum,0,cell_history.fcnt] );
                            }
                            channel.statistics.chart_temperature.push( [timeMs, cell_history.temperature, cell_history.temperature_2] );
                        }
                        else if(validTime&&!_items_counter[timeMs]&&validSensor)
                        {
                            _items_counter[timeMs]=true;
                            channel.statistics.chart_consumption.push( [timeMs,cell_history.sensor_rate_sum,0,cell_history.fcnt] );
                        }
                        
                    }
                    else if ( type_package == 3 && mbus1 )
                    {
                        if ( typeof channel.statistics.countPackage_type_3 !== 'number' ) channel.statistics.countPackage_type_3 = 0;
                        channel.statistics.countPackage_type_3++;
                    }
                    else if ( type_package == 4 && mbus1 )
                    {
                        if ( typeof channel.statistics.countPackage_type_4 !== 'number' ) channel.statistics.countPackage_type_4 = 0;
                        channel.statistics.countPackage_type_4++;
                    }
                    else if ( type_package == 5 && mbus1 )
                    {
                        channel.statistics.count_full++;
                        if ( typeof channel.statistics.countPackage_type_5 !== 'number' ) channel.statistics.countPackage_type_5 = 0;
                        channel.statistics.countPackage_type_5++;

                        if ( correctTs && !_items_counter[cell_history.ts] )
                        {
                            var last = cell_history.ts > channel.statistics.last_ts;
                            _items_counter[cell_history.ts] = true;
                            if ( channel.num_channel == 11 && cell_history.num_channel == 1 )
                            {
                                channel.statistics.count_danger++;
                                channel.statistics.chart.push( [cell_history.ts, danger, 1] );
                                channel.statistics.chart_consumption.push( [cell_history.ts, danger, 1,cell_history.fcnt] );
                                arr_value.push( [cell_history.ts,1]);
                            } 
                            if ( channel.num_channel == 12 && cell_history.num_channel == 2 ) 
                            {
                                channel.statistics.count_danger++;
                                channel.statistics.chart.push( [cell_history.ts, danger2, 1] );

                                channel.statistics.chart_consumption.push( [cell_history.ts, danger2, 1,cell_history.fcnt] );

                                arr_value.push( [cell_history.ts,1] );
                            }
                            if ( last )
                            {
                                channel.statistics.last_ts = cell_history.ts;
                                channel.statistics.last_charge = cell_history.charge;
                                channel.statistics.last_ts_rus = moment( cell_history.ts ).format( 'DD.MM.YY HH:mm' );
                                channel.statistics.last_fcnt = cell_history.fcnt;
                                channel.statistics.last_cell_history = cell_history
                            }
                        }
                        else if( validTs && !_items_counter[cell_history.ts] )
                        {
                            _items_counter[cell_history.ts] = true;
                            if ( channel.num_channel == 11 && cell_history.num_channel == 1 )
                            {
                                channel.statistics.chart_consumption.push( [cell_history.ts, danger, 1,cell_history.fcnt] );
                            } 
                            if ( channel.num_channel == 12 && cell_history.num_channel == 2 ) 
                            {
                                channel.statistics.chart_consumption.push( [cell_history.ts, danger2, 1,cell_history.fcnt] );
                            }
                        }
                    }
                    else if ( type_package == 6 && mbus1 )
                    {
                        if ( typeof channel.statistics.countPackage_type_6 !== 'number' ) channel.statistics.countPackage_type_6 = 0;
                        channel.statistics.countPackage_type_6++;
                        var valid_sensor_out_1_lastTime = last_sensor_out_1.time === null || cell_history.time > last_sensor_out_1.time;
                        var valid_sensor_out_2_lastTime = last_sensor_out_2.time === null || cell_history.time > last_sensor_out_2.time;
                        if (valid_sensor_out_1_lastTime && cell_history.num_out_channel == 1 )
                        {
                            last_sensor_out_1.ts = cell_history.ts;
                            last_sensor_out_1.time = cell_history.time;
                            last_sensor_out_1.value = cell_history.sensor_out_1;
                        }
                        if (valid_sensor_out_2_lastTime && cell_history.num_out_channel == 2)
                        {
                            last_sensor_out_2.ts = cell_history.ts;
                            last_sensor_out_2.time = cell_history.time;
                            last_sensor_out_2.value = cell_history.sensor_out_2;
                        }
                    }

                }
                else if (si13)
                {
                    //si13
                    var type_package = cell_history.type;
                    var validTime = cell_history.ts!==undefined&&cell_history.ts>0;
                    var correctTime = validTime&&cell_history.ts>=(date.getStart()*1000)&&cell_history.ts<=(date.getEnd()*1000);
                    var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                    var validSensor1 = cell_history.sensor_7!==undefined&&typeof cell_history.sensor_7 === 'number';
                    var validSensor2 = cell_history.sensor_8!==undefined&&typeof cell_history.sensor_8 === 'number';
                    var _item_counter = [cell_history.ts,0,0];
                    var port = cell_history.port;
                    channel.statistics.count_full++;
                    if (validTime&&!_items_counter[cell_history.ts]&&port==2)
                    {
                        channel.statistics.count_all++;
                        var ia;
                        var validMer = false;
                        if (cell_history.address!==undefined)
                        {
                            ia = parseInt(cell_history.address,16);
                            if (!isNaN(ia)&&ia==channel.ia)
                            {
                                validMer = true;
                            }
                        }
                        if (type_package == 4 && validMer)
                        {
                            channel.statistics.countPackage_type_4++;
                            if (cell_history.ts>channel.statistics.last_ts)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_ts=cell_history.ts;
                                channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                            }
                            if (correctTime&&!_items_counter[cell_history.ts])
                            {
                                _items_counter[cell_history.ts]=true;
                                if (cell_history.ts<minDate)
                                {
                                    minDate = cell_history.ts;
                                    minS = cell_history.sensor_rate_sum;
                                }
                                if (cell_history.ts>maxDate)
                                {
                                    maxDate = cell_history.ts;
                                    maxS = cell_history.sensor_rate_sum;
                                }
                                if (validSensor)
                                {
                                    //ГРАФИК ПОКАЗАНИЙ
                                    channel.statistics.chart.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_rate_sum,0] );
                                    channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_rate_sum,0,cell_history.fcnt] );
                                    channel.statistics.arr_value_consum_v4.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_rate_1,cell_history.sensor_rate_2,cell_history.sensor_rate_3,cell_history.sensor_rate_4,cell_history.sensor_rate_sum,undefined,undefined]);
                                }
                            }
                            else if( validTime && !_items_counter[cell_history.ts] && validSensor)
                            {
                                _items_counter[cell_history.ts]=true;
                                channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_rate_sum,0,cell_history.fcnt] );
                            }
                        }
                        else if (type_package == 1)
                        {
                            channel.statistics.countPackage_type_1++;
                            if (cell_history.ts>channel.statistics.last_ts)
                            {
                                channel.statistics.last_temperature=cell_history.temperature
                                channel.statistics.last_ts=cell_history.ts;
                                if (channel.num_channel==7)channel.statistics.last_sensor=cell_history.sensor_7;
                                if (channel.num_channel==8)channel.statistics.last_sensor=cell_history.sensor_8;
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                            }
                            if (correctTime&&!_items_counter[cell_history.ts])
                            {
                                _items_counter[cell_history.ts]=true;
                                if (cell_history.ts<minDate)
                                {
                                    minDate = cell_history.ts;
                                    if (channel.num_channel==7) minS = cell_history.sensor_7;
                                    if (channel.num_channel==8) minS = cell_history.sensor_8;
                                }
                                if (cell_history.ts>maxDate)
                                {
                                    maxDate = cell_history.ts;
                                    if (channel.num_channel==7) maxS = cell_history.sensor_7;
                                    if (channel.num_channel==8) maxS = cell_history.sensor_8;
                                }
                                    //ГРАФИК ПОКАЗАНИЙ
                                if (channel.num_channel==7&&validSensor1) channel.statistics.chart.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_7,0]);
                                if (channel.num_channel==8&&validSensor2) channel.statistics.chart.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_8,0]);

                                if (channel.num_channel==7&&validSensor1) channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_7,0,cell_history.fcnt]);
                                if (channel.num_channel==8&&validSensor2) channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_8,0,cell_history.fcnt]);
                                
                                if (cell_history.temperature!==undefined) channel.statistics.chart_temperature.push( [cell_history.ts,cell_history.temperature]);
                            }
                            else if( validTime && !_items_counter[cell_history.ts] )
                            {
                                _items_counter[cell_history.ts]=true;
                                if (channel.num_channel==7&&validSensor1) channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_7,0,cell_history.fcnt]);
                                if (channel.num_channel==8&&validSensor2) channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_8,0,cell_history.fcnt]);
                            }
                        }
                        else if (type_package == 2)
                        {
                            channel.statistics.countPackage_type_2++;
                            if (cell_history.ts>channel.statistics.last_ts)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_ts=cell_history.ts;
                                if (channel.num_channel==7)channel.statistics.last_sensor=cell_history.sensor_7;
                                if (channel.num_channel==8)channel.statistics.last_sensor=cell_history.sensor_8;
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                            }
                            var validNumChannel = (cell_history.num_channel+6)==channel.num_channel;
                            if (correctTime&&!_items_counter[cell_history.ts])
                            {
                                _items_counter[cell_history.ts]=true;
                                if (cell_history.ts<minDate)
                                {
                                    minDate = cell_history.ts;
                                    if (channel.num_channel==7) minS = cell_history.sensor_7;
                                    if (channel.num_channel==8) minS = cell_history.sensor_8;
                                }
                                if (cell_history.ts>maxDate)
                                {
                                    maxDate = cell_history.ts;
                                    if (channel.num_channel==7) maxS = cell_history.sensor_7;
                                    if (channel.num_channel==8) maxS = cell_history.sensor_8;
                                }
                                if (channel.num_channel==7&&validSensor1&&validNumChannel )
                                {
                                    channel.statistics.chart.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_7,1]);
                                    channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_7,1,cell_history.fcnt]);
                                    
                                    if (!arr_ts_list[cell_history.ts])
                                    {
                                        arr_ts_list[cell_history.ts] = true;
                                        channel.statistics.count_danger++;
                                    }
                                }
                                if (channel.num_channel==8&&validSensor2&&validNumChannel) 
                                {
                                    channel.statistics.chart.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_8,1]);
                                    channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_8,1,cell_history.fcnt]);
                                    
                                    if (!arr_ts_list[cell_history.ts])
                                    {
                                        arr_ts_list[cell_history.ts] = true;
                                        channel.statistics.count_danger++;
                                    }
                                }
                                arr_value.push( [cell_history.ts,1]);
                                
                            }
                            else if( validTime && !_items_counter[cell_history.ts] )
                            {
                                _items_counter[cell_history.ts]=true;
                                if (channel.num_channel==7&&validSensor1&&validNumChannel )
                                {
                                    channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_7,1,cell_history.fcnt]);
                                }
                                if (channel.num_channel==8&&validSensor2&&validNumChannel) 
                                {
                                    channel.statistics.chart_consumption.push( [cell_history.ts+(get_time_zone()*1000),cell_history.sensor_8,1,cell_history.fcnt]);
                                }
                            }
                        }
                    }
                }
                else if (si13rev2)
                {
                    //si13
                    var type_package = cell_history.type;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    var validSensor = cell_history.sensor_rate_sum!==undefined&&typeof cell_history.sensor_rate_sum === 'number';
                    var validSensor1 = cell_history.sensor_7!==undefined&&typeof cell_history.sensor_7 === 'number';
                    var validSensor2 = cell_history.sensor_8!==undefined&&typeof cell_history.sensor_8 === 'number';
                    var timeMS = validTime?cell_history.time*1000:undefined;
                    var _item_counter = [timeMS,0,0];
                    var port = 2;
                    channel.statistics.count_full++;
                    if (validTime&&!_items_counter[timeMS]&&port==2)
                    {
                        channel.statistics.count_all++;
                        var ia;
                        var validMer = false;
                        if (cell_history.address!==undefined)
                        {
                            ia = parseInt(cell_history.address,16);
                            if (!isNaN(ia)&&ia==channel.ia)
                            {
                                validMer = true;
                            }
                        }
                        if (type_package == 4 && validMer)
                        {
                            channel.statistics.countPackage_type_4++;
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_ts=cell_history.ts;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_sensor_rate_sum=cell_history.sensor_rate_sum;
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                            }
                            if (correctTime&&!_items_counter[timeMS])
                            {
                                _items_counter[timeMS]=true;
                                if (timeMS<minDate)
                                {
                                    minDate = timeMS;
                                    minS = cell_history.sensor_rate_sum;
                                }
                                if (timeMS>maxDate)
                                {
                                    maxDate = timeMS;
                                    maxS = cell_history.sensor_rate_sum;
                                }
                                if (validSensor)
                                {
                                    //ГРАФИК ПОКАЗАНИЙ
                                    channel.statistics.chart.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_rate_sum,0]);

                                    channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_rate_sum,0,cell_history.fcnt]);
                                    
                                    channel.statistics.arr_value_consum_v4.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_rate_1,cell_history.sensor_rate_2,cell_history.sensor_rate_3,cell_history.sensor_rate_4,cell_history.sensor_rate_sum,undefined,undefined]);
                                }
                            }
                            else if( validTime && !_items_counter[timeMS] && validSensor )
                            {
                                _items_counter[timeMS]=true;
                                channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_rate_sum,0,cell_history.fcnt]);
                            }
                        }
                        else if (type_package == 1)
                        {
                            channel.statistics.countPackage_type_1++;
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature
                                channel.statistics.last_ts=cell_history.ts;
                                channel.statistics.last_time=cell_history.time;
                                if (channel.num_channel==7)channel.statistics.last_sensor=cell_history.sensor_7;
                                if (channel.num_channel==8)channel.statistics.last_sensor=cell_history.sensor_8;
                                channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                            }
                            if (correctTime&&!_items_counter[timeMS])
                            {
                                _items_counter[timeMS]=true;
                                if (timeMS<minDate)
                                {
                                    minDate = timeMS;
                                    if (channel.num_channel==7) minS = cell_history.sensor_7;
                                    if (channel.num_channel==8) minS = cell_history.sensor_8;
                                }
                                if (timeMS>maxDate)
                                {
                                    maxDate = timeMS;
                                    if (channel.num_channel==7) maxS = cell_history.sensor_7;
                                    if (channel.num_channel==8) maxS = cell_history.sensor_8;
                                }
                                    //ГРАФИК ПОКАЗАНИЙ
                                if (channel.num_channel==7&&validSensor1) channel.statistics.chart.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_7,0]);
                                if (channel.num_channel==8&&validSensor2) channel.statistics.chart.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_8,0]);

                                if (channel.num_channel==7&&validSensor1) channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_7,0,cell_history.fcnt]);
                                if (channel.num_channel==8&&validSensor2) channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_8,0,cell_history.fcnt]);

                                
                                if (cell_history.temperature!==undefined) channel.statistics.chart_temperature.push( [timeMS,cell_history.temperature]);
                            }
                            else if( validTime && !_items_counter[timeMS] )
                            {
                                _items_counter[timeMS]=true;
                                if (channel.num_channel==7&&validSensor1) channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_7,0,cell_history.fcnt]);
                                if (channel.num_channel==8&&validSensor2) channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_8,0,cell_history.fcnt]);
                            }
                        }
                        else if (type_package == 2)
                        {
                            channel.statistics.countPackage_type_2++;
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_time=cell_history.time;
                                if (channel.num_channel==7)channel.statistics.last_sensor=cell_history.sensor_7;
                                if (channel.num_channel==8)channel.statistics.last_sensor=cell_history.sensor_8;
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                            }
                            var validNumChannel = (cell_history.num_channel+6)==channel.num_channel;
                            if (correctTime&&!_items_counter[timeMS])
                            {
                                _items_counter[timeMS]=true;
                                if (timeMS<minDate)
                                {
                                    minDate = timeMS;
                                    if (channel.num_channel==7) minS = cell_history.sensor_7;
                                    if (channel.num_channel==8) minS = cell_history.sensor_8;
                                }
                                if (timeMS>maxDate)
                                {
                                    maxDate = timeMS;
                                    if (channel.num_channel==7) maxS = cell_history.sensor_7;
                                    if (channel.num_channel==8) maxS = cell_history.sensor_8;
                                }
                                if (channel.num_channel==7&&validSensor1&&validNumChannel) 
                                {
                                    channel.statistics.chart.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_7,1]);
                                    channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_7,1,cell_history.fcnt]);

                                    
                                    if (!arr_ts_list[timeMS])
                                    {
                                        arr_ts_list[timeMS] = true;
                                        channel.statistics.count_danger++;
                                    }
                                }
                                if (channel.num_channel==8&&validSensor2&&validNumChannel) 
                                {
                                    channel.statistics.chart.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_8,1]);
                                    channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_8,1,cell_history.fcnt]);

                                    if (!arr_ts_list[timeMS])
                                    {
                                        arr_ts_list[timeMS] = true;
                                        channel.statistics.count_danger++;
                                    }
                                }
                                arr_value.push( [timeMS,1]);
                            }
                            else if( validTime && !_items_counter[timeMS] )
                            {
                                _items_counter[timeMS]=true;
                                if (channel.num_channel==7&&validSensor1&&validNumChannel) 
                                {
                                    channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_7,1,cell_history.fcnt]);
                                }
                                if (channel.num_channel==8&&validSensor2&&validNumChannel) 
                                {
                                    channel.statistics.chart_consumption.push( [timeMS+(get_time_zone()*1000),cell_history.sensor_8,1,cell_history.fcnt]);
                                }
                            }
                        }
                        else if( type_package == 6 )
                        {
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_ts=cell_history.ts;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                            }
                        }
                    }
                    if(correctTime)
                    {
                        arr_date.push(moment(timeMS).format('LLL' ).replace(',',' ' ));
                    }
                }
                else if ( si21 || si22 || si11 || si12 ) 
                {
                    var type_package = cell_history.type;
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var validTs = cell_history.ts!==undefined&&cell_history.ts>0;
                    var timeMS = validTime?cell_history.time*1000:undefined;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                    var port = cell_history.port;
                    channel.statistics.count_full++;
                    if(validTs&&!_items_counter[timeMS]&&port==2)
                    {
                        _items_counter[timeMS]=true;
                        channel.statistics.count_all++;
                        if (type_package == 1 && correctTime)
                        {
                            channel.statistics.countPackage_type_1++;
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_charge=cell_history.charge;
                                channel.statistics.last_ts=cell_history.ts;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_sensor= cell_history.sensor;
                                channel.statistics.last_sensor_1 = cell_history.sensor_1;
                                channel.statistics.last_sensor_2 = cell_history.sensor_2;
                                channel.statistics.last_sensor_3 = cell_history.sensor_3;
                                channel.statistics.last_sensor_4 = cell_history.sensor_4;
                                channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                            }
                        }
                        else if (type_package == 2 && correctTime)
                        {
                            channel.statistics.countPackage_type_2++;
                            if (cell_history.time>channel.statistics.last_time)
                            {
                                channel.statistics.last_temperature=cell_history.temperature;
                                channel.statistics.last_time=cell_history.time;
                                channel.statistics.last_sensor= cell_history.sensor;
                                channel.statistics.last_charge=cell_history.charge;
                                channel.statistics.last_sensor_1 = cell_history.sensor_1;
                                channel.statistics.last_sensor_2 = cell_history.sensor_2;
                                channel.statistics.last_sensor_3 = cell_history.sensor_3;
                                channel.statistics.last_sensor_4 = cell_history.sensor_4;
                                channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                                channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                            }
                            if (!arr_ts_list[timeMS] && channel.num_channel == cell_history.num_channel && correctTime)
                            {
                                arr_ts_list[timeMS] = true;
                                channel.statistics.count_danger++;
                                arr_value.push( [timeMS,1]);
                            }
                        }
                        else if(cell_history.type==4 && si12 && correctTime)
                        {
                            var valid_state_energy = last_state_energy.time === null || cell_history.time > last_state_energy.time;
                            if(valid_state_energy)
                            {
                                last_state_energy.ts = cell_history.ts;
                                last_state_energy.time = cell_history.time;
                                last_state_energy.value = cell_history.state_energy;
                            }
                        }
                        else if(cell_history.type==5 && si12 && correctTime)
                        {
                            var valid_sensor_out_1 = typeof cell_history.sensor_out_1 === 'boolean';
                            var valid_sensor_out_2 = typeof cell_history.sensor_out_2 === 'boolean';
                            var valid_sensor_out_1_lastTime = last_sensor_out_1.time === null || cell_history.time > last_sensor_out_1.time;
                            var valid_sensor_out_2_lastTime = last_sensor_out_2.time === null || cell_history.time > last_sensor_out_2.time;
                            if (valid_sensor_out_1_lastTime && valid_sensor_out_1)
                            {
                                last_sensor_out_1.ts = cell_history.ts;
                                last_sensor_out_1.time = cell_history.time;
                                last_sensor_out_1.value = cell_history.sensor_out_1;
                            }
                            if (valid_sensor_out_2_lastTime && valid_sensor_out_2)
                            {
                                last_sensor_out_2.ts = cell_history.ts;
                                last_sensor_out_2.time = cell_history.time;
                                last_sensor_out_2.value = cell_history.sensor_out_2;
                            }
                        }
                        if (cell_history.ts>channel.statistics.last_ts)
                        {
                            channel.statistics.last_ts=cell_history.ts;
                            channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                        }
                    }
                }
                else if ( tp11 )
                {
                    //tp
                    var validTs = cell_history.ts!==undefined && typeof cell_history.ts == 'number' && cell_history.ts>0 ;
                    var validTime = cell_history.ts > ( date.getStart() * 1000 ) && cell_history.ts < ( date.getEnd() * 1000 );
                    if(validTime && !_items_all[cell_history.ts])
                    {
                        var _item = [cell_history.ts, 1, 0];
                        channel.statistics.count_all++;
                        _items_all[cell_history.ts]=true;
                        if (cell_history.type==1)
                        {
                            var valid_sensor_out_1_lastTs=last_sensor_out_1.ts===null||cell_history.ts>last_sensor_out_1.ts;
                            var valid_sensor_out_2_lastTs=last_sensor_out_2.ts===null||cell_history.ts>last_sensor_out_2.ts;
                            var valid_charge_lastTs=last_charge.ts===null||cell_history.ts>last_charge.ts;
                            var valid_type_1_lastTs=last_charge.ts===null||cell_history.ts>last_type_1.ts;
                            channel.statistics.chart_temperature.push( [cell_history.ts,cell_history.temperature]);
                            var checkEvent = false;
                            var sensorEvent = false;
                            var dangerEvent = cell_history.sensor_danger_1||cell_history.sensor_danger_2?true:false;
                            var min_v = parseFloat(channel.min_v);
                            var max_v = parseFloat(channel.max_v);
                            var min = parseFloat(channel.min_normal_v);
                            var max = parseFloat(channel.max_normal_v);
                            var value = cell_history.sensorTP;
                            if (!isNaN(min_v)&&!isNaN(max_v)&&!isNaN(min)&&!isNaN(max))
                            {
                                if (value===0)
                                {
                                    sensorEvent=true;
                                }
                                else if (value<=25&&value>=2)
                                {
                                    var newvalue = my_converter.parse4_20mA(value,min_v,max_v);
                                    if (typeof newvalue === 'number' )
                                    {
                                        if (newvalue<=min||newvalue>=max)
                                        {
                                            sensorEvent = true;
                                        }
                                    }
                                }
                                else
                                {
                                    sensorEvent=true;
                                }
                            }
                            if (sensorEvent&&dangerEvent)
                            {
                                _dangers[cell_history.ts]=true;
                                channel.statistics.count_danger++;
                                _item[2] = 3;
                                _item[3] = 'sensorAndDanger';
                            }
                            else if (sensorEvent)
                            {
                                _dangers[cell_history.ts]=true;
                                channel.statistics.count_danger++;
                                _item[2] = 2;
                                _item[3] = 'sensor';
                            }
                            else if (dangerEvent)
                            {
                                _dangers[cell_history.ts]=true;
                                channel.statistics.count_danger++;
                                _item[2] = 1;
                                _item[3] = 'danger';
                            }
                            if (cell_history.sensorTP>=2&&cell_history.sensorTP<=25&&!_items[cell_history.ts])
                            {
                                _items[cell_history.ts]=true;
                                channel.statistics.count++; 
                                channel.statistics.chart.push( [convert_time(moment(cell_history.ts).unix()),cell_history.sensorTP]);
                                channel.statistics.chart_consumption.push( [convert_time(moment(cell_history.ts).unix()),cell_history.sensorTP,0,cell_history.fcnt]);

                                
                                summ_sensorTP += cell_history.sensorTP;
                                count_sensorTP++;
                            }
                            if (valid_charge_lastTs)
                            {
                                last_charge.ts = cell_history.ts;
                                last_charge.value = cell_history.charge;
                            }
                            if (valid_sensor_out_1_lastTs)
                            {
                                last_sensor_out_1.time = cell_history.time;
                                last_sensor_out_1.ts = cell_history.ts;
                                last_sensor_out_1.value = cell_history.sensor_out_1;
                            }
                            if (valid_sensor_out_2_lastTs)
                            {
                                last_sensor_out_2.ts = cell_history.ts;
                                last_sensor_out_2.time = cell_history.time;
                                last_sensor_out_2.value = cell_history.sensor_out_2;
                            }
                            if (valid_type_1_lastTs)
                            {
                                last_type_1=cell_history;
                            }
                        }
                        else if (cell_history.type==5)
                        {
                            var valid_sensor_out_1=typeof cell_history.sensor_out_1 === 'boolean';
                            var valid_sensor_out_2=typeof cell_history.sensor_out_2 === 'boolean';
                            var valid_sensor_out_1_lastTs=last_sensor_out_1.ts===null||cell_history.ts>last_sensor_out_1.ts;
                            var valid_sensor_out_2_lastTs=last_sensor_out_2.ts===null||cell_history.ts>last_sensor_out_2.ts;
                            var valid_charge_lastTs=last_charge.ts===null||cell_history.ts>last_charge.ts;
                            if (valid_charge_lastTs)
                            {
                                last_charge.ts = cell_history.ts;
                                last_charge.value = cell_history.charge;
                            }
                            if (valid_sensor_out_1&&valid_sensor_out_1_lastTs)
                            {
                                last_sensor_out_1.ts = cell_history.ts;
                                last_sensor_out_1.time = cell_history.time;
                                last_sensor_out_1.value = cell_history.sensor_out_1;
                            }
                            else if (valid_sensor_out_2&&valid_sensor_out_2_lastTs)
                            {
                                last_sensor_out_2.ts = cell_history.ts;
                                last_sensor_out_2.time = cell_history.time;
                                last_sensor_out_2.value = cell_history.sensor_out_2;
                            }
                        }
                        arr_value_4.push(_item);
                    }
                    else if( validTs && !_items_all[cell_history.ts] && cell_history.type == 1 && cell_history.sensorTP>=2 && cell_history.sensorTP<=25 )
                    {
                        _items_all[cell_history.ts]=true;
                        channel.statistics.chart_consumption.push( [convert_time(moment(cell_history.ts).unix()),cell_history.sensorTP,0,cell_history.fcnt]);
                    }
                }
                else if ( tp11rev2 )
                {
                    //tp
                    var vTime = cell_history.time !== undefined && typeof cell_history.time == 'number' && cell_history.time > 0;
                    var validTime = cell_history.time > date.getStart() && cell_history.time < date.getEnd();
                    if(validTime && !_items_all[cell_history.time])
                    {
                        channel.statistics.count_all++;
                        _items_all[cell_history.time] = true;
                        var timeMs = cell_history.time * 1000;
                        if ( cell_history.type == 1 )
                        {
                            var reason = cell_history.reason;
                            var _item = [cell_history.time, 1 ,reason];
                            
                            // var valid_sensor_out_1 = typeof cell_history.sensor_out_1 === 'boolean';
                            // var valid_sensor_out_2 = typeof cell_history.sensor_out_2 === 'boolean';

                            var valid_sensor_out_1_lastTime = last_sensor_out_1.time === null || cell_history.time > last_sensor_out_1.time;
                            var valid_sensor_out_2_lastTime = last_sensor_out_2.time === null || cell_history.time > last_sensor_out_2.time;
                            var valid_charge_lastTime = last_charge.time === null || cell_history.time > last_charge.time;
                            var valid_type_1_lastTime = last_charge.time === null || cell_history.time > last_type_1.time;
                            channel.statistics.chart_temperature.push( [cell_history.time * 1000, cell_history.temperature]);
                            var checkEvent = false;
                            var sensorEvent = false;
                            var dangerEvent = cell_history.sensor_danger_1||cell_history.sensor_danger_2?true:false;
                            var min_v = parseFloat(channel.min_v);
                            var max_v = parseFloat(channel.max_v);
                            var value = cell_history.sensorTP;
                            if(reason != 0 && reason != 5)
                            {
                                 _dangers[cell_history.time] = true;
                                 channel.statistics.count_danger++;
                            }
                            if (cell_history.sensorTP >= 2 && cell_history.sensorTP <= 25 && !_items[cell_history.time] )
                            {
                                _items[cell_history.time]=true;
                                channel.statistics.count++; 
                                channel.statistics.chart.push( [convert_time(cell_history.time),cell_history.sensorTP]);
                                channel.statistics.chart_consumption.push( [convert_time(cell_history.time),cell_history.sensorTP,0,cell_history.fcnt]);

                                
                                channel.statistics.chart_sensor_new.push( [timeMs+(tz*1000),cell_history.sensorTP,cell_history.min_sensor,cell_history.max_sensor, cell_history.limit_exceeded]);
                                summ_sensorTP += cell_history.sensorTP;
                                count_sensorTP++;
                            }
                            if (valid_charge_lastTime)
                            {
                                last_charge.time = cell_history.time;
                                last_charge.value = cell_history.charge;
                            }
                            if (valid_sensor_out_1_lastTime)
                            {
                                last_sensor_out_1.ts = cell_history.ts;
                                last_sensor_out_1.time = cell_history.time;
                                last_sensor_out_1.value = cell_history.sensor_out_1;
                            }
                            if (valid_sensor_out_2_lastTime)
                            {
                                last_sensor_out_2.ts = cell_history.ts;
                                last_sensor_out_2.time = cell_history.time;
                                last_sensor_out_2.value = cell_history.sensor_out_2;
                            }
                            if (valid_type_1_lastTime)
                            {
                                last_type_1 = cell_history;
                            }
                            arr_value_4.push(_item);
                        }
                        else if ( cell_history.type == 5 )
                        {
                            var valid_sensor_out_1 = typeof cell_history.sensor_out_1 === 'boolean';
                            var valid_sensor_out_2 = typeof cell_history.sensor_out_2 === 'boolean';
                            var valid_sensor_out_1_lastTime = last_sensor_out_1.time === null || cell_history.time > last_sensor_out_1.time;
                            var valid_sensor_out_2_lastTime = last_sensor_out_2.time === null || cell_history.time > last_sensor_out_2.time;
                            var valid_charge_lastTime = last_charge.time === null || cell_history.time > last_charge.time;
                            if ( valid_charge_lastTime )
                            {
                                last_charge.time = cell_history.time;
                                last_charge.value = cell_history.charge;
                            }
                            if ( valid_sensor_out_1 && valid_sensor_out_1_lastTime )
                            {
                                last_sensor_out_1.ts = cell_history.ts;
                                last_sensor_out_1.time = cell_history.time;
                                last_sensor_out_1.value = cell_history.sensor_out_1;
                            }
                            else if ( valid_sensor_out_2 && valid_sensor_out_2_lastTime)
                            {
                                last_sensor_out_2.ts = cell_history.ts;
                                last_sensor_out_2.time = cell_history.time;
                                last_sensor_out_2.value = cell_history.sensor_out_2;
                            }
                        }
                    }
                    else if( vTime && !_items_all[cell_history.time] && cell_history.type == 1 && cell_history.sensorTP>=2 && cell_history.sensorTP<=25 )
                    {
                        _items_all[cell_history.time] = true;
                        channel.statistics.chart_consumption.push( [convert_time(cell_history.time),cell_history.sensorTP,0,cell_history.fcnt]);
                    }
                }
                else if ( impulse && cell_history.type==1 )
                {
                    var validTimeHistory = cell_history.time>channel.statistics.last_time;
                    if (validTimeHistory)
                    {
                        channel.statistics.last_temperature=cell_history.temperature;
                        channel.statistics.last_charge=cell_history.charge;
                        channel.statistics.last_time=cell_history.time;
                        channel.statistics.last_fcnt=cell_history.fcnt;
                        channel.statistics.last_sensor= cell_history.sensor;
                        channel.statistics.last_sensor_1 = cell_history.sensor_1;
                        channel.statistics.last_sensor_2 = cell_history.sensor_2;
                        channel.statistics.last_sensor_3 = cell_history.sensor_3;
                        channel.statistics.last_sensor_4 = cell_history.sensor_4;
                    }
                    if (cell_history.ts > channel.statistics.last_ts)
                    {
                        channel.statistics.last_ts = cell_history.ts;
                    }
                }
                else if ( hs )
                {
                      var validTime = cell_history.time!==undefined&&cell_history.time>0;
                      var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                      var timeMS = validTime?cell_history.time*1000:undefined;
                      tz = get_time_zone() * 1000;
                      if(validTime && !_items[timeMS])
                      {
                        _items[timeMS]=true;
                        var reason = cell_history.reason;
                        var _item = [timeMS,1,0];
                        var validTemperature = cell_history.temperature!==undefined;
                        var validDamp = cell_history.damp!==undefined;
                        var validDataTemperature = validTemperature && cell_history.min_temperature !== undefined && cell_history.max_temperature !== undefined;
                        var validDataDamp = validDamp && cell_history.min_sensor !== undefined && cell_history.max_sensor !== undefined;
                        var dangerTemperature = validDataTemperature && (cell_history.min_temperature > cell_history.temperature || cell_history.max_temperature < cell_history.temperature);
                        var dangerDamp =  validDataDamp && ( cell_history.min_sensor > cell_history.damp || cell_history.max_sensor < cell_history.damp );
                        var limit_exceeded_temperature = dangerTemperature || cell_history.reason == 6;
                        var limit_exceeded_damp = dangerDamp || cell_history.reason == 5;
                        channel.statistics.count++;

                        if(correctTime)
                        {
                            if (cell_history.reason!=undefined&&parseInt(cell_history.reason)>1)
                            {
                                channel.statistics.count_danger++;
                            }
                            if (reason!==undefined)
                            {
                                reason = parseInt(reason);
                                if (!isNaN(reason))
                                {
                                    _item[2]=reason;
                                }
                            }
                            arr_value_4.push(_item);
                            channel.statistics.chart_temperature.push( [timeMS,cell_history.temperature]);
                            channel.statistics.chart_temperature_new.push( [timeMS+tz,cell_history.temperature,cell_history.min_temperature,cell_history.max_temperature, limit_exceeded_temperature]);
                            channel.statistics.chart_sensor_new.push( [timeMS+tz,cell_history.damp,cell_history.min_sensor,cell_history.max_sensor, limit_exceeded_damp]);
                        }
                        
                        
                        
                        if (cell_history.ts>channel.statistics.last_ts)
                        {
                            channel.statistics.last_ts=cell_history.ts;
                            channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                        }
                        if (cell_history.time>channel.statistics.last_time)
                        {
                            channel.statistics.last_min_temperature = cell_history.min_temperature;
                            channel.statistics.last_max_temperature = cell_history.max_temperature;
                            channel.statistics.last_min_sensor = cell_history.min_sensor;
                            channel.statistics.last_max_sensor = cell_history.max_sensor;
                            channel.statistics.last_sensor_in_1 = cell_history.sensor_in_1;
                            channel.statistics.last_sensor_in_2 = cell_history.sensor_in_2;
                            channel.statistics.last_angle=cell_history.angle;
                            channel.statistics.last_damp=cell_history.damp;
                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_fcnt=cell_history.fcnt; 
                            channel.statistics.last_reason = cell_history.reason;  
                            channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                            channel.statistics.last_time=cell_history.time;
                        }
                    }
                }
                else if ( um )
                {
                      //надо переделать
                      //   result.last_lux = cell_history.lux;
                      //   result.last_dB = cell_history.dB;
                      //   result.last_CO2 = cell_history.CO2;
                      var validTime = cell_history.time!==undefined&&cell_history.time>0;
                      var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();
                      var timeMS = validTime?cell_history.time*1000:undefined;
                      tz = get_time_zone() * 1000;
                      if(validTime && !_items[timeMS])
                      {
                        _items[timeMS]=true;
                        var reason = cell_history.reason;
                        var _item = [timeMS,1,0];
                        var validTemperature = cell_history.temperature!==undefined;
                        var validDamp = cell_history.damp!==undefined;

                        var validLux = cell_history.lux!==undefined;
                        var validDB = cell_history.dB!==undefined;
                        var validCO2 = cell_history.CO2!==undefined;

                        var validDataTemperature = validTemperature && cell_history.min_temperature !== undefined && cell_history.max_temperature !== undefined;
                        var validDataDamp = validDamp && cell_history.min_damp !== undefined && cell_history.max_damp !== undefined;

                        var validDataLux = validLux && cell_history.min_lux !== undefined && cell_history.max_lux !== undefined;
                        var validDataDB = validDB && cell_history.min_dB !== undefined && cell_history.max_dB !== undefined;
                        var validDataCO2 = validCO2 && cell_history.min_CO2 !== undefined && cell_history.max_CO2 !== undefined;

                        var dangerTemperature = validDataTemperature && (cell_history.min_temperature > cell_history.temperature || cell_history.max_temperature < cell_history.temperature);
                        var dangerDamp =  validDataDamp && ( cell_history.min_damp > cell_history.damp || cell_history.max_damp < cell_history.damp );

                        var dangerLux =  validDataLux && ( cell_history.min_lux > cell_history.lux || cell_history.max_lux < cell_history.lux );
                        var dangerDB =  validDataDB && ( cell_history.min_dB > cell_history.dB || cell_history.max_dB < cell_history.dB );
                        var dangerCO2 =  validDataCO2 && ( cell_history.min_CO2 > cell_history.CO2 || cell_history.max_CO2 < cell_history.CO2 );

                        var limit_exceeded_temperature = dangerTemperature || cell_history.reason == 6;
                        var limit_exceeded_damp = dangerDamp || cell_history.reason == 5;

                        var limit_exceeded_lux = dangerLux || cell_history.reason == 3;
                        var limit_exceeded_dB = dangerDB || cell_history.reason == 7;
                        var limit_exceeded_CO2 = dangerCO2 || cell_history.reason == 2;
                        channel.statistics.count++;

                        if(correctTime)
                        {
                            if (cell_history.reason!=undefined&&parseInt(cell_history.reason)>1)
                            {
                                channel.statistics.count_danger++;
                            }
                            if (reason!==undefined)
                            {
                                reason = parseInt(reason);
                                if (!isNaN(reason))
                                {
                                    _item[2]=reason;
                                }
                            }
                            arr_value_4.push(_item);
                            //channel.statistics.chart_temperature.push( [timeMS,cell_history.temperature]);
                            channel.statistics.chart_temperature_new.push( [timeMS+tz,cell_history.temperature,cell_history.min_temperature,cell_history.max_temperature, limit_exceeded_temperature]);

                            channel.statistics.chart_damp_new.push( [timeMS+tz,cell_history.damp,cell_history.min_damp,cell_history.max_damp, limit_exceeded_damp]);
                            channel.statistics.chart_lux_new.push( [timeMS+tz,cell_history.lux,cell_history.min_lux,cell_history.max_lux, limit_exceeded_lux]);
                            channel.statistics.chart_dB_new.push( [timeMS+tz,cell_history.dB,cell_history.min_dB,cell_history.max_dB, limit_exceeded_dB]);
                            channel.statistics.chart_CO2_new.push( [timeMS+tz,cell_history.CO2,cell_history.min_CO2,cell_history.max_CO2, limit_exceeded_CO2]);

                            //channel.statistics.chart_sensor_new.push( [timeMS+tz,cell_history.damp,cell_history.min_sensor,cell_history.max_sensor, limit_exceeded_damp]);

                            // channel.statistics.chart_lux.push( [timeMS+tz,cell_history.lux,cell_history.min_sensor,cell_history.max_sensor, limit_exceeded_damp]);
                        }
                        
                        
                        
                        if (cell_history.ts>channel.statistics.last_ts)
                        {
                            channel.statistics.last_ts=cell_history.ts;
                            channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                        }
                        if (cell_history.time>channel.statistics.last_time)
                        {
                            channel.statistics.last_min_temperature = cell_history.min_temperature;
                            channel.statistics.last_max_temperature = cell_history.max_temperature;

                            channel.statistics.last_min_damp= cell_history.min_damp;
                            channel.statistics.last_max_damp = cell_history.max_damp;
                            
                            channel.statistics.last_min_lux = cell_history.min_lux;
                            channel.statistics.last_max_lux = cell_history.max_lux;

                            channel.statistics.last_min_dB = cell_history.min_dB;
                            channel.statistics.last_max_dB = cell_history.max_dB;
                            
                            channel.statistics.last_min_CO2 = cell_history.min_CO2;
                            channel.statistics.last_max_CO2 = cell_history.max_CO2;

                            channel.statistics.last_min_sensor = cell_history.min_sensor;
                            channel.statistics.last_max_sensor = cell_history.max_sensor;
                            channel.statistics.last_sensor_in_1 = cell_history.sensor_in_1;
                            channel.statistics.last_sensor_in_2 = cell_history.sensor_in_2;
                            channel.statistics.last_angle=cell_history.angle;
                            channel.statistics.last_damp=cell_history.damp;

                            channel.statistics.last_lux=cell_history.lux;
                            channel.statistics.last_dB=cell_history.dB;
                            channel.statistics.last_CO2=cell_history.CO2;

                            channel.statistics.last_temperature=cell_history.temperature;
                            channel.statistics.last_charge=cell_history.charge;
                            channel.statistics.last_fcnt=cell_history.fcnt; 
                            channel.statistics.last_reason = cell_history.reason;  
                            channel.statistics.last_time_rus=moment(timeMS).format('DD.MM.YY HH:mm' );
                            channel.statistics.last_time=cell_history.time;
                        }
                    }
                }
                else if (!_items[cell_history.ts]&&smart&&cell_history.type==1)
                {
                    //smart
                    var checkValidTs=cell_history.ts>(date.getStart()*1000)&&cell_history.ts<(date.getEnd()*1000);
                    if (checkValidTs)
                    {
                        _items[cell_history.ts]=true;
                        var reason = cell_history.reason;
                        var _item = [cell_history.ts,1,0];
                        channel.statistics.count++;
                        if (cell_history.reason!=undefined&&parseInt(cell_history.reason)!==0)
                        {
                            channel.statistics.count_danger++;
                        }
                        if (reason)
                        {
                            reason = parseInt(reason);
                            if (!isNaN(reason)&&reason>0)
                            {
                                _item[2]=reason;
                            }
                        }
                        arr_value_4.push(_item);
                        channel.statistics.chart_temperature.push( [cell_history.ts,cell_history.temperature]);
                    }
                    if (cell_history.ts>channel.statistics.last_ts)
                    {
                        channel.statistics.last_temperature=cell_history.temperature;
                        channel.statistics.last_charge=cell_history.charge;
                        channel.statistics.last_ts=cell_history.ts;
                        channel.statistics.last_fcnt=cell_history.fcnt; 
                        channel.statistics.last_period_connect_minute = cell_history.period_connect_minute;  
                        channel.statistics.last_reason = cell_history.reason;  
                        channel.statistics.last_state_ack = cell_history.state_ack;  
                        channel.statistics.last_state_sensor_0 = cell_history.state_sensor_0; 
                        channel.statistics.last_state_sensor_1 = cell_history.state_sensor_1;
                        channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                    }
                }
                else if (!_items[cell_history.time]&&smartRev2&&cell_history.type==1)
                {
                    //smart
                    var checkValidTime=cell_history.time>(date.getStart())&&cell_history.time<(date.getEnd());
                    if (checkValidTime)
                    {
                        _items[cell_history.time]=true;
                        var reason = cell_history.reason;
                        var _item = [cell_history.time*1000,1,0];
                        channel.statistics.count++;
                        if (cell_history.reason!=undefined&&parseInt(cell_history.reason)!==0)
                        {
                            channel.statistics.count_danger++;
                        }
                        if (reason)
                        {
                            reason = parseInt(reason);
                            if (!isNaN(reason)&&reason>0)
                            {
                                _item[2]=reason;
                            }
                        }
                        arr_value_4.push(_item);
                        channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                    }
                    if (cell_history.ts>channel.statistics.last_ts)
                    {
                        channel.statistics.last_ts=cell_history.ts;
                        channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                    }
                    if (cell_history.time>channel.statistics.last_time)
                    {
                        channel.statistics.last_angle=cell_history.angle;
                        channel.statistics.last_temperature=cell_history.temperature;
                        channel.statistics.last_charge=cell_history.charge;
                        channel.statistics.last_time=cell_history.time;
                        channel.statistics.last_fcnt=cell_history.fcnt; 
                        channel.statistics.last_period_connect_minute = cell_history.period_connect_minute;  
                        channel.statistics.last_reason = cell_history.reason;  
                        channel.statistics.last_state_ack = cell_history.state_ack;  
                        channel.statistics.last_state_sensor_0 = cell_history.state_sensor_0; 
                        channel.statistics.last_state_sensor_1 = cell_history.state_sensor_1;
                        channel.statistics.last_time_rus=moment(cell_history.time*1000).format('DD.MM.YY HH:mm' );
                    }
                }
                else if ( cell_history.type==2 && channelDanger )
                {
                    channel.statistics.count_danger_full++;

                    
                    var validTime = cell_history.time!==undefined&&cell_history.time>0;
                    var timeMs = validTime&&cell_history.time * 1000;
                    var correctTime = validTime&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd();

                    if (cell_history.ts>channel.statistics.last_ts)
                    {
                        channel.statistics.last_ts=cell_history.ts;
                    }
                    if (cell_history.time>channel.statistics.last_time)
                    {
                        channel.statistics.last_time=cell_history.time;
                        channel.statistics.last_charge=cell_history.charge;
                        channel.statistics.last_sensor=cell_history.sensor;
                        channel.statistics.last_sensor_1 = cell_history.sensor_1;
                        channel.statistics.last_sensor_2 = cell_history.sensor_2;
                        channel.statistics.last_sensor_3 = cell_history.sensor_3;
                        channel.statistics.last_sensor_4 = cell_history.sensor_4;
                    }
                    
                    if (!arr_ts_list[timeMs] && channel.num_channel == cell_history.num_channel && correctTime)
                    {
                        arr_ts_list[timeMs] = true;
                        channel.statistics.count_danger++;
                        arr_value.push( [timeMs,1]);
                    }
                }
                
                // else if(si21||si22)
                // {
                //     var validTs = cell_history.ts!==undefined&&cell_history.ts>0;
                //     if (cell_history.ts>channel.statistics.last_ts)
                //     {
                //         channel.statistics.last_ts=cell_history.ts;
                //         channel.statistics.last_ts_rus=moment(cell_history.ts).format('DD.MM.YY HH:mm' );
                //     }
                // }
                
                if (cell_history.time!==undefined&&cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd()&&cell_history.type==1&&(si11||si21||si12||si22))
                {
                    channel.statistics.chart.push( [convert_time(cell_history.time)*1000,cell_history.sensor]);
                    channel.statistics.chart_consumption.push( [convert_time(cell_history.time)*1000,cell_history.sensor,0,cell_history.fcnt]);
                    

                    if (cell_history.time<minDate)
                    {
                        minDate = cell_history.time;
                        minS = cell_history.sensor;
                    }
                    if (cell_history.time>maxDate)
                    {
                        maxDate = cell_history.time;
                        maxS = cell_history.sensor;
                    }
                    if (cell_history.time!=undefined&&cell_history.sensor!=undefined)
                    {
                        tempArr.push( [cell_history.time,cell_history.sensor]);
                        temp[cell_history.time]=cell_history.sensor;
                    }
                    else if (cell_history.time!=undefined)
                    {
                        console.log('Error 1294' );
                    }
                    channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                } 
                else if( cell_history.time!==undefined && cell_history.type==1&&(si11||si21||si12||si22) )
                {
                    _items_all[cell_history.time] = true;
                    channel.statistics.chart_consumption.push( [convert_time(cell_history.time)*1000,cell_history.sensor,0,cell_history.fcnt]);
                }
                else if ( cell_history.ts > ( date.getStart() * 1000 ) && cell_history.ts < ( date.getEnd() * 1000 ) && cell_history.type == 1 && ( td11 ) )
                {
                   channel.statistics.chart_temperature.push( [cell_history.ts, cell_history.temperature] );
                }
                else if (cell_history.time>=date.getStart()&&cell_history.time<=date.getEnd()&&cell_history.type==1&&device.device_type==9)
                {
                    if (!_items_1[cell_history.time])
                    {
                        _items_1[cell_history.time]=true;
                        channel.statistics.chart.push( [convert_time(cell_history.time)*1000,cell_history.sensorKB]);

                        channel.statistics.chart_consumption.push( [convert_time(cell_history.time)*1000,cell_history.sensorKB,0,cell_history.fcnt]);
                        
                        if (cell_history.time<minDate)
                        {
                            minDate = cell_history.time;
                            minS = cell_history.sensorKB;
                        }
                        if (cell_history.time>maxDate)
                        {
                            maxDate = cell_history.time;
                            maxS = cell_history.sensorKB;
                        }
                        if (cell_history.time!=undefined&&cell_history.sensorKB!=undefined)
                        {
                            tempArr.push( [cell_history.time,cell_history.sensorKB]);
                            temp[cell_history.time]=cell_history.sensorKB;
                        }
                        else if (cell_history.time!=undefined)
                        {
                            console.log('Error 1294' );
                        }
                       channel.statistics.chart_temperature.push( [cell_history.time*1000,cell_history.temperature]);
                   }
                }
                else if( cell_history.time!==undefined &&cell_history.type==1&&device.device_type==9 && !_items_1[cell_history.time])
                {
                    _items_1[cell_history.time]=true;
                    channel.statistics.chart_consumption.push( [convert_time(cell_history.time)*1000,cell_history.sensorKB,0,cell_history.fcnt]);
                }
                if (cell_history.ts>(date.getStart()*1000)&&cell_history.ts<(date.getEnd()*1000)&&!si13rev2)
                {
                    connect.push({date: cell_history.ts,rssi: my_converter.round_size(cell_history.rssi,2),snr:my_converter.round_size(cell_history.snr,2)});
                    arr_date.push(moment(cell_history.ts).format('LLL' ).replace(',',' ' ));
                    arr_rssi.push(my_converter.round_size(cell_history.rssi,2));
                    arr_snr.push(my_converter.round_size(cell_history.snr,2));
                    snr.val+=cell_history.snr;snr.count++;
                    rssi.val+=cell_history.rssi;rssi.count++;  
                }
                
            }
            if (tp11)
            {
                var ts_arr = new Array();
                var validLastTsType1=last_type_1!=undefined&&last_type_1.ts>0;
                var validLastTsSensorOut1=last_sensor_out_1!=undefined&&last_sensor_out_1.ts;
                var validLastTsSensorOut2=last_sensor_out_2!=undefined&&last_sensor_out_2.ts;
                var validLastTsCharge=last_charge!=undefined&&last_charge.ts;
                if (validLastTsType1)
                {
                    ts_arr.push(last_type_1.ts);
                    if (!validLastTsSensorOut1)
                    {
                        channel.statistics.last_sensor_out_1 = last_type_1.sensor_out_1;
                    }
                    if (!validLastTsSensorOut2)
                    {
                        channel.statistics.last_sensor_out_2 = last_type_1.sensor_out_2;
                    }
                    if (!validLastTsCharge)
                    {
                        channel.statistics.last_charge = last_type_1.charge;
                    }
                    channel.statistics.last_sensorTP = last_type_1.sensorTP;
                    channel.statistics.last_temperature = last_type_1.temperature;
                    channel.statistics.last_sensor_danger_1 = last_type_1.sensor_danger_1;
                    channel.statistics.last_sensor_danger_2 = last_type_1.sensor_danger_2;
                    channel.statistics.last_type_powered = last_type_1.type_powered;
                }
                if (validLastTsSensorOut1)
                {
                    ts_arr.push(last_sensor_out_1.ts);
                    if (validLastTsType1&&last_type_1.ts>last_sensor_out_1.ts)
                    {
                        channel.statistics.last_sensor_out_1 = last_type_1.sensor_out_1;
                    }
                    else
                    {
                        channel.statistics.last_sensor_out_1 = last_sensor_out_1.value;
                    }
                }
                if (validLastTsSensorOut2)
                {
                    ts_arr.push(last_sensor_out_2.ts);
                    if (validLastTsType1&&last_type_1.ts>last_sensor_out_2.ts)
                    {
                        channel.statistics.last_sensor_out_2 = last_type_1.sensor_out_2;
                    }
                    else
                    {
                        channel.statistics.last_sensor_out_2 = last_sensor_out_2.value;
                    }
                }
                if (validLastTsCharge)
                {
                    ts_arr.push(last_charge.ts);
                    if (validLastTsType1&&last_type_1.ts>last_charge.ts)
                    {
                        channel.statistics.last_charge = last_type_1.charge;
                    }
                    else
                    {
                        channel.statistics.last_charge = last_charge.value;
                    }
                }
                for(var i = 0; i<ts_arr.length;i++)
                {
                    if (!channel.statistics.last_ts||channel.statistics.last_ts<ts_arr[i])
                    {
                        channel.statistics.last_ts = ts_arr[i];
                    }
                }
                channel.statistics.avgSensorTP = Math.round(((summ_sensorTP/count_sensorTP)*1000))/1000;
            } 
            else if (tp11rev2)
            {
                var time_arr = new Array();
                var validLastTimeType1 = last_type_1 != undefined && last_type_1.time > 0;
                var validLastTimeSensorOut1 = last_sensor_out_1 != undefined && last_sensor_out_1.time;
                var validLastTimeSensorOut2 = last_sensor_out_2 != undefined && last_sensor_out_2.time;
                var validLastTimeCharge = last_charge != undefined && last_charge.time;
                if ( validLastTimeType1 )
                {
                    time_arr.push(last_type_1.time);
                    if (!validLastTimeSensorOut1)
                    {
                        channel.statistics.last_sensor_out_1 = last_type_1.sensor_out_1;
                    }
                    if (!validLastTimeSensorOut2)
                    {
                        channel.statistics.last_sensor_out_2 = last_type_1.sensor_out_2;
                    }
                    if (!validLastTimeCharge)
                    {
                        channel.statistics.last_charge = last_type_1.charge;
                    }
                    channel.statistics.last_sensorTP = last_type_1.sensorTP;
                    channel.statistics.last_temperature = last_type_1.temperature;
                    channel.statistics.last_sensor_danger_1 = last_type_1.sensor_danger_1;
                    channel.statistics.last_sensor_danger_2 = last_type_1.sensor_danger_2;
                    channel.statistics.last_type_powered = last_type_1.type_powered;
                }
                if ( validLastTimeSensorOut1 )
                {
                    time_arr.push( last_sensor_out_1.time );
                    if ( validLastTimeType1 && last_type_1.time > last_sensor_out_1.time )
                    {
                        channel.statistics.last_sensor_out_1 = last_type_1.sensor_out_1;
                    }
                    else
                    {
                        channel.statistics.last_sensor_out_1 = last_sensor_out_1.value;
                    }
                }
                if ( validLastTimeSensorOut2 )
                {
                    time_arr.push( last_sensor_out_2.time );
                    if (validLastTimeType1&&last_type_1.time>last_sensor_out_2.time)
                    {
                        channel.statistics.last_sensor_out_2 = last_type_1.sensor_out_2;
                    }
                    else
                    {
                        channel.statistics.last_sensor_out_2 = last_sensor_out_2.value;
                    }
                }
                if ( validLastTimeCharge )
                {
                    time_arr.push( last_charge.time );
                    if ( validLastTimeType1 && last_type_1.time > last_charge.time )
                    {
                        channel.statistics.last_charge = last_type_1.charge;
                    }
                    else
                    {
                        channel.statistics.last_charge = last_charge.value;
                    }
                }
                for( var i = 0; i < time_arr.length; i++ )
                {
                    if ( !channel.statistics.last_time || channel.statistics.last_time < time_arr[i] )
                    {
                        channel.statistics.last_time = time_arr[i];
                    }
                }
                channel.statistics.avgSensorTP = Math.round( ( ( summ_sensorTP / count_sensorTP ) * 1000 ) ) / 1000;
            }
            if (channel.type_channel==6)
            {

            }
            else
            {
                fcnt = arr_date.length;
                var tempData = [];
                var tempTime =minDate;
                var block = 0;
                while(tempTime<=maxDate)
                {
                    if (temp[tempTime]!=undefined&&temp[tempTime+HOUR_S]!=undefined)
                    {
                        try
                        {
                            var resultV=temp[tempTime+HOUR_S]-temp[tempTime];
                            if (resultV>0)
                            {
                                tempData.push( [(convert_time(tempTime)+HOUR_S)*1000,resultV]);  
                            }
                        }
                        catch(err)
                        {
                            tempData.push( [(convert_time(tempTime)+HOUR_S)*1000,0]);
                        }
                    }
                    else if ((convert_time(tempTime)+HOUR_S)*1000<=new Date().getTime())
                    {
                        tempData.push( [(convert_time(tempTime)+HOUR_S)*1000,0]);
                    }
                    tempTime=tempTime+HOUR_S;
                }    
                for (var i =tempArr.length-1;i>=0;i--)
                {
                    if (tempArr[i][0]>=block)
                    {
                        var temp_next = tempArr[i][0]+HOUR_S;
                        if (temp[temp_next]!=undefined)
                        {
                            if ((temp_next-block)%HOUR_S==0||block==0)
                            {
                                arr_value.push( [(convert_time(temp_next))*1000,temp[temp_next]-tempArr[i][1]]);
                                block = temp_next;
                                delete temp[temp_next];
                            }
                        }
                        else
                        {
                            arr_value.push( [convert_time(temp_next)*1000,0]);
                            block = temp_next;
                        }
                    }
                }
                if (device.device_type!=4)
                {
                    arr_value = tempData;
                }
                channel.statistics.chart.sort(function(a,b){
                    return a[0]-b[0];
                });
                channel.statistics.chart_consumption.sort(function(a,b){
                    return a[0]-b[0];
                });
            }
            if( si12 || mbus1 )
            {
                channel.statistics.last_sensor_out_1 = last_sensor_out_1.value;
                channel.statistics.last_sensor_out_2 = last_sensor_out_2.value;
                channel.statistics.last_state_energy = last_state_energy.value;
            }
            arr_value.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.arr_value_events.sort(function(a,b){
                return a[0]-b[0];
            });
            arr_value_4.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.arr_power_profile.sort(function(a,b){
                return a.date_1-b.date_1;
            });
            channel.statistics.arr_value_consum.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.arr_value_consum_v4.sort(function(a,b){
                return a[0]-b[0];
            });
            
            channel.statistics.chart_temperature.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.chart_damp_new.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.chart_lux_new.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.chart_dB_new.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.chart_CO2_new.sort(function(a,b){
                return a[0]-b[0];
            });

            channel.statistics.chart_temperature_new.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.chart_sensor_new.sort(function(a,b){
                return a[0]-b[0];
            });
            channel.statistics.chart_t.sort(function(a,b){
                return a[0]-b[0];
            });
            for (var i = 0 ; i< channel.statistics.chart_t.length;i++)
            {
                channel.statistics.chart_t[i][0]= channel.statistics.chart_t[i][0]+(1000*get_time_zone());
            }
            for (var i = 0 ; i< channel.statistics.chart_temperature.length;i++)
            {
                if (device.device_type!=12) channel.statistics.chart_temperature[i][0]= channel.statistics.chart_temperature[i][0]+(1000*get_time_zone());
            }
            for (var i = 0 ; i< arr_value.length;i++)
            {
                
                arr_value[i][0]= arr_value[i][0]+(get_time_zone()*1000);
                
            }
            for (var i = 0 ; i< arr_value_4.length;i++)
            {
                if ( device.device_type == 9 )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] * 1000 + ( 1000 * get_time_zone() );
                }
                else if ( device.device_type == 15 )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] + ( 1000 * get_time_zone() );
                }
                else if ( device.device_type == 12 || device.device_type == 24 )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] * 1000;
                }
                else if ( device.device_type == 13 )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] * 1000 + ( 1000 * get_time_zone() );
                } 
                else if ( smart || smartRev2 || hs || um )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] + ( 1000 * get_time_zone() );
                }
                else if ( tp11 )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] + get_time_zone()*1000;
                }
                else if ( tp11rev2 )
                {
                    arr_value_4[i][0] = arr_value_4[i][0] * 1000 + ( 1000 * get_time_zone() );
                }
                else if (td11)
                {
                    arr_value_4[i][0] = arr_value_4[i][0] + get_time_zone()*1000;
                }
                else if (td11rev2)
                {
                    arr_value_4[i][0] = arr_value_4[i][0] * 1000 + ( 1000 * get_time_zone() );
                }
                else
                {
                    arr_value_4[i][0] = arr_value_4[i][0] * 1000 + ( 1000 * get_time_zone() );
                }
                
            }
            if (channel.history.length>0)
            {
                channel.statistics.avgTemperatur=Math.round(((summ_t/channel.history.length)*10))/10;
            }
            else
            {
                channel.statistics.avgTemperatur = false;
            }
            if (device.device_type==9)
            {
                channel.statistics.consumed = maxS-minS;
                channel.statistics.start_sensorKB = minS;
                channel.statistics.end_sensorKB = maxS;
            }
            else if (si13 || si13rev2)
            {
                if(impulse)
                {
                    channel.statistics.consumed = my_converter.round_size(((maxS-minS)/this.select_channel_temp.division)*kt,2);
                    channel.statistics.start_sensor = my_converter.round_size((((this.select_channel_temp.init_value*this.select_channel_temp.division)+minS)/this.select_channel_temp.division)*kt,3);
                    channel.statistics.end_sensor = my_converter.round_size((((this.select_channel_temp.init_value*this.select_channel_temp.division)+maxS)/this.select_channel_temp.division)*kt,3);
                }
                else
                {
                    channel.statistics.consumed = maxS-minS;
                    channel.statistics.start_sensor = minS;
                    channel.statistics.end_sensor = maxS;
                }
            }
            else if (device.device_type==12 || device.device_type==24)
            {
                
                channel.statistics.consumed = maxS-minS;
                channel.statistics.start_value = minS;
                channel.statistics.end_value = maxS;
                channel.statistics.avgTemperatur=Math.round(((summ_t/count_t)*10))/10;
            }
            else if (device.device_type==4)
            {
                channel.statistics.avgTemperatur=Math.round(((summ_t/count_t)*10))/10;
            }
            else
            {
                channel.statistics.consumed = my_converter.round_size(((maxS-minS)/this.select_channel_temp.division)*kt,2);
                channel.statistics.start_sensor = my_converter.round_size((((this.select_channel_temp.init_value*this.select_channel_temp.division)+minS)/this.select_channel_temp.division)*kt,3);
                channel.statistics.end_sensor = my_converter.round_size((((this.select_channel_temp.init_value*this.select_channel_temp.division)+maxS)/this.select_channel_temp.division)*kt,3);
            }
            
            channel.statistics.last_time_rus=channel.type_channel==14?moment.unix(channel.statistics.last_time).utc().format('DD.MM.YY HH:mm' ):moment.unix(channel.statistics.last_time).format('DD.MM.YY HH:mm' );
            channel.statistics.fcnt = fcnt;
            channel.statistics.avr_snr = my_converter.round_size((snr.val/snr.count),2);
            channel.statistics.avr_rssi = my_converter.round_size((rssi.val/rssi.count),2);
            channel.statistics.arr_date = arr_date;
            channel.statistics.arr_rssi = arr_rssi;
            channel.statistics.arr_snr = arr_snr;
            channel.statistics.connect = connect;
            channel.statistics.arr_value = arr_value;
            channel.statistics.arr_value_4 = arr_value_4;
            channel.statistics.type_channel = channel.type_channel;
            channel.statistics.dangers = _dangers;
        }
        catch (err)
        {
            return false;
        }
    }
}
class cell_history
{
    constructor(mess,data,num_sensor,dt,version)
    {
        this.version = version;
        this.ts=mess.ts;
        this.ack=mess.ack;
        this.fcnt=mess.fcnt;
        this.port=mess.port;
        this.freq=mess.freq;
        this.dr=mess.dr;
        this.temperature=data.temperature;
        this.temperature_2=data.temperature_2;
        this.sensor_1=data.sensors.sensor_1;
        this.sensor_2=data.sensors.sensor_2;
        this.sensor_3=data.sensors.sensor_3;
        this.sensor_4=data.sensors.sensor_4;
        this.sensor_5=data.sensor_5;
        this.sensor_6=data.sensor_6;
        this.sensor_7=data.sensor_7;
        this.sensor_8=data.sensor_8;
        this.sensor_9=data.sensor_9;
        this.sensor_10=data.sensor_10;
        this.sensor_11=data.sensor_11;
        this.sensor_12=data.sensor_12;
        this.rssi=mess.rssi;
        this.snr=mess.snr;
        this.charge=data.charge;
        this.switch_device=data.switch_device;
        this.time=data.time;
        this.num_channel = data.num_channel;
        this.sensor=data.sensors['sensor_'+num_sensor];
        this.type = data.type_package;
        
        this.sensorKB = data.sensorKB;
        
        this.state_tamper=data.state_tamper;
        this.state_display=data.state_display;
        this.state_security=data.state_security;
        this.leaking = data.leaking;
        this.breakthrough = data.breakthrough;
        this.reason=data.reason;
        this.hall_1=data.hall_1;
        this.hall_2=data.hall_2;
        
        this.limit_exceeded=data.limit_exceeded;
        this.min_temperature=data.min_temperature;
        this.min_damp=data.min_damp;
        this.max_damp=data.max_damp;
        this.min_lux=data.min_lux;
        this.max_lux=data.max_lux;
        this.min_dB=data.min_dB;
        this.max_dB=data.max_dB;
        this.min_CO2=data.min_CO2;
        this.max_CO2=data.max_CO2;
        this.max_temperature=data.max_temperature;
        this.min_sensor=data.min_sensor;
        this.max_sensor=data.max_sensor;

        
        
        this.dt = dt;
        
        this.type_activation=data.type_activation;
        this.state_ack=data.state_ack;
        this.period_connect_minute=data.period_connect_minute;
        this.collection_period_minute=data.collection_period_minute;
        this.time_zone_minute=data.time_zone_minute;
        this.type_powered=data.type_powered;
        this.sensor_danger_1=data.sensor_danger_1;
        this.sensor_danger_2=data.sensor_danger_2;
        this.sensor_out_1=data.sensor_out_1;
        this.sensor_out_2=data.sensor_out_2;
        this.num_out_channel=data.num_out_channel;
        
        this.sensor_in_1=data.sensor_in_1;
        this.sensor_in_2=data.sensor_in_2;
        this.sensorTP=data.sensorTP;
        this.state_sensor_0=data.state_sensor_0;
        this.state_sensor_1=data.state_sensor_1;
        
        
        this.count_rate_active = data.count_rate_active;
        this.rate_active = data.rate_active;
        this.kt = data.kt;
        this.note_1 = data.note_1;
        this.note_2 = data.note_2;
        this.event = data.event;
        this.state = data.state;
        this.type_in = data.type_in;
        this.result = data.result;
        this.sensor_rate_0 = data.sensor_rate_0;
        this.sensor_rate_1 = data.sensor_rate_1;
        this.sensor_rate_2 = data.sensor_rate_2;
        this.sensor_rate_3 = data.sensor_rate_3;
        this.sensor_rate_4 = data.sensor_rate_4;
        this.sensor_rate_sum = data.sensor_rate_sum;
        
        this.in_move = data.in_move;
        this.angle = data.angle;
        this.led_duty = data.led_duty;
        this.damp = data.damp;
        this.lux = data.lux;
        this.dB = data.dB;
        this.CO2 = data.CO2;
        this.coord_status = data.coord_status;
        this.lat = data.lat;
        this.lng = data.lng;
        this.dir = data.dir;
        this.speed = data.speed;
        this.alt = data.alt;
        this.sat_visible = data.sat_visible;
        this.sat_used = data.sat_used;
        this.alarm = data.alarm;


        this.note_1 = data.note_1;
        this.note_2 = data.note_2;
        this.network_address = data.network_address;
        this.time_zone = data.time_zone;
        this.period_reconnection_h = data.period_reconnection_h;
        this.flag_event = data.flag_event;
        this.flag_helf_hour = data.flag_helf_hour;
        this.flag_ack = data.flag_ack;
        this.limit_power = data.limit_power;
        this.accumulation_period_info = data.accumulation_period_info;
        this.accumulation_period_day_info = data.accumulation_period_day_info;
        this.accumulation_period_month_info = data.accumulation_period_month_info;
        this.accumulation_period_readings = data.accumulation_period_readings;
        this.accumulation_period_day_readings = data.accumulation_period_day_readings;
        this.accumulation_period_month_readings = data.accumulation_period_month_readings;
        this.accumulation_period_instant = data.accumulation_period_instant;
        this.accumulation_period_day_instant = data.accumulation_period_day_instant;
        this.accumulation_period_month_instant = data.accumulation_period_month_instant;
        this.tariff_schedule_season_number = data.tariff_schedule_season_number;
        this.tariff_schedule_code = data.tariff_schedule_code;
        for(var i = 1; i<=15; i++)
        {
            this[`tariff_schedule_minute_end_${i}_zone`] = data[`tariff_schedule_minute_end_${i}_zone`];
            this[`tariff_schedule_hour_end_${i}_zone`] = data[`tariff_schedule_hour_end_${i}_zone`];
            this[`tariff_schedule_number_${i}_zone`] = data[`tariff_schedule_number_${i}_zone`];
        }
        this.version_soft_device = data.version_soft_device;
        this.code_error_1 = data.code_error_1;
        this.code_error_2 = data.code_error_2;
        this.code_error_3 = data.code_error_3;
        this.code_error_4 = data.code_error_4;
        this.code_error_5 = data.code_error_5;
        this.diagnostic_codes = data.diagnostic_codes;
        this.location_description = data.location_description;
        this.version_module_electronic = data.version_module_electronic;
        this.version_parameterization = data.version_parameterization;
        this.flag_translation_time = data.flag_translation_time;
        this.flag_relay = data.flag_relay;
        this.state_relay = data.state_relay;
        this.info_radiomodule = data.info_radiomodule;
        this.nomenal_voltage = data.nomenal_voltage;
        this.nomenal_current = data.nomenal_current;
        this.code_model = data.code_model;
        this.maximal_current = data.maximal_current;
        this.limit_active_power = data.limit_active_power;
        this.time_minute_averages_power = data.time_minute_averages_power;
        this.mode_measuring_power = data.mode_measuring_power;
        this.time_minute_delay_automatic_switching_relay = data.time_minute_delay_automatic_switching_relay;


        this.selected_d_archive = data.selected_d_archive;
        this.selected_m_archive = data.selected_m_archive;
        this.selected_y_archive = data.selected_y_archive;
        this.cumulative_energy_summary = data.cumulative_energy_summary;
        this.cumulative_energy_rate_1 = data.cumulative_energy_rate_1;
        this.cumulative_energy_rate_2 = data.cumulative_energy_rate_2;
        this.cumulative_energy_rate_3 = data.cumulative_energy_rate_3;
        this.cumulative_energy_rate_4 = data.cumulative_energy_rate_4;

        this.count_automatic_switching_relay = data.count_automatic_switching_relay;
        for(var i = 1; i<=31; i++)
        {
            this[`code_page_indication_${i}`] = data[`code_page_indication_${i}`] ;
            this[`time_seconds_show_page_indication_${i}`] = data[`time_seconds_show_page_indication_${i}`];
        }
        this.voltage_A = data.voltage_A;
        this.voltage_B = data.voltage_B;
        this.voltage_C = data.voltage_C;
        this.electric_current_A = data.electric_current_A;
        this.electric_current_B = data.electric_current_B;
        this.electric_current_C = data.electric_current_C;
        this.power_factor_A = data.power_factor_A;
        this.power_factor_B = data.power_factor_B;
        this.power_factor_C = data.power_factor_C;
        
        this.power_factor_summary = data.power_factor_summary;
        this.frequency_network = data.frequency_network;
        this.status_state = data.status_state;
        for(var i = 1; i<=31; i++)
        {
            this[`special_day_d_${i}`] = data[`special_day_d_${i}`];
            this[`special_day_m_${i}`] = data[`special_day_m_${i}`];
            this[`special_day_t_${i}`] = data[`special_day_t_${i}`];
        }
        this.selected_d_archive = data.selected_d_archive;
        this.selected_date = data.selected_date;
        for(var i = 2; i<=48; i++)
        {
            this[`date_${i}`] = data[`date_${i}`];
        }
        this.half_hour_slices_active_power = data.half_hour_slices_active_power;
        this.temp = data.temp;
        this.journal = data.journal;
        this.number_phase = data.number_phase;
        this.active_power_A = data.active_power_A;
        this.active_power_B = data.active_power_B;
        this.active_power_C = data.active_power_C;
        this.active_power_summary = data.active_power_summary;
        this.reactive_power_A = data.reactive_power_A;
        this.reactive_power_B = data.reactive_power_B;
        this.reactive_power_C = data.reactive_power_C;
        this.reactive_power_summary = data.reactive_power_summary;
        this.total_power_A = data.total_power_A;
        this.total_power_B = data.total_power_B;
        this.total_power_C = data.total_power_C;
        this.total_power_summary = data.total_power_summary;



        
        this.date_1 = data.date_1;
        this.date_2 = data.date_2;
        this.period_avg_1 = data.period_avg_1;
        this.period_avg_2 = data.period_avg_2;
        this.A_p_1 = data.A_p_1;
        this.A_m_1 = data.A_m_1;
        this.R_p_1 = data.R_p_1;
        this.R_m_1 = data.R_m_1;
        this.A_p_2 = data.A_p_2;
        this.A_m_2 = data.A_m_2;
        this.R_p_2 = data.R_p_2;
        this.R_m_2 = data.R_m_2;
        this.UUID = data.UUID;
        this.serial = data.serial;
        this.model = data.model;
        this.count_phase = data.count_phase;
        this.count_rate = data.count_rate;
        this.relay_state = data.relay_state;
        this.release_date = data.release_date;
        this.version_soft = data.version_soft;
        this.kt = data.kt;
        this.data_b = data.data_b;
        this.size_package_in = data.size_package_in;
        this.size_package_out = data.size_package_out;
        this.num_out = data.num_out;
        this.count_package = data.count_package;
        this.B_1 = data.B_1;
        this.B_2 = data.B_2;
        this.B_3 = data.B_3;
        this.A_1 = data.A_1;
        this.A_2 = data.A_2;
        this.A_3 = data.A_3;
        this.P_1 = data.P_1;
        this.P_2 = data.P_2;
        this.P_3 = data.P_3;
        this.Q_1 = data.Q_1;
        this.Q_2 = data.Q_2;
        this.Q_3 = data.Q_3;
        this.S_1 = data.S_1;
        this.S_2 = data.S_2;
        this.S_3 = data.S_3;
        this.address = data.address;
        this.cmd_code = data.cmd_code;
        this.state_energy = data.state_energy;
        this.size_data = data.size_data;
        this.size_data_package = data.size_data_package;
        this.num_package = data.num_package;
        //клеммная крышка
        this.state_1 = data.state_1;
        //корпуса крышка
        this.state_2 = data.state_2;
        //Состояние реле ограничения нагрузки
        this.state_3 = data.state_3;

        this.total_heat_carrier_volume = data.total_heat_carrier_volume;
        this.active_channel_security_1 = data.active_channel_security_1;
        this.active_channel_security_2 = data.active_channel_security_2;
        this.current_coolant_flow      = data.current_coolant_flow;
        this.working_time_h            = data.working_time_h;
    }
    get_time()
    {
        return this.time!==undefined?moment.unix(this.time).format('LLL' ).replace(',','' ):'-';
    }
    get_time_original_2(yy,mm,dd)
    {
        try
        {
            yy = parseInt(yy);
            mm = parseInt(mm);
            dd = parseInt(dd);
            var validYY = typeof yy == 'number' && !isNaN(yy) && yy >= 0 && yy <= 99;
            var validMM = typeof mm == 'number' && !isNaN(mm) && mm >= 1 && mm <= 12;
            var validDD = typeof dd == 'number' && !isNaN(dd) && dd >= 1 && dd <= 31;
            if( validYY && validMM )
            {
                yy = yy + 2000;
                mm--;
                if (!validDD) return moment().utc().year(yy).month(mm).format('MM.YYYY');
                return moment().utc().year(yy).month(mm).date(dd).format('L');
            }
            else
            {
                return '-';
            }
        }
        catch(e)
        {
            return '-';
        }
    }
    get_time_original_range(param)
    {
        if( param === undefined ) return '-';

        var begin_h = moment.unix(param).utc().hour();
        if( begin_h.toString().length === 1) begin_h = '0' + begin_h;
        var begin_m = moment.unix(param).utc().minute();
        if( begin_m.toString().length === 1) begin_m = '0' + begin_m;

        var end_h = moment.unix(param+(60*30)).utc().hour();
        if( end_h.toString().length === 1) end_h = '0' + end_h;
        var end_m = moment.unix(param+(60*30)).utc().minute();
        if( end_m.toString().length === 1) end_m = '0' + end_m;

        return begin_h+':'+begin_m+' - '+end_h+':'+end_m;
    }
    get_time_original_3(param,format)
    {
        if( format === undefined ) format = 'lll';
        if( param === undefined ) return '-';
        return moment.unix(param).utc().format(format).replace(',','' );
    }
    get_time_original(param,format)
    {
        if( format === undefined ) format = 'LLL';
        if( param === undefined ) param = 'time';
        return this[param]!==undefined?moment.unix(this[param]).utc().format(format).replace(',','' ):'-';
    }
    get_ts_original(param,format)
    {
        if( format === undefined ) format = 'LLL';
        if( param === undefined ) param = 'time';
        return this[param]!==undefined?moment(this[param]).utc().format(format).replace(',','' ):'-';
    }
    get_ts()
    {
        return moment(this.ts).format('LLL' ).replace(',','' );
    }
    get_value(init_val,division,kt)
    {
        var my_convert =  new vega_converter();
        var valid = new valid_data();
        kt = valid.isNumber(kt)?kt:1;
        return my_convert.round_size(((init_val*division+this.sensor)/division)*kt,3);
    }
}
class data_lora
{
    constructor(dt,data,port,version)
    {
        this.type_package;
        this.comment;
        this.hex;
        this.hex_array;
        this.charge;
        this.switch_device = new Array();
        this.time;
        this.temperature;
        this.temperature_2;
        this.sensors = new Object();
        this.num_channel = 0;
        this.count = 0;
        this.type_archive;
        this.last_time;
        this.reason;
        this.archive = new Array();
        this.device_type = parseInt(dt);
        this.version = version;
        // if( this.version === undefined ) this.version = 0;
        this.count_rate_active;
        this.rate_active;
        this.kt;
        this.note_1;
        this.note_2;
        this.event;
        this.state;
        this.type_in;
        this.result;
        this.sensor_rate_0;
        this.sensor_rate_1;
        this.sensor_rate_2;
        this.sensor_rate_3;
        this.sensor_rate_4;
        this.sensor_rate_sum;
        
        this.in_move;
        this.angle;
        this.led_duty;
        this.damp;
        this.lux;
        this.dB;
        this.CO2;
        this.coord_status;
        this.lat;
        this.lng;
        this.dir;
        this.speed;
        this.alt;
        this.sat_visible;
        this.sat_used;
        this.alarm;
        
        this.date_1;
        this.date_2;
        this.period_avg_1;
        this.period_avg_2;
        this.A_p_1;
        this.A_m_1;
        this.R_p_1;
        this.R_m_1;
        this.A_p_2;
        this.A_m_2;
        this.R_p_2;
        this.R_m_2;
        this.UUID;
        this.serial;
        this.model;
        this.count_phase;
        this.count_rate;
        this.relay_state;
        this.release_date;
        this.version_soft;
        this.kt;
        this.data_b;
        this.size_package_in;
        this.size_package_out;
        this.num_out;
        this.count_package;
        this.B_1;
        this.B_2;
        this.B_3;
        this.A_1;
        this.A_2;
        this.A_3;
        this.P_1;
        this.P_2;
        this.P_3;
        this.Q_1;
        this.Q_2;
        this.Q_3;
        this.S_1;
        this.S_2;
        this.S_3;
        //клеммная крышка
        this.state_1;
        //корпуса крышка
        this.state_2;
        //Состояние реле ограничения нагрузки
        this.state_3;


        this.note_1;
        this.note_2;
        this.network_address;
        this.time_zone;
        this.period_reconnection_h;
        this.flag_event;
        this.flag_helf_hour;
        this.flag_ack;
        this.limit_power;
        this.accumulation_period_info;
        this.accumulation_period_day_info;
        this.accumulation_period_month_info;
        this.accumulation_period_readings;
        this.accumulation_period_day_readings;
        this.accumulation_period_month_readings;
        this.accumulation_period_instant;
        this.accumulation_period_day_instant;
        this.accumulation_period_month_instant;
        this.tariff_schedule_season_number;
        this.tariff_schedule_code;
        for(var i = 1; i<=15; i++)
        {
            this[`tariff_schedule_minute_end_${i}_zone`] = undefined;
            this[`tariff_schedule_hour_end_${i}_zone`] = undefined;
            this[`tariff_schedule_number_${i}_zone`] = undefined;
        }
        this.version_soft_device;
        this.code_error_1;
        this.code_error_2;
        this.code_error_3;
        this.code_error_4;
        this.code_error_5;
        this.diagnostic_codes;
        this.location_description;
        this.version_module_electronic;
        this.version_parameterization;
        this.flag_translation_time;
        this.flag_relay;
        this.state_relay;
        this.info_radiomodule;
        this.nomenal_voltage;
        this.nomenal_current;
        this.code_model;
        this.maximal_current;
        this.limit_active_power;
        this.time_minute_averages_power;
        this.mode_measuring_power;
        this.time_minute_delay_automatic_switching_relay;

        this.selected_d_archive;
        this.selected_m_archive;
        this.selected_y_archive;
        this.cumulative_energy_summary;
        this.cumulative_energy_rate_1;
        this.cumulative_energy_rate_2;
        this.cumulative_energy_rate_3;
        this.cumulative_energy_rate_4;


        this.count_automatic_switching_relay;
        for(var i = 1; i<=31; i++)
        {
            this[`code_page_indication_${i}`] = undefined;
            this[`time_seconds_show_page_indication_${i}`] = undefined;
        }
        this.voltage_A;
        this.voltage_B;
        this.voltage_C;
        this.electric_current_A;
        this.electric_current_B;
        this.electric_current_C;
        this.power_factor_A;
        this.power_factor_B;
        this.power_factor_C;
        this.power_factor_summary;
        this.frequency_network;
        this.status_state;
        for(var i = 1; i<=31; i++)
        {
            this[`special_day_d_${i}`] = undefined;
            this[`special_day_m_${i}`] = undefined;
            this[`special_day_t_${i}`] = undefined;
        }
        this.selected_d_archive;
        this.selected_date;
        for(var i = 2; i<=48; i++)
        {
            this[`date_${i}`] = undefined;
        }
        this.half_hour_slices_active_power = [];
        this.temp;
        this.journal = [];
        this.number_phase;
        this.active_power_A;
        this.active_power_B;
        this.active_power_C;
        this.active_power_summary;
        this.reactive_power_A;
        this.reactive_power_B;
        this.reactive_power_C;
        this.reactive_power_summary;
        this.total_power_A;
        this.total_power_B;
        this.total_power_C;
        this.total_power_summary;

        this.address;
        this.cmd_code;
        this.state_energy;
        this.size_data;
        this.size_data_package;
        this.num_package;
        this.port = port;
        this.validParse=this.set_data(data);
    }
    _set_last_time()
    {
        try
        {
            var time = this.hex_array[9].toString()+this.hex_array[8].toString()+this.hex_array[7].toString()+this.hex_array[6].toString();
            this.last_time = parseInt(time,16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_hex(hex)
     {
        this.hex = hex;
        this.hex_array = new Array();
        for (var i =0;i<this.hex.length-1;i=i+2)
        {
           this.hex_array.push( this.hex.substring(i, i+2) ); 
        }
        try
        {
           this.type_package=parseInt(this.hex_array[0],16);
           return true;
        }
        catch(err){
           return false;
        }
    }
    _set_type_archive()
    {
        try
        {
            this.type_archive=parseInt(this.hex_array[5],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_charge()
    {
        try
        {
            this.charge=parseInt(this.hex_array[1],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_num_channel()
    {
        try
        {
            this.num_channel=parseInt(this.hex_array[3],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_count()
    {
        try
        {
            this.count=parseInt(this.hex_array[4],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_temperature(b)
    {
        try
        {
            this.temperature=parseInt(this.hex_array[b],16);
            if (this.temperature>127)
            {
                var hex = this.hex_array[b];
                if (hex.length % 2 != 0) {
                    hex = "0" + hex;
                }
                var num = parseInt(hex, 16);
                var maxVal = Math.pow(2, hex.length / 2 * 8);
                if (num > maxVal / 2 - 1) {
                    num = num - maxVal
                }
                this.temperature=num;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_temperature_b2()
    {
        try
        {  
            var hex = this.hex_array[4]+this.hex_array[3];
            if (hex.length % 4 != 0) {
                hex = "0" + hex;
            }
            var num = parseInt(hex, 16);
            var maxVal = Math.pow(2, hex.length / 2 * 8);
            if (num > maxVal / 2 - 1) {
                num = num - maxVal;
            }
            this.temperature=num/10;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_sensors_opt(IndexByteBegin)
    {
        try
        {
            if (IndexByteBegin!==undefined)
            {
                for(var i=1; i<=4; i++)
                {
                    var hex = '';
                    for(var j=1; j<=4; j++)
                    {
                       if (this.hex_array[IndexByteBegin]===undefined) return false;
                       hex = this.hex_array[IndexByteBegin]+hex;
                       IndexByteBegin++;
                    }
                    this.sensors['sensor_'+i] = parseInt(hex,16);
                }
            }
            else
            {
                this.sensors.sensor_1=parseInt(this.hex_array[7]+this.hex_array[6]+this.hex_array[5]+this.hex_array[4],16);
                this.sensors.sensor_2=parseInt(this.hex_array[11]+this.hex_array[10]+this.hex_array[9]+this.hex_array[8],16);
                this.sensors.sensor_3=parseInt(this.hex_array[15]+this.hex_array[14]+this.hex_array[13]+this.hex_array[12],16);
                this.sensors.sensor_4=parseInt(this.hex_array[19]+this.hex_array[18]+this.hex_array[17]+this.hex_array[16],16);
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_sensors(begin_byte)
    {
        try
        {
            if ( begin_byte == undefined ) begin_byte = 8;
            this.sensors.sensor_1=parseInt(this.hex_array[begin_byte+3]+this.hex_array[begin_byte+2]+this.hex_array[begin_byte+1]+this.hex_array[begin_byte],16);
            this.sensors.sensor_2=parseInt(this.hex_array[begin_byte+7]+this.hex_array[begin_byte+6]+this.hex_array[begin_byte+5]+this.hex_array[begin_byte+4],16);
            this.sensors.sensor_3=parseInt(this.hex_array[begin_byte+11]+this.hex_array[begin_byte+10]+this.hex_array[begin_byte+9]+this.hex_array[begin_byte+8],16);
            this.sensors.sensor_4=parseInt(this.hex_array[begin_byte+15]+this.hex_array[begin_byte+14]+this.hex_array[begin_byte+13]+this.hex_array[begin_byte+12],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_reason(b)
    {
        try
        {
            switch (this.hex_array[b]) {
                case '00':
                    this.reason=0
                    break;
                case '01':
                    this.reason=1
                    break;  
                case '02':
                    this.reason=2
                    break;
                case '03':
                    this.reason=3
                    break;
                case '04':
                    this.reason=4
                    break;
                case '05':
                    this.reason=5
                    break;
                case '06':
                    this.reason=5
                    break;
                case '07':
                    this.reason=5
                    break;
                default:
                    this.reason=undefined;
                    break;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_status_smart()
    {
        try
        {
            var status=parseInt(this.hex_array[6],16).toString(2).split('' ).reverse().splice(0,6);
            if (status[0]==1)
            {
                this.state_sensor_0 = true;
            }
            else
            {
                this.state_sensor_0 =false;
            }
            if (status[1]==1)
            {
                this.state_sensor_1 = true;
            }
            else
            {
                this.state_sensor_1 =false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_status(indexByte)
    {
        try
        {
            if (indexByte === undefined) indexByte = 6;
            var status=parseInt(this.hex_array[indexByte],16).toString(2).split('' ).reverse().splice(0,6);
            if (status[0]==1)
            {
                //размокнут
                this.state_security = true;
            }
            else
            {
                //замкнут
                this.state_security =false;
            }
            if (status[1]==1)
            {
                //вскрыт
                this.state_tamper = true;
            }
            else
            {
                //не вскрыт
                this.state_tamper = false;
            }
            if (status[2]==1)
            {
                this.hall_1=true;
            }
            else
            {
                this.hall_1=false;
            }
            if (status[3]==1)
            {
                this.hall_2=true;
            }
            else
            {
                this.hall_2=false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_state(bytes)
    {
        try
        {
            if( !isObject(bytes)||bytes.length < 4 ) bytes = [28,29,30,31];
            bytes.reverse();
            var state = '';
            for(var i = 0; i<bytes.length; i++)
            {
                state+=this.hex_array[bytes[i]];
            }
            //var state = this.hex_array[31].toString()+this.hex_array[30].toString()+this.hex_array[29].toString()+this.hex_array[28].toString();
            this.state = state;
            var state_int= parseInt(state,16);
            var state_binary = state_int.toString(2).split('' ).reverse();
            if (state_binary[0]==1) 
            {
                this.state_1 = true;
            }
            else
            {
                this.state_1 = false;
            }
            if (state_binary[1]==1) 
            {
                this.state_2 = true;
            }
            else
            {
                this.state_2 = false;
            }
            if (state_binary[2]==1) 
            {
                this.state_3 = true;
            }
            else
            {
                this.state_3 = false;
            }
            return true;
        }
        catch(e)
        {
            return false;
        }
    }
    _set_status_tp11(byte)
    {
        try
        {
            if( byte === undefined ) byte = 5;
            var status= parseInt(this.hex_array[byte],16).toString(2).split('' ).reverse().splice(0,6);
            if (status[0]==1&&parseInt(status[0]))
            {
                this.type_powered = 'external';
            }
            else
            {
                this.type_powered = 'battery';
            }
            if (status[1]==1&&parseInt(status[1]))
            {
                this.sensor_danger_1 = true;
            }
            else
            {
               this.sensor_danger_1 = false;
            }
            if (status[2]==1&&parseInt(status[2]))
            {
                this.sensor_danger_2 = true;
            }
            else
            {
               this.sensor_danger_2 = false;
            }
            if (status[3]==1&&parseInt(status[3]))
            {
                this.sensor_out_1 = true;
            }
            else
            {
               this.sensor_out_1 = false;
            }
            if (status[4]==1&&parseInt(status[4]))
            {
                this.sensor_out_2 = true;
            }
            else
            {
               this.sensor_out_2 = false;
            }
            return true;
        }
        catch(e)
        {
            return false;
        }
    }
    _set_switch_state_tl11(b)
    {
        try
        {
            var ss= parseInt(this.hex_array[b],16).toString(2).split('' ).reverse().splice(0,6);
            var b1 = ss[0]!==undefined?ss[0].toString():'0';
            var b2 = ss[1]!==undefined?ss[1].toString():'0';
            var reason = b2+b1;
            this.reason = reason;
            //00 - по времени
            //01 - по срабатыванию тампера
            //10 - сработал датчик Холла двери
            //11 - сработал датчик Холла отрыва
           // this.hall_1 = parseInt(ss[2])?true:false;
           // this.hall_2 = parseInt(ss[3])?true:false;
            this.state_tamper = parseInt(ss[2])?true:false;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_switch_state_td12(b)
    {
        try
        {
            var ss= parseInt(this.hex_array[b],16).toString(2).split('' ).reverse().splice(0,6);
            var b1 = ss[0]!==undefined?ss[0].toString():'0';
            var b2 = ss[1]!==undefined?ss[1].toString():'0';
            var reason = b2+b1;
            this.reason = reason;
            //00 - по времени
            //01 - по срабатыванию тампера
            //10 - сработал датчик Холла двери
            //11 - сработал датчик Холла отрыва
            this.hall_1 = parseInt(ss[2])?true:false;
            this.hall_2 = parseInt(ss[3])?true:false;
            this.state_tamper = parseInt(ss[4])?true:false;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_switch_device_smart()
    {
        try
        {
            var sw= parseInt(this.hex_array[2],16).toString(2).split('' ).reverse().splice(0,6);
            this.switch_device=sw;
            if (sw[0]==1)
            {
                this.type_activation = 'ABP';
            }
            else
            {
                this.type_activation = 'OTAA';
            }
            if (sw[1]==1)
            {
                this.state_ack = true;
            }
            else
            {
                this.state_ack = false;
            }
            var b1 = sw[2]!==undefined?sw[2].toString():'0';
            var b2 = sw[3]!==undefined?sw[3].toString():'0';
            var period_connect = b1+b2;
            switch (period_connect) {
                case '00':
                    this.period_connect_minute = 60;
                    break;
                case '10':
                    this.period_connect_minute = 360;
                    break;
                case '01':
                    this.period_connect_minute = 720;
                    break;
                case '11':
                    this.period_connect_minute = 1440;
                break;
                
                default:

                    break;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_switch_device_mbus()
    {
        try
        {
            var sw= parseInt(this.hex_array[2],16).toString(2).split('' ).reverse().splice(0,6);
            this.switch_device=sw;
            if (sw[0]==1)
            {
                this.type_activation = 'ABP';
            }
            else
            {
                this.type_activation = 'OTAA';
            }
            if (sw[1]==1)
            {
                this.state_ack = true;
            }
            else
            {
                this.state_ack = false;
            }
            var b1 = sw[2]!==undefined?sw[2].toString():'0';
            var b2 = sw[3]!==undefined?sw[3].toString():'0';
            var period_connect = b1+b2;
            switch (period_connect) {
                case '00':
                    this.period_connect_minute = 60;
                    break;
                case '10':
                    this.period_connect_minute = 360;
                    break;
                case '01':
                    this.period_connect_minute = 720;
                    break;
                case '11':
                    this.period_connect_minute = 1440;
                break;
                
                default:

                    break;
            }
            this.active_channel_security_1 = sw[4] == 1;
            this.active_channel_security_2 = sw[5] == 1;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_switch_device_tp11()
    {
        try
        {
            var sw= parseInt(this.hex_array[2],16).toString(2).split('' ).reverse().splice(0,6);
            this.switch_device=sw;
            if (sw[0]==1)
            {
                this.type_activation = 'ABP';
            }
            else
            {
                this.type_activation = 'OTAA';
            }
            if (sw[1]==1)
            {
                this.state_ack = true;
            }
            else
            {
                this.state_ack = false;
            }
            var b1 = sw[2]!==undefined?sw[2].toString():'0';
            var b2 = sw[3]!==undefined?sw[3].toString():'0';
            var b3 = sw[4]!==undefined?sw[4].toString():'0';
            var b4 = sw[5]!==undefined?sw[5].toString():'0';
            var period_connect = b1+b2+b3+b4;
            switch (period_connect) {
                case '0000':
                    this.period_connect_minute = 60;
                    break;
                case '1000':
                    this.period_connect_minute = 360;
                    break;
                case '0100':
                    this.period_connect_minute = 720;
                    break;
                case '1100':
                    this.period_connect_minute = 1440;
                break;
                case '0010':
                    this.period_connect_minute = 1;
                    break;
                case '1010':
                    this.period_connect_minute = 3;
                    break;
                case '0110':
                    this.period_connect_minute = 5;
                    break;
                case '1110':
                    this.period_connect_minute = 10;
                    break;
                case '0001':
                    this.period_connect_minute = 15;
                    break;
                case '1001':
                    this.period_connect_minute = 30;
                    break;
                default:
                    
                    break;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_period_connect_minute(b)
    {
        try
        {
            if (b!==undefined)
            {
                var result_int = parseInt(this.hex_array[b],16);
                if (!isNaN(result_int))
                {
                    switch (result_int) {
                        case 1:
                            this.period_connect_minute = 60;
                            break;
                        case 2:
                            this.period_connect_minute = 360;
                            break;
                        case 3:
                            this.period_connect_minute = 720;
                            break;
                        case 4:
                            this.period_connect_minute = 1440;
                        break;
                        default:
                            return false;
                            break;
                    }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_collection_period_minute(b)
    {
        try
        {
            if (b!==undefined)
            {
                var result_int = parseInt(this.hex_array[b],16);
                if (!isNaN(result_int))
                {
                    switch (result_int) {
                        case 1:
                            this.collection_period_minute = 5;
                            break;
                        case 2:
                            this.collection_period_minute = 15;
                            break;
                        case 3:
                            this.collection_period_minute = 30;
                            break;
                        case 4:
                            this.collection_period_minute = 60;
                            break;
                        case 5:
                            this.collection_period_minute = 360;
                            break;
                        case 6:
                            this.collection_period_minute = 720;
                            break;
                        case 7:
                            this.collection_period_minute = 1440;
                        break;
                        default:
                            return false;
                        break;
                    }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_switch_device()
    {
        try
        {
            this.switch_device=parseInt(this.hex_array[2],16).toString(2).split('' ).reverse().splice(0,6);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_archive()
    {
       for (var i =10;i<this.hex_array.length-3;i=i+4)
         {
             var time = this.last_time*1000;
             if (this.type_archive==0)
             {
                 time = moment(time).subtract((i-10)/4, 'hour' ).unix();
             }
             else if (this.type_archive==1)
             {
                 time = moment(time).subtract((i-10)/4, 'day' ).unix();
             } else if (this.type_archive==2)
             {
                 time = moment(time).subtract((i-10)/4, 'month' ).unix();
             } else if (this.type_archive==3)
             {
             }
             else 
             {
                 return false;
             }
            this.archive.push( [this.hex_array[i+3].toString()+this.hex_array[i+2].toString()+this.hex_array[i+1].toString()+this.hex_array[i].toString(),time]); 
         }
    }
    _set_sensorTP()
    {
        try
        {
            var sensorTP = this.hex_array[7].toString()+this.hex_array[6].toString();
            this.sensorTP = parseInt(sensorTP,16)/100;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_sensorKB()
    {
        try
        {
            var sensorKB = this.hex_array[14].toString()+this.hex_array[13].toString()+this.hex_array[12].toString()+this.hex_array[11].toString();
            this.sensorKB = parseInt(sensorKB,16)/10000;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
   
    _set_model()
    {
        try
        {
            this.model = this.hex_array[9].toString();
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_serial(b1,b2,b3,b4)
    {
        try
        {
            if( b1 === undefined ) b1 = 1;
            if( b2 === undefined ) b2 = 2;
            if( b3 === undefined ) b3 = 3;
            if( b4 === undefined ) b4 = 4;
            this.serial = this.hex_array[b4].toString()+this.hex_array[b3].toString()+this.hex_array[b2].toString()+this.hex_array[b1].toString();
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_time(b1,b2,b3,b4)
    {
        try
        {
            var time = this.hex_array[b4].toString()+this.hex_array[b3].toString()+this.hex_array[b2].toString()+this.hex_array[b1].toString();
            this.time = parseInt(time,16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_release_date(b1,b2,b3,b4)
    {
        try
        {
            if( b1 === undefined ) b1 = 13;
            if( b2 === undefined ) b2 = 14;
            if( b3 === undefined ) b3 = 15;
            if( b4 === undefined ) b4 = 16;
            var time = this.hex_array[b4].toString()+this.hex_array[b3].toString()+this.hex_array[b2].toString()+this.hex_array[b1].toString();
            this.release_date = parseInt(time,16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_version_soft()
    {
        try
        {
            this.version_soft = this.hex_array[20].toString()+this.hex_array[19].toString()+this.hex_array[18].toString()+this.hex_array[17].toString();
            return true;
        }
        catch(err)
        {
            return false;
        }
    } 
    _set_universal_string_ASCII(beginByte, endByte, param)
    {
        try
        {
            var data_h = [];
            for(var i = beginByte; i<=endByte;i++)
            {
                data_h.push(String.fromCharCode('0x'+this.hex_array[i]));
            }
            data_h = data_h.join('');
            this[param] = data_h;
            return true;
        }
        catch(e)
        {
            return false;
        }
    }
    _set_data_b(before)
    {
        try
        {
            this.data_b = [];
            for(var i = this.hex_array.length-1; i>=before;i--)
            {
                this.data_b.push(this.hex_array[i]);
            }
            return true;
        }
        catch(e)
        {
            return false;
        }
    }
    _set_size_package_in()
    {
        try
        {
            var size = this.hex_array[2].toString()+this.hex_array[1].toString();
            this.size_package_in = parseInt(size,16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_count_package()
    {
        try
        {
            this.count_package = parseInt(this.hex_array[5],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_num_out()
    {
        try
        {
            this.num_out = parseInt(this.hex_array[4],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_size_package_out()
    {
        try
        {
            this.size_package_out = parseInt(this.hex_array[3],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_kt()
    {
        try
        {
            var kt = this.hex_array[22].toString()+this.hex_array[21].toString();
            this.kt = parseInt(kt,16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_relay_state()
    {
        try
        {
            var relay_state = parseInt(this.hex_array[12],16);
            this.relay_state = relay_state===1?true:false;
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_count_rate()
    {
        try
        {
            this.count_rate = parseInt(this.hex_array[11],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_count_rate_active()
    {
        try
        {
            this.count_rate_active = this.hex_array[9]!='ff'?parseInt(this.hex_array[9],16):'-';
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_note(b1,num)
    {
        try
        {
            this['note_'+num] = parseInt(this.hex_array[b1],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_period_avg(b1,num)
    {
        try
        {
            this['period_avg_'+num] = parseInt(this.hex_array[b1],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_rate_active(b)
    {
        try
        {
            if(!b) b = 10;
            this.rate_active = parseInt(this.hex_array[b],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_count_phase(b1)
    {
        try
        {
            this.count_phase = parseInt(this.hex_array[b1],16);
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_tamper()
    {
        try
        {
            var tamper = parseInt(this.hex_array[3],16);
            if (tamper==1)
            {
                this.state_tamper = false;
            }
            else
            {
                this.state_tamper = true;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_leaking()
    {
        try
        {
            var leaking = parseInt(this.hex_array[9],16);
            if (leaking==1)
            {
                this.leaking = true;
            }
            else
            {
                this.leaking = false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_breakthrough()
    {
        try
        {
            var breakthrough = parseInt(this.hex_array[10],16);
            if (breakthrough==1)
            {
                this.breakthrough = true;
            }
            else
            {
                this.breakthrough = false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_status_sensor_out(byteNum,byteVal)
    {
        try
        {
            if( byteNum == undefined ) byteNum = 2;
            if( byteVal == undefined ) byteVal = 3;
            var numSensor = parseInt(this.hex_array[byteNum],16);
            if (numSensor===1||numSensor===2)
            {
                var valueSensor = parseInt(this.hex_array[byteVal],16);
                if (valueSensor)
                {
                    this['sensor_out_'+numSensor]=true;
                }
                else
                {
                    this['sensor_out_'+numSensor]=false;
                }    
            } 
            else
            {
                return false;
            }
            return true;
        }
        catch(e)
        {
            return false;
        }
    }
    _set_hall_1()
    {
        try
        {
            var hall_1 = parseInt(this.hex_array[3],16);
            if (hall_1==1)
            {
                this.hall_1 = true;
            }
            else
            {
                this.hall_1 = false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _get_number_phaseV2(b1,val)
    {
        var result = {
            status_parse:false
        };
        try
        {
            if(val === undefined) result.hex = this.hex_array[b1];
            else result.hex = val.toString();
            result.binary = parseInt(result.hex,16).toString(2).split('' ).reverse();
            while ( result.binary.length < 8 ) result.binary.push('0');
            result.A = result.binary[0] == 1?true:false;
            result.B = result.binary[1] == 1?true:false;
            result.C = result.binary[2] == 1?true:false;
            var validABC = typeof result.A === 'boolean' && typeof result.B === 'boolean' && typeof result.C === 'boolean';
            if( validABC ) result.status_parse = true;
        }
        catch(err)
        {
            result.status_parse = false;
        }
        finally
        {
            return result;
        }
    }
    _get_number_phase(b1,val)
    {
        var result = {
            status_parse:false
        };
        try
        {
            if(val === undefined) result.hex = this.hex_array[b1];
            else result.hex = val.toString();
            result.binary = parseInt(result.hex,16).toString(2).split('' ).reverse();
            while ( result.binary.length < 8 ) result.binary.push('0');
            result.A = result.binary[0] == 1?true:false;
            result.B = result.binary[1] == 1?true:false;
            result.C = result.binary[2] == 1?true:false;
            var code_phase_binary = result.binary[5] + result.binary[4];
            var code_phase = parseInt(code_phase_binary,2);
            if( code_phase == 1 ) result.code_phase_max_deviation = 'A';
            if( code_phase == 2 ) result.code_phase_max_deviation = 'B';
            if( code_phase == 3 ) result.code_phase_max_deviation = 'C';
            result.end_event_power_lost  = result.binary[6] == 1?true:false;
            result.end_event_exec_cmd_settings = result.binary[7] == 1?true:false;
            var validABC = typeof result.A === 'boolean' && typeof result.B === 'boolean' && typeof result.C === 'boolean';
            var validCodePhase = result.code_phase_max_deviation !== undefined;
            var validEndEvent = typeof result.end_event_power_lost === 'boolean' && typeof result.end_event_exec_cmd_settings === 'boolean';
            if( validABC && validCodePhase && validEndEvent ) result.status_parse = true;
        }
        catch(err)
        {
            result.status_parse = false;
        }
        finally
        {
            return result;
        }
    }
    _get_byte_state(b1)
    {
        var result = {
            status_parse:false
        };
        try
        {
            result.hex = b1;
            result.binary = parseInt(this.hex_array[b1],16).toString(2).split('' ).reverse();
            var code_reason_binary = result.binary[0]+result.binary[1]+result.binary[2]+result.binary[3]+result.binary[4]+result.binary[5]+result.binary[6];
            result.code_reason = parseInt(code_reason_binary,2);
            result.relay_state  = result.binary[7] == 1?true:false;
            var validCodeReason= result.code_reason !== undefined && !isNaN(result.code_reason);
            var validRelayState = typeof result.relay_state === 'boolean';
            if( validCodeReason && validRelayState ) result.status_parse = true;
        }
        catch(err)
        {
            result.status_parse = false;
        }
        finally
        {
            return result;
        }
    }
    _get_universal_float_negative_noFF(arr_b,divider)
    {
        this.temp = undefined;
        this._set_universal_float_negative_noFF(arr_b,divider,'temp');
        var res = this.temp;
        this.temp = undefined;
        return res;
    }
    _get_universal_float_noFF(arr_b,divider)
    {
        this.temp = undefined;
        this._set_universal_float_noFF(arr_b,divider,'temp');
        var res = this.temp;
        this.temp = undefined;
        return res;
    }
    _get_universal_int_noFF(arr_b)
    {
        this.temp = undefined;
        this._set_universal_int_noFF(arr_b,'temp');
        var res = this.temp;
        this.temp = undefined;
        return res;
    }
    _get_universal_hex_noFF(arr_b)
    {
        this.temp = undefined;
        this._set_universal_hex(arr_b,'temp');
        var res = this.temp;
        this.temp = undefined;
        return res;
    }
    _get_universal_int_negative_noFF(arr_b)
    {
        this.temp = undefined;
        this._set_universal_int_negative_noFF(arr_b,'temp');
        var res = this.temp;
        this.temp = undefined;
        return res;
        
    }
    _set_universal_int_noFF(arr_b,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_param)
            {
                var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    if (countMAX!==arr_b.length)
                    {
                        this[param] = result_int;
                    }
                    else
                    {
                        this[param] = undefined;
                    }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_int(arr_b,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_param)
            {
                // var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    // if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    // if (countMAX!==arr_b.length)
                    // {
                        this[param] = result_int;
                    // }
                    // else
                    // {
                    //     this[param] = undefined;
                    // }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_int_negative(arr_b,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_param)
            {
                // var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    // if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    var maxVal = Math.pow(2, result.length / 2 * 8);
                    if (result_int > maxVal / 2 - 1) {
                        result_int = result_int - maxVal;
                    }
                    // if (countMAX!==arr_b.length)
                    // {
                        this[param] = result_int;
                    // }
                    // else
                    // {
                    //     this[param] = undefined;
                    // }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_int_negative_noFF(arr_b,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_param)
            {
                var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    var maxVal = Math.pow(2, result.length / 2 * 8);
                    if (result_int > maxVal / 2 - 1) {
                        result_int = result_int - maxVal;
                    }
                    if (countMAX!==arr_b.length)
                    {
                        this[param] = result_int;
                    }
                    else
                    {
                        this[param] = undefined;
                    }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_float_negative(arr_b,divider,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_divider = typeof divider === 'number';
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_divider&&valid_param)
            {
                // var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    // if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    var maxVal = Math.pow(2, result.length / 2 * 8);
                    if (result_int > maxVal / 2 - 1) {
                        result_int = result_int - maxVal;
                    }
                    // if (countMAX!==arr_b.length)
                    // {
                        this[param] = result_int/divider;
                    // }
                    // else
                    // {
                    //     this[param] = undefined;
                    // }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_float(arr_b,divider,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_divider = typeof divider === 'number';
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_divider&&valid_param)
            {
                // var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    // if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    // if (countMAX!==arr_b.length)
                    // {
                        this[param] = result_int/divider;
                    // }
                    // else
                    // {
                    //     this[param] = undefined;
                    // }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_float_negative_noFF(arr_b,divider,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_divider = typeof divider === 'number';
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_divider&&valid_param)
            {
                var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    var maxVal = Math.pow(2, result.length / 2 * 8);
                    if (result_int > maxVal / 2 - 1) {
                        result_int = result_int - maxVal;
                    }
                    if (countMAX!==arr_b.length)
                    {
                        this[param] = result_int/divider;
                    }
                    else
                    {
                        this[param] = undefined;
                    }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_float_noFF(arr_b,divider,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_divider = typeof divider === 'number';
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_divider&&valid_param)
            {
                var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                var result_int = parseInt(result,16);
                if (!isNaN(result_int))
                {
                    if (countMAX!==arr_b.length)
                    {
                        this[param] = result_int/divider;
                    }
                    else
                    {
                        this[param] = undefined;
                    }
                    return true;
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_version(arr_b,param)
    {
        var valid_arr = isObject(arr_b) && arr_b.length;
        try
        {
            if(valid_arr)
            {
                var validB1 = this.hex_array[arr_b[1]]!==undefined;
                var validB0 = this.hex_array[arr_b[0]]!==undefined;
                if(validB1&&validB0)
                {
                    this[param] = parseInt(this.hex_array[arr_b[1]])+'.'+parseInt(this.hex_array[arr_b[0]]);
                    return true;
                }
                return false;
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_version_soft_device(arr_b)
    {
        var valid_arr = isObject(arr_b) && arr_b.length;
        try
        {
            if(valid_arr)
            {
                var validB1 = this.hex_array[arr_b[1]]!==undefined;
                var validB0 = this.hex_array[arr_b[0]]!==undefined;
                if(validB1&&validB0)
                {
                    this.version_soft_device = parseInt(this.hex_array[arr_b[1]])+'.'+parseInt(this.hex_array[arr_b[0]]);
                    return true;
                }
                return false;
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    // _set_status_electronic_meter(arr_b)
    // {
    //     var valid_arr = isObject(arr_b) && arr_b.length;
    //     try
    //     {
    //         if(valid_arr)
    //         {
    //             var validB1 = this.hex_array[arr_b[1]]!==undefined;
    //             var validB0 = this.hex_array[arr_b[0]]!==undefined;
    //             if(validB1&&validB0)
    //             {
    //                 // var sw= parseInt(this.hex_array[2],16).toString(2).split('' ).reverse().splice(0,6);
    //                 // this.version_soft_device = parseInt(this.hex_array[arr_b[1]],16)+'.'+parseInt(this.hex_array[arr_b[0]],16);
    //                 return true;
    //             }
    //         }
    //         else
    //         {
    //             return false;
    //         }
    //     }
    //     catch(e)
    //     {
    //         return false;
    //     }
    // }
    _set_universal_hex(arr_b,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_param)
            {
                // var countMAX = 0;
                arr_b.reverse();
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    // if (this.hex_array[arr_b[i]]=='ff' ) countMAX++;
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                // if (countMAX!==arr_b.length)
                // {
                    this[param] = result;
                // }
                // else
                // {
                //     this[param] = undefined;
                // }
                return true;
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    _set_universal_hex2(arr_b,param)
    {
        try
        {
            var valid_arr = isObject(arr_b) && arr_b.length;
            var valid_param = typeof param === 'string';
            if (valid_arr&&valid_param)
            {
                var result='';
                for(var i = 0; i<arr_b.length;i++)
                {
                    result+=this.hex_array[arr_b[i]]===undefined?'00':this.hex_array[arr_b[i]].toString();
                }
                this[param] = result;
                return true;
            }
            else
            {
                return false;
            }
        }
        catch(e)
        {
            return false;
        }
    }
    
    _set_universal_boolean(b1,param)
    {
        try
        {
            var res = parseInt(this.hex_array[b1],16);
            if (res===1)
            {
                this[param] = true;
            }
            else
            {
                this[param] = false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    _set_display()
    {
        try
        {
            var display = parseInt(this.hex_array[4],16);
            if (display==1)
            {
                this.state_display = true;
            }
            else
            {
                this.state_display = false;
            }
            return true;
        }
        catch(err)
        {
            return false;
        }
    }
    set_package_3()
    {
        //Не используется больше
        return true;
        if (this._set_charge())
        {
            if (this._set_switch_device())
            {
                if (this._set_num_channel())
                {
                    if (this._set_count())
                    {
                         if (this._set_type_archive())
                         {
                             if (this._set_last_time())
                             {
                                this._set_archive();
                                this.comment=JSON.stringify(this);
                             }
                             else
                             {
                                 return false;
                             }
                         }
                         else
                         {
                               return false;
                         }
                    }
                    else
                    {
                         return false;
                    }             
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        else
        {
           return false;
        }
        return true;
    }
    si_11_package_1()
    {
        if (this._set_charge())
         {
             if (this._set_switch_device())
             {
                 if (this._set_time(3,4,5,6))
                 {
                     if (this._set_temperature(7))
                     {
                          if (this._set_sensors())
                          {
                               this.comment=JSON.stringify(this);
                          }
                          else
                          {
                                return false;
                          }
                     }
                     else
                     {
                          return false;
                     }             
                 }
                 else
                 {
                     return false;
                 }
             }
             else
             {
                 return false;
             }
         }
         else
         {
            return false;
         }
         return true;
    }


    si_21_or_22_package_1_rev2()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_time(3,4,5,6);
        res = res && this._set_universal_float_negative( [7,8],10,'temperature' );
        res = res && this._set_sensors(9);
        return res;
    }
    si_21_or_22_package_2_rev2()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_num_channel();
        res = res && this._set_time(4,5,6,7);
        res = res && this._set_sensors();
        return res;
    }
    si_21_or_22_package_85_rev2()
    {
        var res = true;
        if ( this.result === undefined ) this.result = {};
        this.result.count_reboot_power = this._get_universal_int_noFF([0]);
        this.result.count_reboot_soft = this._get_universal_int_noFF([1]);
        this.result.count_reboot = this._get_universal_int_noFF([2]);
        this.result.charge = this._get_universal_int_noFF([3]);
        this.result.count_reboot = this._get_universal_hex_noFF([4]);
        return res;
    }
    si_21_or_22_package_195_rev2()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'reason' );
        res = res && this._set_universal_string_ASCII(2,3,'info_radiomodule');
        res = res && this._set_universal_string_ASCII(4,5,'model');
        res = res && this._set_universal_int( [9,8,7,6],'release_date' );
        res = res && this._set_version( [10,11],'version_module_electronic' );
        res = res && this._set_version( [12,13],'version_soft_device' );
        res = res && this._set_version( [14,15],'version_parameterization' );
        res = res && this._set_universal_int( [16],'charge' );
        res = res && this._set_universal_int( [20,19,18,17],'count_package' );
        return res;
    }


    si_22_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_time(3,4,5,6);
        res = res && this._set_temperature(7);
        res = res && this._set_sensors();
        return res;
    }
    si_12_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_time(3,4,5,6);
        res = res && this._set_temperature(7);
        res = res && this._set_sensors();
        return res;
    }
    si_12_package_2()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_num_channel();
        res = res && this._set_time(4,5,6,7);
        res = res && this._set_sensors();
        return res;
    }
    si_22_package_2()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_num_channel();
        res = res && this._set_time(4,5,6,7);
        res = res && this._set_sensors();
        return res;
    }
    si_12_package_4()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_universal_boolean(3,'state_energy' );
        res = res && this._set_time(4,5,6,7);
        return res;
    }
    si_12_package_5()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_switch_device();
        res = res && this._set_status_sensor_out(3,4);
        res = res && this._set_time(5,6,7,8);
        return res;
    }
    si_13_package_1()
    {
        var res = true;
        res = res && this._set_switch_device();
        res = res && this._set_temperature(7);
        res = res && this._set_universal_int( [8,9,10,11],'sensor_7' );
        res = res && this._set_universal_int( [12,13,14,15],'sensor_8' );
        this.sensors.sensor_7=this.sensor_7;
        this.sensors.sensor_8=this.sensor_8;
        return res;
    }
    si_13rev2_package_1()
    {
        var res = true;
        res = res && this._set_switch_device();
        res = res && this._set_time(3,4,5,6);
        res = res && this._set_temperature(7);
        res = res && this._set_universal_int( [8,9,10,11],'sensor_7' );
        res = res && this._set_universal_int( [12,13,14,15],'sensor_8' );
        this.sensors.sensor_7=this.sensor_7;
        this.sensors.sensor_8=this.sensor_8;
        return res;
    }
    si_13_package_2()
    {
        var res = true;
        res = res && this._set_switch_device();
        res = res && this._set_num_channel();
        res = res && this._set_universal_int( [4,5,6,7],'sensor_7' );
        res = res && this._set_universal_int( [8,9,10,11],'sensor_8' );
        this.sensors.sensor_7=this.sensor_7;
        this.sensors.sensor_8=this.sensor_8;
        return res;
    }
    si_13rev2_package_2()
    {
        var res = true;
        res = res && this._set_switch_device();
        res = res && this._set_num_channel();
        res = res && this._set_universal_int( [4,5,6,7],'sensor_7' );
        res = res && this._set_universal_int( [8,9,10,11],'sensor_8' );
        res = res && this._set_time(12,13,14,15);
        this.sensors.sensor_7=this.sensor_7;
        this.sensors.sensor_8=this.sensor_8;
        return res;
    }
    si_13_package_3()
    {
        var res = true;
        res = res && this._set_universal_int( [1,2],'size_data' );
        res = res && this._set_universal_int( [3],'size_data_package' );
        res = res && this._set_universal_int( [4],'num_package' );
        res = res && this._set_universal_int( [5],'count_package' );
        res = res && this._set_data_b(6);
        return res;
    }
    si_13_package_4()
    {
        var res = true;
        res = res && this._set_universal_hex( [1,2,3,4],'address' );
        res = res && this._set_universal_boolean(5,'result' );
        res = res && this._set_universal_float( [6,7,8,9],1000,'sensor_rate_1' );
        res = res && this._set_universal_float( [10,11,12,13],1000,'sensor_rate_2' );
        res = res && this._set_universal_float( [14,15,16,17],1000,'sensor_rate_3' );
        res = res && this._set_universal_float( [18,19,20,21],1000,'sensor_rate_4' );
        if (!isNaN(this.sensor_rate_1)&&!isNaN(this.sensor_rate_2)&&!isNaN(this.sensor_rate_3)&&!isNaN(this.sensor_rate_4))
        {
            this.sensor_rate_sum = ((this.sensor_rate_1*100)+(this.sensor_rate_2*100)+(this.sensor_rate_3*100)+(this.sensor_rate_4*100))/100;
        }
        return res;
    }
    si_13rev2_package_4()
    {
        var res = true;
        res = res && this._set_universal_hex( [1,2,3,4],'address' );
        res = res && this._set_universal_boolean(5,'result' );
        res = res && this._set_universal_float( [6,7,8,9],1000,'sensor_rate_1' );
        res = res && this._set_universal_float( [10,11,12,13],1000,'sensor_rate_2' );
        res = res && this._set_universal_float( [14,15,16,17],1000,'sensor_rate_3' );
        res = res && this._set_universal_float( [18,19,20,21],1000,'sensor_rate_4' );
        res = res && this._set_time(22,23,24,25);
        if (!isNaN(this.sensor_rate_1)&&!isNaN(this.sensor_rate_2)&&!isNaN(this.sensor_rate_3)&&!isNaN(this.sensor_rate_4))
        {
            this.sensor_rate_sum = ((this.sensor_rate_1*100)+(this.sensor_rate_2*100)+(this.sensor_rate_3*100)+(this.sensor_rate_4*100))/100;
        }
        return res;
    }
    si_13_package_5()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'cmd_code' );
        res = res && this._set_universal_boolean(2,'result' );
        return res;
    }
    si_13rev2_package_6()
    {
        var res = true;
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_data_b(5);
        this.data_b = this.data_b.reverse();
        this.address = this.data_b[0];
        return res;
    }
    package_correction_time()
    {
        var res = true;
        res = res && this._set_time(1,2,3,4);
        return res;
    }
    package_settings()
    {
        var res = true;
        return res;
    }
    spbzip_package_1()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_time(5,6,7,8);
        res = res && this._set_model();
        res = res && this._set_count_phase(10);
        res = res && this._set_count_rate();
        // res = res && this._set_relay_state();
        res = res && this._set_release_date();
        res = res && this._set_version_soft();
        // res = res && this._set_kt();
        res = res && this._set_universal_float_noFF( [21,22,23,24],1000,'sensor_rate_sum' );
        res = res && this._set_temperature(25);
        res = res && this._set_state([26,27,28,29]);
        res = res && this._set_universal_int( [30,31],'event' );
        res = res && this._set_universal_hex2( [32,33],'UUID' );
        return res;
    }
    spbzip_package_2()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_time(5,6,7,8);
        res = res && this._set_universal_float_noFF( [9,10],100,'voltage_A' );
        res = res && this._set_universal_float_noFF( [11,12],100,'voltage_B' );
        res = res && this._set_universal_float_noFF( [13,14],100,'voltage_C' );
        res = res && this._set_universal_float_noFF( [15,16,17,18],1000,'electric_current_A' );
        res = res && this._set_universal_float_noFF( [19,20,21,22],1000,'electric_current_B' );
        res = res && this._set_universal_float_noFF( [23,24,25,26],1000,'electric_current_C' );
        res = res && this._set_universal_float_noFF( [27,28],1000,'power_factor_A' );
        res = res && this._set_universal_float_noFF( [29,30],1000,'power_factor_B' );
        res = res && this._set_universal_float_noFF( [31,32],1000,'power_factor_C' );
        res = res && this._set_universal_float_noFF( [33,34],1000,'power_factor_summary' );
        res = res && this._set_universal_float_noFF( [35,36],100,'frequency_network' );
        res = res && this._set_universal_int_noFF( [37,38,39,40],'total_power_summary' );
        res = res && this._set_universal_hex2( [41,42],'UUID' );
        return res;
    }
    spbzip_package_3()
    {
        var res = true;
        res = res && this._set_size_package_in();
        res = res && this._set_size_package_out();
        res = res && this._set_num_out();
        res = res && this._set_count_package();
        res = res && this._set_data_b(6);
        return res;
    }
    spbzip_package_4()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_time(5,6,7,8);
        // res = res && this._set_count_rate_active();
        res = res && this._set_rate_active(9);
        // res = res && this._set_universal_float( [11,12],100,'kt' );
        res = res && this._set_universal_float_noFF( [10,11,12,13],1000,'sensor_rate_sum' );
        res = res && this._set_universal_float_noFF( [14,15,16,17],1000,'sensor_rate_1' );
        res = res && this._set_universal_float_noFF( [18,19,20,21],1000,'sensor_rate_2' );
        res = res && this._set_universal_float_noFF( [22,23,24,25],1000,'sensor_rate_3' );
        res = res && this._set_universal_float_noFF( [26,27,28,29],1000,'sensor_rate_4' );
        res = res && this._set_universal_hex2( [30,31],'UUID' );
        return res;
    }
    spbzip_package_5()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_universal_int( [5,6,7,8],'date_1' );
        res = res && this._set_universal_int( [9],'note_1' );
        res = res && this._set_universal_int_noFF( [10,11,12,13],'A_p_1' );
        res = res && this._set_universal_int( [14,15,16,17],'date_2' );
        res = res && this._set_universal_int( [18],'note_2' );
        res = res && this._set_universal_int_noFF( [19,20,21,22],'A_p_2' );
        res = res && this._set_universal_hex2( [23,24],'UUID' );
        return res;
    }
    spbzip_package_6()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_universal_int_noFF( [5],'result' );
        res = res && this._set_universal_hex2( [6,7],'UUID' );
        return res;
    }
    spbzip_package_7()
    {
        var res = true;
        res = res && this._set_universal_int_noFF( [1,2,3,4],'network_address' );
        res = res && this._set_universal_int_negative( [5,6],'time_zone' );
        res = res && this._set_universal_int_noFF( [7],'period_reconnection_h' );
        res = res && this._set_universal_int_noFF( [8],'flag_event' );
        res = res && this._set_universal_int_noFF( [9],'flag_helf_hour' );
        res = res && this._set_universal_int_noFF( [10],'flag_ack' );
        res = res && this._set_universal_float_noFF( [11,12,13,14],10,'limit_power' );
        // res = res && this._set_universal_int_noFF( [15,16,17,18],'limit_energy' );

        res = res && this._set_universal_int_noFF( [19],'accumulation_period_info' );
        res = res && this._set_universal_int_noFF( [20],'accumulation_period_day_info' );
        res = res && this._set_universal_int_noFF( [21],'accumulation_period_month_info' );

        res = res && this._set_universal_int_noFF( [22],'accumulation_period_readings' );
        res = res && this._set_universal_int_noFF( [23],'accumulation_period_day_readings' );
        res = res && this._set_universal_int_noFF( [24],'accumulation_period_month_readings' );

        res = res && this._set_universal_int_noFF( [25],'accumulation_period_instant' );
        res = res && this._set_universal_int_noFF( [26],'accumulation_period_day_instant' );
        res = res && this._set_universal_int_noFF( [27],'accumulation_period_month_instant' );

        res = res && this._set_universal_hex2( [28,29],'UUID' );

        return res;
    }
    spbzip_package_8()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'tariff_schedule_season_number' );
        res = res && this._set_universal_int_noFF( [2],'tariff_schedule_code' );

        res = res && this._set_universal_int_noFF( [3],'tariff_schedule_minute_end_1_zone' );
        res = res && this._set_universal_int_noFF( [4],'tariff_schedule_hour_end_1_zone' );
        res = res && this._set_universal_int_noFF( [5],'tariff_schedule_number_1_zone' );

        res = res && this._set_universal_int_noFF( [6],'tariff_schedule_minute_end_2_zone' );
        res = res && this._set_universal_int_noFF( [7],'tariff_schedule_hour_end_2_zone' );
        res = res && this._set_universal_int_noFF( [8],'tariff_schedule_number_2_zone' );

        res = res && this._set_universal_int_noFF( [9],'tariff_schedule_minute_end_3_zone' );
        res = res && this._set_universal_int_noFF( [10],'tariff_schedule_hour_end_3_zone' );
        res = res && this._set_universal_int_noFF( [11],'tariff_schedule_number_3_zone' );

        res = res && this._set_universal_int_noFF( [12],'tariff_schedule_minute_end_4_zone' );
        res = res && this._set_universal_int_noFF( [13],'tariff_schedule_hour_end_4_zone' );
        res = res && this._set_universal_int_noFF( [14],'tariff_schedule_number_4_zone' );

        res = res && this._set_universal_int_noFF( [15],'tariff_schedule_minute_end_5_zone' );
        res = res && this._set_universal_int_noFF( [16],'tariff_schedule_hour_end_5_zone' );
        res = res && this._set_universal_int_noFF( [17],'tariff_schedule_number_5_zone' );

        res = res && this._set_universal_int_noFF( [18],'tariff_schedule_minute_end_6_zone' );
        res = res && this._set_universal_int_noFF( [19],'tariff_schedule_hour_end_6_zone' );
        res = res && this._set_universal_int_noFF( [20],'tariff_schedule_number_6_zone' );

        res = res && this._set_universal_int_noFF( [21],'tariff_schedule_minute_end_7_zone' );
        res = res && this._set_universal_int_noFF( [22],'tariff_schedule_hour_end_7_zone' );
        res = res && this._set_universal_int_noFF( [23],'tariff_schedule_number_7_zone' );

        res = res && this._set_universal_int_noFF( [24],'tariff_schedule_minute_end_8_zone' );
        res = res && this._set_universal_int_noFF( [25],'tariff_schedule_hour_end_8_zone' );
        res = res && this._set_universal_int_noFF( [26],'tariff_schedule_number_8_zone' );

        res = res && this._set_universal_int_noFF( [27],'tariff_schedule_minute_end_9_zone' );
        res = res && this._set_universal_int_noFF( [28],'tariff_schedule_hour_end_9_zone' );
        res = res && this._set_universal_int_noFF( [29],'tariff_schedule_number_9_zone' );

        res = res && this._set_universal_int_noFF( [30],'tariff_schedule_minute_end_10_zone' );
        res = res && this._set_universal_int_noFF( [31],'tariff_schedule_hour_end_10_zone' );
        res = res && this._set_universal_int_noFF( [32],'tariff_schedule_number_10_zone' );

        res = res && this._set_universal_int_noFF( [33],'tariff_schedule_minute_end_11_zone' );
        res = res && this._set_universal_int_noFF( [34],'tariff_schedule_hour_end_11_zone' );
        res = res && this._set_universal_int_noFF( [35],'tariff_schedule_number_11_zone' );

        res = res && this._set_universal_int_noFF( [36],'tariff_schedule_minute_end_12_zone' );
        res = res && this._set_universal_int_noFF( [37],'tariff_schedule_hour_end_12_zone' );
        res = res && this._set_universal_int_noFF( [38],'tariff_schedule_number_12_zone' );

        res = res && this._set_universal_int_noFF( [39],'tariff_schedule_minute_end_13_zone' );
        res = res && this._set_universal_int_noFF( [40],'tariff_schedule_hour_end_13_zone' );
        res = res && this._set_universal_int_noFF( [41],'tariff_schedule_number_13_zone' );

        res = res && this._set_universal_int_noFF( [42],'tariff_schedule_minute_end_14_zone' );
        res = res && this._set_universal_int_noFF( [43],'tariff_schedule_hour_end_14_zone' );
        res = res && this._set_universal_int_noFF( [44],'tariff_schedule_number_14_zone' );

        res = res && this._set_universal_int_noFF( [45],'tariff_schedule_minute_end_15_zone' );
        res = res && this._set_universal_int_noFF( [46],'tariff_schedule_hour_end_15_zone' );
        res = res && this._set_universal_int_noFF( [47],'tariff_schedule_number_15_zone' );

        res = res && this._set_universal_hex2( [48,49],'UUID' );

        return res;
    }
    spbzip_package_9()
    {
        var res = true;

        res = res && this._set_time(1,2,3,4);
        res = res && this._set_version_soft_device( [5,6] );
        res = res && this._set_release_date(7,8,9,10);
        // res = res && this._set_serial(11,12,13,14);
        res = res && this._set_universal_int_noFF( [11,12,13,14],'serial' );
        res = res && this._set_universal_int_noFF( [15,16,17,18],'network_address' );
        res = res && this._set_universal_string_ASCII(19,34,'location_description');
        res = res && this._set_universal_int_noFF( [35],'version_module_electronic' );
        res = res && this._set_universal_hex2( [36],'version_parameterization' );
        res = res && this._set_universal_hex2( [37,38,39],'info_radiomodule' );
        res = res && this._set_universal_float_noFF( [40,41],100,'nomenal_voltage' );
        res = res && this._set_universal_int_noFF( [42],'nomenal_current' );
        res = res && this._set_universal_int_noFF( [43],'maximal_current' );
        res = res && this._set_universal_int_noFF( [44],'code_model' );

        // res = res && this._set_universal_int_noFF( [7,8],'code_error_1' );
        // res = res && this._set_universal_int_noFF( [9,10],'code_error_2' );
        // res = res && this._set_universal_int_noFF( [11,12],'code_error_3' );
        // res = res && this._set_universal_hex( [13,14,15,16],'diagnostic_codes' );
        // res = res && this._set_universal_hex( [20,19,18,17],'serial' );
        // res = res && this._set_universal_int_noFF( [21,22,23,24],'network_address' );
        // res = res && this._set_universal_string_ASCII(25,40,'location_description');
        // res = res && this._set_universal_int_noFF( [41],'version_module_electronic' );
        // res = res && this._set_universal_int_noFF( [42],'version_parameterization' );
        // // res = res && this._set_status_electronic_meter( [43,44] );
        // res = res && this._set_universal_int_noFF( [45],'flag_relay' );
        // res = res && this._set_universal_int_noFF( [46],'flag_state_relay' );
        // res = res && this._set_universal_int_noFF( [47],'flag_translation_time' );

        res = res && this._set_universal_hex2( [45,46],'UUID' );

        return res;
    }
    spbzip_package_10()
    {
        var res = true;

        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_hex2( [5,6],'code_error_1' );
        res = res && this._set_universal_hex2( [7,8],'code_error_2' );
        res = res && this._set_universal_hex2( [9,10],'code_error_3' );
        res = res && this._set_universal_hex2( [11,12],'code_error_4' );
        res = res && this._set_universal_hex2( [13,14],'code_error_5' );
        res = res && this._set_universal_hex2( [15,16,17,18],'diagnostic_codes' );
        res = res && this._set_universal_hex2( [19,20],'status_state' );
        res = res && this._set_universal_int_noFF( [21],'flag_relay' );
        res = res && this._set_universal_int_noFF( [22],'flag_state_relay' );
        res = res && this._set_universal_int_noFF( [23],'flag_translation_time' );

        res = res && this._set_universal_hex2( [24,25],'UUID' );

        return res;
    }
    spbzip_package_11()
    {
        var res = true;

        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_float_noFF( [5,6,7,8],10,'limit_active_power' );
        res = res && this._set_universal_int_noFF( [9],'time_minute_averages_power' );
        res = res && this._set_universal_hex2( [10],'mode_measuring_power' );

        res = res && this._set_universal_hex2( [11,12],'UUID' );
        return res;
    }
    spbzip_package_12()
    {
        var res = true;

        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_float_noFF( [5,6,7,8],10,'limit_active_power' );
        res = res && this._set_universal_int_noFF( [9],'time_minute_averages_power' );
        res = res && this._set_universal_hex2( [10],'mode_measuring_power' );

        res = res && this._set_universal_int_noFF( [11],'time_minute_delay_automatic_switching_relay' );
        res = res && this._set_universal_int_noFF( [12],'count_automatic_switching_relay' );

        res = res && this._set_universal_hex2( [13,14],'UUID' );

        return res;
    }
    spbzip_package_13()
    {
        var res = true;

        res = res && this._set_time(1,2,3,4);

        res = res && this._set_universal_int_noFF( [5],'code_page_indication_1' );
        res = res && this._set_universal_int_noFF( [6],'time_seconds_show_page_indication_1' );
        res = res && this._set_universal_int_noFF( [7],'code_page_indication_2' );
        res = res && this._set_universal_int_noFF( [8],'time_seconds_show_page_indication_2' );
        res = res && this._set_universal_int_noFF( [9],'code_page_indication_3' );
        res = res && this._set_universal_int_noFF( [10],'time_seconds_show_page_indication_3' );
        res = res && this._set_universal_int_noFF( [11],'code_page_indication_4' );
        res = res && this._set_universal_int_noFF( [12],'time_seconds_show_page_indication_4' );
        res = res && this._set_universal_int_noFF( [13],'code_page_indication_5' );
        res = res && this._set_universal_int_noFF( [14],'time_seconds_show_page_indication_5' );
        res = res && this._set_universal_int_noFF( [15],'code_page_indication_6' );
        res = res && this._set_universal_int_noFF( [16],'time_seconds_show_page_indication_6' );
        res = res && this._set_universal_int_noFF( [17],'code_page_indication_7' );
        res = res && this._set_universal_int_noFF( [18],'time_seconds_show_page_indication_7' );
        res = res && this._set_universal_int_noFF( [19],'code_page_indication_8' );
        res = res && this._set_universal_int_noFF( [20],'time_seconds_show_page_indication_8' );
        res = res && this._set_universal_int_noFF( [21],'code_page_indication_9' );
        res = res && this._set_universal_int_noFF( [22],'time_seconds_show_page_indication_9' );
        res = res && this._set_universal_int_noFF( [23],'code_page_indication_10' );
        res = res && this._set_universal_int_noFF( [24],'time_seconds_show_page_indication_10' );
        res = res && this._set_universal_int_noFF( [25],'code_page_indication_11' );
        res = res && this._set_universal_int_noFF( [26],'time_seconds_show_page_indication_11' );
        res = res && this._set_universal_int_noFF( [27],'code_page_indication_12' );
        res = res && this._set_universal_int_noFF( [28],'time_seconds_show_page_indication_12' );
        res = res && this._set_universal_int_noFF( [29],'code_page_indication_13' );
        res = res && this._set_universal_int_noFF( [30],'time_seconds_show_page_indication_13' );
        res = res && this._set_universal_int_noFF( [31],'code_page_indication_14' );
        res = res && this._set_universal_int_noFF( [32],'time_seconds_show_page_indication_14' );
        res = res && this._set_universal_int_noFF( [33],'code_page_indication_15' );
        res = res && this._set_universal_int_noFF( [34],'time_seconds_show_page_indication_15' );

        res = res && this._set_universal_hex2( [35,36],'UUID' );

        return res;
    }
    
    spbzip_package_14()
    {
        var res = true;

        res = res && this._set_time(1,2,3,4);

        res = res && this._set_universal_int_noFF( [5],'code_page_indication_16' );
        res = res && this._set_universal_int_noFF( [6],'time_seconds_show_page_indication_16' );
        res = res && this._set_universal_int_noFF( [7],'code_page_indication_17' );
        res = res && this._set_universal_int_noFF( [8],'time_seconds_show_page_indication_17' );
        res = res && this._set_universal_int_noFF( [9],'code_page_indication_18' );
        res = res && this._set_universal_int_noFF( [10],'time_seconds_show_page_indication_18' );
        res = res && this._set_universal_int_noFF( [11],'code_page_indication_19' );
        res = res && this._set_universal_int_noFF( [12],'time_seconds_show_page_indication_19' );
        res = res && this._set_universal_int_noFF( [13],'code_page_indication_20' );
        res = res && this._set_universal_int_noFF( [14],'time_seconds_show_page_indication_20' );
        res = res && this._set_universal_int_noFF( [15],'code_page_indication_21' );
        res = res && this._set_universal_int_noFF( [16],'time_seconds_show_page_indication_21' );
        res = res && this._set_universal_int_noFF( [17],'code_page_indication_22' );
        res = res && this._set_universal_int_noFF( [18],'time_seconds_show_page_indication_22' );
        res = res && this._set_universal_int_noFF( [19],'code_page_indication_23' );
        res = res && this._set_universal_int_noFF( [20],'time_seconds_show_page_indication_23' );
        res = res && this._set_universal_int_noFF( [21],'code_page_indication_24' );
        res = res && this._set_universal_int_noFF( [22],'time_seconds_show_page_indication_24' );
        res = res && this._set_universal_int_noFF( [23],'code_page_indication_25' );
        res = res && this._set_universal_int_noFF( [24],'time_seconds_show_page_indication_25' );
        res = res && this._set_universal_int_noFF( [25],'code_page_indication_26' );
        res = res && this._set_universal_int_noFF( [26],'time_seconds_show_page_indication_26' );
        res = res && this._set_universal_int_noFF( [27],'code_page_indication_27' );
        res = res && this._set_universal_int_noFF( [28],'time_seconds_show_page_indication_27' );
        res = res && this._set_universal_int_noFF( [29],'code_page_indication_28' );
        res = res && this._set_universal_int_noFF( [30],'time_seconds_show_page_indication_28' );
        res = res && this._set_universal_int_noFF( [31],'code_page_indication_29' );
        res = res && this._set_universal_int_noFF( [32],'time_seconds_show_page_indication_29' );
        res = res && this._set_universal_int_noFF( [33],'code_page_indication_30' );
        res = res && this._set_universal_int_noFF( [34],'time_seconds_show_page_indication_30' );
        res = res && this._set_universal_int_noFF( [35],'code_page_indication_31' );
        res = res && this._set_universal_int_noFF( [36],'time_seconds_show_page_indication_31' );

        res = res && this._set_universal_hex2( [37,38],'UUID' );

        return res;
    }

    spbzip_package_15()
    {
        var res = true;

        res = res && this._set_universal_int( [1],'special_day_d_1' );
        res = res && this._set_universal_int( [2],'special_day_m_1' );
        res = res && this._set_universal_int( [3],'special_day_t_1' );
        res = res && this._set_universal_int( [4],'special_day_d_2' );
        res = res && this._set_universal_int( [5],'special_day_m_2' );
        res = res && this._set_universal_int( [6],'special_day_t_2' );
        res = res && this._set_universal_int( [7],'special_day_d_3' );
        res = res && this._set_universal_int( [8],'special_day_m_3' );
        res = res && this._set_universal_int( [9],'special_day_t_3' );
        res = res && this._set_universal_int( [10],'special_day_d_4' );
        res = res && this._set_universal_int( [11],'special_day_m_4' );
        res = res && this._set_universal_int( [12],'special_day_t_4' );
        res = res && this._set_universal_int( [13],'special_day_d_5' );
        res = res && this._set_universal_int( [14],'special_day_m_5' );
        res = res && this._set_universal_int( [15],'special_day_t_5' );
        res = res && this._set_universal_int( [16],'special_day_d_6' );
        res = res && this._set_universal_int( [17],'special_day_m_6' );
        res = res && this._set_universal_int( [18],'special_day_t_6' );
        res = res && this._set_universal_int( [19],'special_day_d_7' );
        res = res && this._set_universal_int( [20],'special_day_m_7' );
        res = res && this._set_universal_int( [21],'special_day_t_7' );
        res = res && this._set_universal_int( [22],'special_day_d_8' );
        res = res && this._set_universal_int( [23],'special_day_m_8' );
        res = res && this._set_universal_int( [24],'special_day_t_8' );
        res = res && this._set_universal_int( [25],'special_day_d_9' );
        res = res && this._set_universal_int( [26],'special_day_m_9' );
        res = res && this._set_universal_int( [27],'special_day_t_9' );
        res = res && this._set_universal_int( [28],'special_day_d_10' );
        res = res && this._set_universal_int( [29],'special_day_m_10' );
        res = res && this._set_universal_int( [30],'special_day_t_10' );
        res = res && this._set_universal_int( [31],'special_day_d_11' );
        res = res && this._set_universal_int( [32],'special_day_m_11' );
        res = res && this._set_universal_int( [33],'special_day_t_11' );
        res = res && this._set_universal_int( [34],'special_day_d_12' );
        res = res && this._set_universal_int( [35],'special_day_m_12' );
        res = res && this._set_universal_int( [36],'special_day_t_12' );
        res = res && this._set_universal_int( [37],'special_day_d_13' );
        res = res && this._set_universal_int( [38],'special_day_m_13' );
        res = res && this._set_universal_int( [39],'special_day_t_13' );
        res = res && this._set_universal_int( [40],'special_day_d_14' );
        res = res && this._set_universal_int( [41],'special_day_m_14' );
        res = res && this._set_universal_int( [42],'special_day_t_14' );
        res = res && this._set_universal_int( [43],'special_day_d_15' );
        res = res && this._set_universal_int( [44],'special_day_m_15' );
        res = res && this._set_universal_int( [45],'special_day_t_15' );
        res = res && this._set_universal_int( [46],'special_day_d_16' );
        res = res && this._set_universal_int( [47],'special_day_m_16' );
        res = res && this._set_universal_int( [48],'special_day_t_16' );

        res = res && this._set_universal_hex2( [49,50],'UUID' );

        return res;
    }

    spbzip_package_16()
    {
        var res = true;
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_int_noFF( [5],'selected_m_archive' );
        res = res && this._set_universal_int_noFF( [6],'selected_y_archive' );
        res = res && this._set_universal_int_noFF( [7,8,9,10],'cumulative_energy_summary' );
        res = res && this._set_universal_int_noFF( [11,12,13,14],'cumulative_energy_rate_1' );
        res = res && this._set_universal_int_noFF( [15,16,17,18],'cumulative_energy_rate_2' );
        res = res && this._set_universal_int_noFF( [19,20,21,22],'cumulative_energy_rate_3' );
        res = res && this._set_universal_int_noFF( [23,25,25,26],'cumulative_energy_rate_4' );
        
        res = res && this._set_universal_hex2( [27,28],'UUID' );

        return res;
    }

    spbzip_package_17()
    {
        var res = true;
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_int_noFF( [5],'selected_d_archive' );
        res = res && this._set_universal_int_noFF( [6],'selected_m_archive' );
        res = res && this._set_universal_int_noFF( [7],'selected_y_archive' );
        res = res && this._set_universal_int_noFF( [8,9,10,11],'cumulative_energy_summary' );
        res = res && this._set_universal_int_noFF( [12,13,14,15],'cumulative_energy_rate_1' );
        res = res && this._set_universal_int_noFF( [16,17,18,19],'cumulative_energy_rate_2' );
        res = res && this._set_universal_int_noFF( [20,21,22,23],'cumulative_energy_rate_3' );
        res = res && this._set_universal_int_noFF( [24,25,26,27],'cumulative_energy_rate_4' );

        res = res && this._set_universal_hex2( [28,29],'UUID' );

        return res;
    }

    spbzip_package_18()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        res = res && this._set_universal_int_noFF( [2,3,4,5],'selected_date' );
        if(this.num_package && this.selected_date)
        {
            if( typeof this.half_hour_slices_active_power !== 'object' ) this.half_hour_slices_active_power = [];
            if(this.num_package == 1)
            {
                this.half_hour_slices_active_power.push({
                    date_begin: moment.unix(this.selected_date).utc().hour(0).minute(0).unix(),
                    status: this._get_universal_int_noFF([6]),
                    active_power: this._get_universal_int_noFF([7,8])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(0).minute(30).unix(),
                    status: this._get_universal_int_noFF([9]),
                    active_power: this._get_universal_int_noFF([10,11])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(1).minute(0).unix(),
                    status: this._get_universal_int_noFF([12]),
                    active_power: this._get_universal_int_noFF([13,14])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(1).minute(30).unix(),
                    status: this._get_universal_int_noFF([15]),
                    active_power: this._get_universal_int_noFF([16,17])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(2).minute(0).unix(),
                    status: this._get_universal_int_noFF([18]),
                    active_power: this._get_universal_int_noFF([19,20])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(2).minute(30).unix(),
                    status: this._get_universal_int_noFF([21]),
                    active_power: this._get_universal_int_noFF([22,23])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(3).minute(0).unix(),
                    status: this._get_universal_int_noFF([24]),
                    active_power: this._get_universal_int_noFF([25,26])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(3).minute(30).unix(),
                    status: this._get_universal_int_noFF([27]),
                    active_power: this._get_universal_int_noFF([28,29])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(4).minute(0).unix(),
                    status: this._get_universal_int_noFF([30]),
                    active_power: this._get_universal_int_noFF([31,32])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(4).minute(30).unix(),
                    status: this._get_universal_int_noFF([33]),
                    active_power: this._get_universal_int_noFF([34,35])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(5).minute(0).unix(),
                    status: this._get_universal_int_noFF([36]),
                    active_power: this._get_universal_int_noFF([37,38])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(5).minute(30).unix(),
                    status: this._get_universal_int_noFF([39]),
                    active_power: this._get_universal_int_noFF([40,41])
                });
                res = res && this._set_universal_hex2( [42,43],'UUID' );
            }
            else if(this.num_package == 2)
            {
                this.half_hour_slices_active_power.push({
                    date_begin: moment.unix(this.selected_date).utc().hour(6).minute(0).unix(),
                    status: this._get_universal_int_noFF([6]),
                    active_power: this._get_universal_int_noFF([7,8])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(6).minute(30).unix(),
                    status: this._get_universal_int_noFF([9]),
                    active_power: this._get_universal_int_noFF([10,11])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(7).minute(0).unix(),
                    status: this._get_universal_int_noFF([12]),
                    active_power: this._get_universal_int_noFF([13,14])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(7).minute(30).unix(),
                    status: this._get_universal_int_noFF([15]),
                    active_power: this._get_universal_int_noFF([16,17])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(8).minute(0).unix(),
                    status: this._get_universal_int_noFF([18]),
                    active_power: this._get_universal_int_noFF([19,20])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(8).minute(30).unix(),
                    status: this._get_universal_int_noFF([21]),
                    active_power: this._get_universal_int_noFF([22,23])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(9).minute(0).unix(),
                    status: this._get_universal_int_noFF([24]),
                    active_power: this._get_universal_int_noFF([25,26])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(9).minute(30).unix(),
                    status: this._get_universal_int_noFF([27]),
                    active_power: this._get_universal_int_noFF([28,29])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(10).minute(0).unix(),
                    status: this._get_universal_int_noFF([30]),
                    active_power: this._get_universal_int_noFF([31,32])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(10).minute(30).unix(),
                    status: this._get_universal_int_noFF([33]),
                    active_power: this._get_universal_int_noFF([34,35])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(11).minute(0).unix(),
                    status: this._get_universal_int_noFF([36]),
                    active_power: this._get_universal_int_noFF([37,38])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(11).minute(30).unix(),
                    status: this._get_universal_int_noFF([39]),
                    active_power: this._get_universal_int_noFF([40,41])
                });
                res = res && this._set_universal_hex2( [42,43],'UUID' );
            }
            else if(this.num_package == 3)
            {
                this.half_hour_slices_active_power.push({
                    date_begin: moment.unix(this.selected_date).utc().hour(12).minute(0).unix(),
                    status: this._get_universal_int_noFF([6]),
                    active_power: this._get_universal_int_noFF([7,8])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(12).minute(30).unix(),
                    status: this._get_universal_int_noFF([9]),
                    active_power: this._get_universal_int_noFF([10,11])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(13).minute(0).unix(),
                    status: this._get_universal_int_noFF([12]),
                    active_power: this._get_universal_int_noFF([13,14])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(13).minute(30).unix(),
                    status: this._get_universal_int_noFF([15]),
                    active_power: this._get_universal_int_noFF([16,17])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(14).minute(0).unix(),
                    status: this._get_universal_int_noFF([18]),
                    active_power: this._get_universal_int_noFF([19,20])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(14).minute(30).unix(),
                    status: this._get_universal_int_noFF([21]),
                    active_power: this._get_universal_int_noFF([22,23])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(15).minute(0).unix(),
                    status: this._get_universal_int_noFF([24]),
                    active_power: this._get_universal_int_noFF([25,26])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(15).minute(30).unix(),
                    status: this._get_universal_int_noFF([27]),
                    active_power: this._get_universal_int_noFF([28,29])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(16).minute(0).unix(),
                    status: this._get_universal_int_noFF([30]),
                    active_power: this._get_universal_int_noFF([31,32])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(16).minute(30).unix(),
                    status: this._get_universal_int_noFF([33]),
                    active_power: this._get_universal_int_noFF([34,35])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(17).minute(0).unix(),
                    status: this._get_universal_int_noFF([36]),
                    active_power: this._get_universal_int_noFF([37,38])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(17).minute(30).unix(),
                    status: this._get_universal_int_noFF([39]),
                    active_power: this._get_universal_int_noFF([40,41])
                });
                res = res && this._set_universal_hex2( [42,43],'UUID' );
            }
            else if(this.num_package == 4)
            {
                this.half_hour_slices_active_power.push({
                    date_begin: moment.unix(this.selected_date).utc().hour(18).minute(0).unix(),
                    status: this._get_universal_int_noFF([6]),
                    active_power: this._get_universal_int_noFF([7,8])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(18).minute(30).unix(),
                    status: this._get_universal_int_noFF([9]),
                    active_power: this._get_universal_int_noFF([10,11])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(19).minute(0).unix(),
                    status: this._get_universal_int_noFF([12]),
                    active_power: this._get_universal_int_noFF([13,14])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(19).minute(30).unix(),
                    status: this._get_universal_int_noFF([15]),
                    active_power: this._get_universal_int_noFF([16,17])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(20).minute(0).unix(),
                    status: this._get_universal_int_noFF([18]),
                    active_power: this._get_universal_int_noFF([19,20])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(20).minute(30).unix(),
                    status: this._get_universal_int_noFF([21]),
                    active_power: this._get_universal_int_noFF([22,23])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(21).minute(0).unix(),
                    status: this._get_universal_int_noFF([24]),
                    active_power: this._get_universal_int_noFF([25,26])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(21).minute(30).unix(),
                    status: this._get_universal_int_noFF([27]),
                    active_power: this._get_universal_int_noFF([28,29])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(22).minute(0).unix(),
                    status: this._get_universal_int_noFF([30]),
                    active_power: this._get_universal_int_noFF([31,32])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(22).minute(30).unix(),
                    status: this._get_universal_int_noFF([33]),
                    active_power: this._get_universal_int_noFF([34,35])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(23).minute(0).unix(),
                    status: this._get_universal_int_noFF([36]),
                    active_power: this._get_universal_int_noFF([37,38])
                });
                this.half_hour_slices_active_power.push({
                    date_begin:moment.unix(this.selected_date).utc().hour(23).minute(30).unix(),
                    status: this._get_universal_int_noFF([39]),
                    active_power: this._get_universal_int_noFF([40,41])
                });
                res = res && this._set_universal_hex2( [42,43],'UUID' );
            }
        }

        return res;
    }

    spbzip_package_19()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 4; i++)
        {
            this.journal.push({
                date_begin: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                date_end: this._get_universal_int_noFF([currentByte+4,currentByte+5,currentByte+6,currentByte+7]),
                code: this._get_universal_hex_noFF([currentByte+8]),
                service_info: this._get_universal_hex_noFF([currentByte+9])
            });
            currentByte = currentByte+10;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );
        return res;
    }

    spbzip_package_20()
    {
        var res = true;
        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 3; i++)
        {
            this.journal.push({
                date_begin: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                date_end: this._get_universal_int_noFF([currentByte+4,currentByte+5,currentByte+6,currentByte+7]),
                service_info: this._get_universal_hex_noFF([currentByte+8]),
                max_value:this._get_universal_float_noFF([currentByte+9,currentByte+10],100),
                max_deviation:this._get_universal_float_negative_noFF([currentByte+11,currentByte+12],100)
            });
            currentByte = currentByte+13;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );
        return res;
    }

    spbzip_package_21()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 3; i++)
        {
            this.journal.push({
                service_info: this._get_universal_hex_noFF([currentByte]),
                date_begin: this._get_universal_int_noFF([currentByte+1,currentByte+2,currentByte+3,currentByte+4]),
                date_end: this._get_universal_int_noFF([currentByte+5,currentByte+6,currentByte+7,currentByte+8]),
                number_phase_deviation:this._get_number_phase(currentByte+9),
                max_value:this._get_universal_float_noFF([currentByte+10,currentByte+11],100),
                max_deviation:this._get_universal_float_negative_noFF([currentByte+12,currentByte+13],100)
            });
            var lastItemJournal = this.journal[this.journal.length-1];
            if( lastItemJournal.date_begin === undefined ) lastItemJournal.number_phase_deviation.status_parse = false;
            currentByte = currentByte+14;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );
        return res;
    }

    spbzip_package_22()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 3; i++)
        {
            this.journal.push({
                service_info: this._get_universal_hex_noFF([currentByte]),
                date_begin: this._get_universal_int_noFF([currentByte+1,currentByte+2,currentByte+3,currentByte+4]),
                date_end: this._get_universal_int_noFF([currentByte+5,currentByte+6,currentByte+7,currentByte+8]),
                number_phase_deviation:this._get_number_phase(currentByte+9),
                max_value:this._get_universal_float_noFF([currentByte+10,currentByte+11],100),
                max_deviation:this._get_universal_float_negative_noFF([currentByte+12,currentByte+13],100)
            });
            var lastItemJournal = this.journal[this.journal.length-1];
            if( lastItemJournal.date_begin === undefined ) lastItemJournal.number_phase_deviation.status_parse = false;
            currentByte = currentByte+14;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );
        return res;
    }

    spbzip_package_23()
    {
        var res = true;
        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 3; i++)
        {
            this.journal.push({
                service_info: this._get_universal_hex_noFF([currentByte]),
                date_begin: this._get_universal_int_noFF([currentByte+1,currentByte+2,currentByte+3,currentByte+4]),
                date_end: this._get_universal_int_noFF([currentByte+5,currentByte+6,currentByte+7,currentByte+8]),
                number_phase_deviation:this._get_number_phase(currentByte+9),
                max_value:this._get_universal_float_noFF([currentByte+10,currentByte+11],100),
                max_deviation:this._get_universal_float_negative_noFF([currentByte+12,currentByte+13],100)
            });
            var lastItemJournal = this.journal[this.journal.length-1];
            if( lastItemJournal.date_begin === undefined ) lastItemJournal.number_phase_deviation.status_parse = false;
            currentByte = currentByte+14;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );
        return res;
    }

    spbzip_package_24()
    {
        var res = true;
        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 8; i++)
        {
            this.journal.push({
                date_begin: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                byte_state: this._get_byte_state(currentByte+4)
            });
            currentByte = currentByte+5;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }

    spbzip_package_25()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 8; i++)
        {
            this.journal.push({
                date: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3])
            });
            currentByte = currentByte+4;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }

    spbzip_package_26()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 4; i++)
        {
            this.journal.push({
                date_begin: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                date_end: this._get_universal_int_noFF([currentByte+4,currentByte+5,currentByte+6,currentByte+7]),
                code: this._get_universal_hex_noFF([currentByte+8])
            });
            currentByte = currentByte+9;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }

    spbzip_package_27()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 4; i++)
        {
            this.journal.push({
                date: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                code: this._get_universal_hex_noFF([currentByte+4]),
                value: this._get_universal_int_negative_noFF([currentByte+5])
            });
            currentByte = currentByte+6;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }

    spbzip_package_28()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 8; i++)
        {
            this.journal.push({
                date: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                code: this._get_universal_hex_noFF([currentByte+4])
            });
            currentByte = currentByte+5;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }

    spbzip_package_29()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 4; i++)
        {
            this.journal.push({
                date: this._get_universal_int_noFF([currentByte,currentByte+1,currentByte+2,currentByte+3]),
                code: this._get_universal_hex_noFF([currentByte+4]),
                cmd: this._get_universal_hex_noFF([currentByte+5])
            });
            currentByte = currentByte+6;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }

    spbzip_package_30()
    {
        var res = true;

        res = res && this._set_universal_int_noFF( [1],'num_package' );
        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 4; i++)
        {
            this.journal.push({
                service_info: this._get_universal_hex_noFF([currentByte]),
                date_begin: this._get_universal_int_noFF([currentByte+1,currentByte+2,currentByte+3,currentByte+4]),
                date_end: this._get_universal_int_noFF([currentByte+5,currentByte+6,currentByte+7,currentByte+8]),
                service_info_2: this._get_universal_hex_noFF([currentByte+9])
            });
            currentByte = currentByte+10;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );

        return res;
    }
    
    spbzip_package_31()
    {
        var res = true;
        res = res && this._set_universal_int_noFF( [1],'num_package' );

        if( typeof this.journal !== 'object' ) this.journal = [];
        var currentByte = 2;
        for(var i = 0; i < 3; i++)
        {
            this.journal.push({
                service_info: this._get_universal_hex_noFF([currentByte]),
                date_begin: this._get_universal_int_noFF([currentByte+1,currentByte+2,currentByte+3,currentByte+4]),
                date_end: this._get_universal_int_noFF([currentByte+5,currentByte+6,currentByte+7,currentByte+8]),
                number_phase_deviation:this._get_number_phase(currentByte+9),
                min_value:this._get_universal_float_noFF([currentByte+10,currentByte+11],100),
                max_deviation:this._get_universal_float_negative_noFF([currentByte+12,currentByte+13],100)
            });
            var lastItemJournal = this.journal[this.journal.length-1];

            if( this.num_package == 2 && ( i===1 || i===2 ) )
            {
                lastItemJournal.number_phase_deviation = this._get_number_phaseV2(currentByte+9);
            }

            if( lastItemJournal.date_begin === undefined ) lastItemJournal.number_phase_deviation.status_parse = false;
            currentByte = currentByte+14;
        }
        res = res && this._set_universal_hex2( [currentByte,currentByte+1],'UUID' );
       
        return res;
    }
    spbzip_package_32()
    {

        var res = true;
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_int_noFF( [5,6,7,8],'active_power_A' );
        res = res && this._set_universal_int_noFF( [9,10,11,12],'active_power_B' );
        res = res && this._set_universal_int_noFF( [13,14,15,16],'active_power_C' );
        res = res && this._set_universal_int_noFF( [17,18,19,20],'active_power_summary' );

        res = res && this._set_universal_int_noFF( [21,22,23,24],'reactive_power_A' );
        res = res && this._set_universal_int_noFF( [25,26,27,28],'reactive_power_B' );
        res = res && this._set_universal_int_noFF( [29,30,31,32],'reactive_power_C' );
        res = res && this._set_universal_int_noFF( [33,34,35,36],'reactive_power_summary' );

        res = res && this._set_universal_int_noFF( [37,18,19,40],'total_power_A' );
        res = res && this._set_universal_int_noFF( [41,42,43,44],'total_power_B' );
        res = res && this._set_universal_int_noFF( [45,46,47,48],'total_power_C' );
        res = res && this._set_universal_hex2( [49,50],'UUID' );

        return res;
    }
    spbzip_package_33()
    {
        var res = true;

        res = res && this._set_universal_int( [1],'special_day_d_17' );
        res = res && this._set_universal_int( [2],'special_day_m_17' );
        res = res && this._set_universal_int( [3],'special_day_t_17' );
        res = res && this._set_universal_int( [4],'special_day_d_18' );
        res = res && this._set_universal_int( [5],'special_day_m_18' );
        res = res && this._set_universal_int( [6],'special_day_t_18' );
        res = res && this._set_universal_int( [7],'special_day_d_19' );
        res = res && this._set_universal_int( [8],'special_day_m_19' );
        res = res && this._set_universal_int( [9],'special_day_t_19' );
        res = res && this._set_universal_int( [10],'special_day_d_20' );
        res = res && this._set_universal_int( [11],'special_day_m_20' );
        res = res && this._set_universal_int( [12],'special_day_t_20' );
        res = res && this._set_universal_int( [13],'special_day_d_21' );
        res = res && this._set_universal_int( [14],'special_day_m_21' );
        res = res && this._set_universal_int( [15],'special_day_t_21' );
        res = res && this._set_universal_int( [16],'special_day_d_22' );
        res = res && this._set_universal_int( [17],'special_day_m_22' );
        res = res && this._set_universal_int( [18],'special_day_t_22' );
        res = res && this._set_universal_int( [19],'special_day_d_23' );
        res = res && this._set_universal_int( [20],'special_day_m_23' );
        res = res && this._set_universal_int( [21],'special_day_t_23' );
        res = res && this._set_universal_int( [22],'special_day_d_24' );
        res = res && this._set_universal_int( [23],'special_day_m_24' );
        res = res && this._set_universal_int( [24],'special_day_t_24' );
        res = res && this._set_universal_int( [25],'special_day_d_25' );
        res = res && this._set_universal_int( [26],'special_day_m_25' );
        res = res && this._set_universal_int( [27],'special_day_t_25' );
        res = res && this._set_universal_int( [28],'special_day_d_26' );
        res = res && this._set_universal_int( [29],'special_day_m_26' );
        res = res && this._set_universal_int( [30],'special_day_t_26' );
        res = res && this._set_universal_int( [31],'special_day_d_27' );
        res = res && this._set_universal_int( [32],'special_day_m_27' );
        res = res && this._set_universal_int( [33],'special_day_t_27' );
        res = res && this._set_universal_int( [34],'special_day_d_28' );
        res = res && this._set_universal_int( [35],'special_day_m_28' );
        res = res && this._set_universal_int( [36],'special_day_t_28' );
        res = res && this._set_universal_int( [37],'special_day_d_29' );
        res = res && this._set_universal_int( [38],'special_day_m_29' );
        res = res && this._set_universal_int( [39],'special_day_t_29' );
        res = res && this._set_universal_int( [40],'special_day_d_30' );
        res = res && this._set_universal_int( [41],'special_day_m_30' );
        res = res && this._set_universal_int( [42],'special_day_t_30' );
        res = res && this._set_universal_int( [43],'special_day_d_31' );
        res = res && this._set_universal_int( [44],'special_day_m_31' );
        res = res && this._set_universal_int( [45],'special_day_t_31' );

        res = res && this._set_universal_hex2( [46,47],'UUID' );

        return res;
    }
    ue_package_1()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_time(5,6,7,8);
        res = res && this._set_model();
        res = res && this._set_count_phase(10);
        res = res && this._set_count_rate();
        res = res && this._set_relay_state();
        res = res && this._set_release_date();
        res = res && this._set_version_soft();
        res = res && this._set_kt();
        res = res && this._set_universal_float( [23,24,25,26],1000,'sensor_rate_sum' );
        res = res && this._set_temperature(27);
        res = res && this._set_state();
        res = res && this._set_universal_int( [32],'event' );
        res = res && this._set_universal_hex( [33,34],'UUID' );
        return res;
    }
    ue_package_2()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_time(5,6,7,8);
        res = res && this._set_count_phase(9);
        res = res && this._set_universal_float_noFF( [10,11],10,'B_1' );
        res = res && this._set_universal_float_noFF( [12,13],10,'B_2' );
        res = res && this._set_universal_float_noFF( [14,15],10,'B_3' );
        res = res && this._set_universal_float_noFF( [16,17],100,'A_1' );
        res = res && this._set_universal_float_noFF( [18,19],100,'A_2' );
        res = res && this._set_universal_float_noFF( [20,21],100,'A_3' );
        res = res && this._set_universal_float_noFF( [22,23,24,25],1,'P_1' );
        res = res && this._set_universal_float_noFF( [26,27,28,29],1,'P_2' );
        res = res && this._set_universal_float_noFF( [30,31,32,33],1,'P_3' );
        res = res && this._set_universal_float_noFF( [34,35,36,37],100,'Q_1' );
        res = res && this._set_universal_float_noFF( [38,39,40,41],100,'Q_2' );
        res = res && this._set_universal_float_noFF( [42,43,44,45],100,'Q_3' );
        res = res && this._set_universal_float_noFF( [46],100,'S_1' );
        res = res && this._set_universal_float_noFF( [47],100,'S_2' );
        res = res && this._set_universal_float_noFF( [48],100,'S_3' );
        res = res && this._set_universal_hex( [49,50],'UUID' );
        return res;
    }
    
    ue_package_3()
    {
        var res = true;
        res = res && this._set_size_package_in();
        res = res && this._set_size_package_out();
        res = res && this._set_num_out();
        res = res && this._set_count_package();
        res = res && this._set_data_b(6);
        return res;
    }
    ue_package_4()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_time(5,6,7,8);
        res = res && this._set_count_rate_active();
        res = res && this._set_rate_active();
        res = res && this._set_universal_float_noFF( [11,12],100,'kt' );
        res = res && this._set_universal_float_noFF( [13,14,15,16],1000,'sensor_rate_sum' );
        res = res && this._set_universal_float_noFF( [17,18,19,20],1000,'sensor_rate_1' );
        res = res && this._set_universal_float_noFF( [21,22,23,24],1000,'sensor_rate_2' );
        res = res && this._set_universal_float_noFF( [25,26,27,28],1000,'sensor_rate_3' );
        res = res && this._set_universal_float_noFF( [29,30,31,32],1000,'sensor_rate_4' );
        res = res && this._set_universal_hex( [33,34],'UUID' );
        return res;
    }
    
    ue_package_5()
    {
        var res = true;
        res = res && this._set_serial();
        
        res = res && this._set_universal_int( [5,6,7,8],'date_1' );
        res = res && this._set_universal_int( [9],'period_avg_1' );
        res = res && this._set_universal_int( [10],'note_1' );
        res = res && this._set_universal_int_noFF( [11,12,13,14],'A_p_1' );
        res = res && this._set_universal_int_noFF( [15,16,17,18],'A_m_1' );
        res = res && this._set_universal_int_noFF( [19,20,21,22],'R_p_1' );
        res = res && this._set_universal_int_noFF( [23,24,25,26],'R_m_1' );
        res = res && this._set_universal_int( [27,28,29,30],'date_2' );
        res = res && this._set_universal_int( [31],'period_avg_2' );
        res = res && this._set_universal_int( [32],'note_2' );
        res = res && this._set_universal_int_noFF( [33,34,35,36],'A_p_2' );
        res = res && this._set_universal_int_noFF( [37,38,39,40],'A_m_2' );
        res = res && this._set_universal_int_noFF( [41,42,43,44],'R_p_2' );
        res = res && this._set_universal_int_noFF( [45,46,47,48],'R_m_2' );
        res = res && this._set_universal_hex( [49,50],'UUID' );
        return res;
    }
    ue_package_6()
    {
        var res = true;
        res = res && this._set_serial();
        res = res && this._set_universal_int( [5],'result' );
        res = res && this._set_universal_hex( [6,7],'UUID' );
        return res;
    }
    ue_package_7()
    {
        var res = true;
        return res;
    }
    ue_package_8()
    {
        var res = true;
        return res;
    }
    lm_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_time(2,3,4,5);
        res = res && this._set_temperature(6);
        res = res && this._set_universal_boolean(7,'in_move' );
        res = res && this._set_universal_float( [8,9],10,'angle' );
        res = res && this._set_universal_boolean(10,'coord_status' );
        res = res && this._set_universal_float( [11,12,13,14],1000000,'lat' );
        res = res && this._set_universal_float( [15,16,17,18],1000000,'lng' );
        res = res && this._set_universal_int( [19,20],'dir' );
        res = res && this._set_universal_int( [21,22],'speed' );
        res = res && this._set_universal_int( [23,24],'alt' );
        res = res && this._set_universal_int( [25],'sat_visible' );
        res = res && this._set_universal_int( [26],'sat_used' );
        res = res && this._set_universal_boolean( [27],'alarm' );
        return res;
    }
    src_package()
    {
        var res = true;
        res = res && this._set_universal_int( [0], 'reason' );
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_hex( [5],'status_state' );
        res = res && this._set_universal_boolean(6,'state_tamper' );
        res = res && this._set_universal_int( [7],'led_duty' );
        res = res && this._set_universal_int( [8],'charge' );
        res = res && this._set_temperature(9);
        res = res && this._set_universal_float( [10],1,'angle' );
        res = res && this._set_universal_int( [11,12],'nomenal_voltage' );
        res = res && this._set_universal_int( [13,14],'total_power_summary' );

        res = res && this._set_universal_float( [15,16,17,18],1000000,'lat' );
        res = res && this._set_universal_float( [19,20,21,22],1000000,'lng' );
        return res;
    }
    ug_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [0],'charge' );
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_temperature(5);
        res = res && this._set_universal_int( [6],'reason' );
        res = res && this._set_universal_boolean(7,'sensor_in_1' );
        res = res && this._set_universal_boolean(8,'sensor_in_2' );
        res = res && this._set_universal_boolean(9,'sensor_out_1' );
        res = res && this._set_universal_boolean(10,'sensor_out_2' );
        res = res && this._set_universal_boolean(11,'hall_1' ); 
        res = res && this._set_universal_boolean(12,'state_tamper' );
        res = res && this._set_universal_float( [13,14,15,16],100,'sensor_rate_sum' );
        res = res && this._set_universal_float( [17,18,19,20],100,'sensor_rate_0' );
        return res;
    }
    sve_1_package_1()
    {
      //  console.log('sve_1_package_1' );
        if (this._set_charge())
        {
            if (this._set_temperature(2))
            {
                if (this._set_hall_1())
                {
                    if (this._set_display())
                    {
                        if (this._set_time(5,6,7,8))
                        {
                            if (this._set_leaking())
                            {
                                if (this._set_breakthrough())
                                {
                                    if (!this._set_sensorKB())
                                    {
                                        return false;
                                    }
                                }
                                else
                                {
                                    return false;
                                }
                            }
                            else
                            {
                                return false;
                            }
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                       return false;
                    }
                }
                else
                {
                   return false;
                }
            }
            else
            {
               return false;
            }
        }
        else
        {
           return false;
        }
        return true;
    }
    tp_11_package_5()
    {
       // console.log('tp_11_package_5' );
        if (this._set_charge())
         {
             if (!this._set_status_sensor_out())
             {
                  return false;
             }
         }
         else
         {
            return false;
         }
         return true;
    }
    tp_11_package_1()
    {
       // console.log('tp_11_package_1' );
        if (this._set_charge())
         {
             if (this._set_switch_device_tp11())
             {
                if (this._set_temperature(3))
                {
                    if (this._set_reason(4))
                    {
                        if (this._set_status_tp11())
                        {
                            if (!this._set_sensorTP())
                            {
                                return false;
                            }
                        }
                        else
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                     return false;
                }    
             }
             else
             {
                 return false;
             }

         }
         else
         {
            return false;
         }
         return true;
    }
    smart_mc11rev2_package_1 ()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_universal_float_negative( [3,4],10,'temperature' );
        res = res && this._set_reason(5);
        res = res && this._set_status_smart();
        res = res && this._set_time(7,8,9,10);
        return res;
    }
    smart_ms0101Rev2_package_1 ()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_universal_float_negative( [3,4],10,'temperature' );
        res = res && this._set_reason(5);
        res = res && this._set_time(6,7,8,9);
        return res;
    }
    smart_as0101Rev2_package_1 ()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_universal_int( [2],'angle' );
        res = res && this._set_universal_float_negative( [3,4],10,'temperature' );
        res = res && this._set_reason(5);
        res = res && this._set_status_smart();
        res = res && this._set_time(7,8,9,10);
        return res;
    }
    smart_package_1()
    {
        if (this._set_charge())
         {
             if (this._set_switch_device_smart())
             {
                if (this._set_temperature_b2())
                {
                    if (this._set_reason(5))
                    {
                        if (!this._set_status_smart())
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                     return false;
                }    
             }
             else
             {
                 return false;
             }

         }
         else
         {
            return false;
         }
         return true;
    }
    tl11_package()
    {
        var res = true;
        res = res && this._set_universal_int( [0],'charge' );
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_float_negative( [5,6],10,'temperature' );
        res = res && this._set_universal_float_negative( [7,8],10,'temperature_2' );
        res = res && this._set_switch_state_tl11(9);
        return res;
    }
    td12_package()
    {
        var res = true;
        res = res && this._set_universal_int( [0],'charge' );
        res = res && this._set_time(1,2,3,4);
        res = res && this._set_universal_float_negative( [5,6],10,'temperature' );
        res = res && this._set_universal_float_negative( [7,8],10,'temperature_2' );
        res = res && this._set_switch_state_td12(9);
        return res;
    }
    td_11rev2_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_universal_boolean(2,'limit_exceeded' );
        res = res && this._set_time(3,4,5,6);
        res = res && this._set_universal_float_negative( [7,8],10,'temperature' );
        res = res && this._set_universal_int_negative( [9],'min_temperature' );
        res = res && this._set_universal_int_negative( [10],'max_temperature' );
        res = res && this._set_reason(11);
        res = res && this._set_status(12);
        return res;
        
    }
    tp_11rev2_package_5()
    {
        if (this._set_charge())
         {
            if(this._set_time(4,5,6,7))
            {
               if (!this._set_status_sensor_out())
               {
                    return false;
               }
            }
            else
            {
                return false;
            }
         }
         else
         {
            return false;
         }
         return true;
    }
    mbus_2_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_switch_device_smart();
        res = res && this._set_universal_hex( [3,4,5,6], 'address' );
        res = res && this._set_time( 7,8,9,10 );
        res = res && this._set_universal_int( [11,12,13,14], 'sensor_rate_sum' );
        res = res && this._set_universal_int( [15,16,17,18], 'total_heat_carrier_volume' );
        res = res && this._set_universal_int( [19,20,21,22], 'working_time_h' );
        res = res && this._set_universal_float_negative( [23,24],100,'temperature' );
        res = res && this._set_universal_float_negative( [25,26],100,'temperature_2' );
        res = res && this._set_universal_int( [27,28], 'current_coolant_flow' );
        return res;
    }
    mbus_1_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_switch_device_mbus();
        res = res && this._set_universal_hex( [3,4,5,6], 'address' );
        res = res && this._set_time( 7,8,9,10 );
        res = res && this._set_universal_int( [11,12,13,14], 'sensor_rate_sum' );
        res = res && this._set_universal_int( [15,16,17,18], 'total_heat_carrier_volume' );
        res = res && this._set_universal_int( [19,20,21,22], 'working_time_h' );
        res = res && this._set_universal_float_negative( [23,24],100,'temperature' );
        res = res && this._set_universal_float_negative( [25,26],100,'temperature_2' );
        res = res && this._set_universal_int( [27,28], 'current_coolant_flow' );
        return res;
    }
    mbus_1_package_3()
    {
        var res = true;
        res = res && this._set_universal_int( [1,2],'size_data' );
        res = res && this._set_universal_int( [3],'size_data_package' );
        res = res && this._set_universal_int( [4],'num_package' );
        res = res && this._set_universal_int( [5],'count_package' );
        res = res && this._set_data_b(6);
        return res;
    }
    mbus_1_package_4()
    {
        var res = true;
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_switch_device_mbus();
        res = res && this._set_universal_boolean(2,'state_energy' );
        return res;
    }
    mbus_1_package_5()
    {
        var res = true;
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_switch_device_mbus();
        res = res && this._set_num_channel();
        res = res && this._set_universal_int( [4], 'sensor_11' );
        res = res && this._set_universal_int( [5], 'sensor_12' ) ;
        this.sensors.sensor_11=this.sensor_11;
        this.sensors.sensor_12=this.sensor_12;
        return res;
    }
    gm_1_package()
    {
        var res = true;
        res = res && this._set_universal_int( [0], 'reason' );
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_time(2,3,4,5);
        res = res && this._set_temperature(6);
        res = res && this._set_universal_float( [7,8,9,10],1000, 'sensor_rate_sum' );
        return res;
    }
    gm_1_package_rev2()
    {
        var res = true;
        res = res && this._set_universal_int( [0], 'reason' );
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_time(2,3,4,5);
        res = res && this._set_temperature(6);
        res = res && this._set_universal_float( [7,8,9,10],1000, 'sensor_rate_sum' );
        res = res && this._set_universal_int( [11], 'count' );
        return res;
    }
    um0101_package()
    {
        var res = true;
        res = res && this._set_universal_int( [0], 'reason' );
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_time(2,3,4,5);
        res = res && this._set_universal_int( [6], 'type_powered' );
        res = res && this._set_universal_float_negative( [7,8],10,'temperature' );
        res = res && this._set_universal_int( [9], 'damp' );

        res = res && this._set_universal_int( [10,11], 'lux' );
        res = res && this._set_universal_int( [12], 'dB' );
        res = res && this._set_universal_int( [13,14], 'CO2' );

        res = res && this._set_universal_int( [15],'angle' );


        res = res && this._set_universal_float_negative( [16],10,'min_temperature' );
        res = res && this._set_universal_float_negative( [17],10,'max_temperature' );

        res = res && this._set_universal_int( [18], 'min_damp' );
        res = res && this._set_universal_int( [19], 'max_damp' );

        res = res && this._set_universal_int( [20,21], 'min_lux' );
        res = res && this._set_universal_int( [22,23], 'max_lux' );

        res = res && this._set_universal_int( [24], 'min_dB' );
        res = res && this._set_universal_int( [25], 'max_dB' );

        res = res && this._set_universal_float( [26],10, 'min_CO2' ); //?
        res = res && this._set_universal_float( [27],10, 'max_CO2' ); //?

        return res;
    }
    hs0101_package()
    {
        var res = true;
        res = res && this._set_universal_int( [0], 'reason' );
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_time(2,3,4,5);
        res = res && this._set_universal_float_negative( [6,7],10,'temperature' );
        res = res && this._set_universal_int( [8], 'damp' );
        res = res && this._set_universal_boolean(9,'sensor_in_1' );
        res = res && this._set_universal_boolean(10,'sensor_in_2' );
        res = res && this._set_universal_int( [11],'angle' );
        res = res && this._set_universal_int( [12],'min_sensor' );
        res = res && this._set_universal_int( [13],'max_sensor' );
        res = res && this._set_universal_int_negative( [14],'min_temperature' );
        res = res && this._set_universal_int_negative( [15],'max_temperature' );
        return res;
    }
    mbus_1_package_6()
    {
        var res = true;
        res = res && this._set_universal_int( [1], 'charge' );
        res = res && this._set_switch_device_mbus();
        res = res && this._set_universal_int( [3],'num_out_channel' );
        if( this.num_out_channel == 1 )
        {
            res = res && this._set_universal_boolean(4,'sensor_out_1' );
        }
        else if( this.num_out_channel == 2 )
        {
            res = res && this._set_universal_boolean(4,'sensor_out_2' );
        }
        return res;
    }
    tp_11rev2_package_1()
    {
        var res = true;
        res = res && this._set_universal_int( [1],'charge' );
        res = res && this._set_universal_boolean(2,'limit_exceeded' );
        res = res && this._set_time(3,4,5,6);
        res = res && this._set_temperature(7);
        res = res && this._set_universal_float_negative( [8,9],100,'min_sensor' );
        res = res && this._set_universal_float_negative( [10,11],100,'max_sensor' );
        res = res && this._set_reason(12);
        res = res && this._set_status_tp11(13);
        res = res && this._set_universal_float( [14,15],100,'sensorTP' );
        return res;
    }
    td_11_package_1()
    {
        if (this._set_charge())
         {
             if (this._set_switch_device())
             {
                if (this._set_temperature_b2())
                {
                    if (this._set_reason(5))
                    {
                        if (!this._set_status())
                        {
                            return false;
                        }
                    }
                    else
                    {
                        return false;
                    }
                }
                else
                {
                     return false;
                }    
             }
             else
             {
                 return false;
             }

         }
         else
         {
            return false;
         }
         return true;
    }
    si_11_package_2()
    {
        if (this._set_charge())
        {
            if (this._set_switch_device())
            {
                if (this._set_num_channel())
                {
                       this.comment=JSON.stringify(this);
                       if (this.hex_array.length==20)
                       {
                           this._set_sensors_opt();
                       }
                       else
                       {
                           this._set_time(4,5,6,7);
                           this._set_sensors_opt(8);
                       }
                }
                else
                {
                    return false;
                }
            }
            else
            {
                return false;
            }
        }
        else
        {
           return false;
        }
        return true;
    }
    back_door(text)
    {
        this.comment=text;
        return true;
    }
    set_data(hex)
    {
        switch (this.device_type) {
            case 1:
                if (this._set_hex(hex))
                {
                     switch(this.type_package) {
                        case 1: 
                           return this.si_11_package_1();
                        break;
                        case 2:  
                           return this.si_11_package_2();
                        break;
                        case 3:  
                           console.log('3 package is no longer used' );
                           return true;
                        break;
                        default:
                            return false;
                        break;
                     }
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 2:
                if (this._set_hex(hex))
                {
                     switch(this.type_package) {
                        case 1: 
                           return this.si_12_package_1();
                        break;
                        case 2:  
                           return this.si_12_package_2();
                        break;
                        case 3:  
                        //    return this.si_12_package_3();
                        break;
                        case 4:
                            return this.si_12_package_4();
                        break;
                        case 5:  
                           return this.si_12_package_5();
                        break;
                        default:
                            return false;
                        break;
                     }
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 11:
              //  console.log('Данные си11' );
                if (this._set_hex(hex))
                {
                    let port = this.port;
                    if( port == 4 )
                    {
                        if(this.type_package == 255) return this.package_correction_time();
                    }
                    else if( port == 3 )
                    {
                        if(this.type_package == 0) return this.package_settings();
                    }
                    if ( this.version == 0 && port == 2)
                    {
                        
                        switch(this.type_package) {
                            case 1: 
                            return this.si_11_package_1();
                            break;
                            case 2:  
                            return this.si_11_package_2();
                            break;
                            case 3:  
                            console.log('3 package is no longer used' );
                            return true;
                            break;
                            default:
                                return false;
                            break;
                        }
                    }
                    else if ( this.version == 1 )
                    {
                        if( port == 2 )
                        {
                            switch(this.type_package) {
                                case 1: 
                                    return this.si_21_or_22_package_1_rev2();
                                break;
                                case 2:  
                                    return this.si_21_or_22_package_2_rev2();
                                break;
                                default:
                                    return false;
                                break;
                            }
                        }
                        else if( port == 195 && this.type_package == 195 ) 
                        {
                            return this.si_21_or_22_package_195_rev2();
                        }
                        else if( port == 85 ) 
                        {
                            return this.si_21_or_22_package_85_rev2();
                        }
                    }
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 3:
             //   console.log('Данные си13' );
                if (this._set_hex(hex))
                {
                    let port = this.port;
                    if ( port == 2 )
                    {
                        if ( this.version == 0 )
                        {
                            switch(this.type_package) {
                                case 1: 
                                return this.si_13_package_1();
                                break;
                                case 2:  
                                return this.si_13_package_2();
                                break;
                                case 3:  
                                return this.si_13_package_3();
                                break;
                                case 4:  
                                return this.si_13_package_4();
                                break;
                                case 5:  
                                return this.si_13_package_5();
                                break;
                                default:
                                    return false;
                                break;
                            }
                        }
                        else if ( this.version == 1 )
                        {
                            switch(this.type_package) {
                                case 1: 
                                return this.si_13rev2_package_1();
                                break;
                                case 2:  
                                return this.si_13rev2_package_2();
                                break;
                                case 3:  
                                return this.si_13_package_3();
                                break;
                                case 4:  
                                return this.si_13rev2_package_4();
                                break;
                                case 5:  
                                return this.si_13_package_5();
                                break;
                                case 6:  
                                return this.si_13rev2_package_6();
                                break;
                                default:
                                    return false;
                                break;
                            }
                        }
                    }
                    else if( port == 4 )
                    {
                        if(this.type_package == 255) return this.package_correction_time();
                    }
                    else if( port == 3 )
                    {
                        if(this.type_package == 0) return this.package_settings();
                    }
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 4:
              //  console.log('Данные td-11' );
                if (this._set_hex(hex))
                {
                    if ( this.version == 0 )
                    {
                        switch ( this.type_package ) {
                            case 1: 
                               return this.td_11_package_1();
                            break;
                            default:
                                return false;
                            break;
                         }
                    }
                    else if ( this.version == 1 )
                    {
                        switch ( this.type_package ) {
                            case 1: 
                               return this.td_11rev2_package_1();
                            break;
                            default:
                                return false;
                            break;
                         }
                    }
                    return false;
                }
                else
                {
                   return false;
                }
                break;
            case 5:
            //    console.log('Данные тп11' );
                if (this._set_hex(hex))
                {
                    if ( this.version == 0 )
                    {
                        switch(this.type_package) {
                           case 1: 
                              return this.tp_11_package_1();
                           break;
                           case 5:  
                              return this.tp_11_package_5();
                           break;
                           default:
                               return false;
                           break;
                        }
                    }
                    else if ( this.version == 1 )
                    {
                        switch(this.type_package) {
                           case 1: 
                              return this.tp_11rev2_package_1();
                           break;
                           case 5:  
                              return this.tp_11rev2_package_5();
                           break;
                           default:
                               return false;
                           break;
                        }
                    }
                    return false;
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 6:
             //   console.log('Данные MC' );
                if (this._set_hex(hex))
                {
                    if ( this.version == 0 )
                    {
                        switch(this.type_package) 
                        {
                            case 1: 
                                return this.smart_package_1();
                            break;
                            default:
                                return false;
                            break;
                        }
                    }
                    else if( this.version == 1 )
                    {
                        switch(this.type_package) {
                            case 1: 
                               return this.smart_mc11rev2_package_1();
                            break;
                            default:
                                return false;
                            break;
                         }
                    }
                }
                else
                {
                   return false;
                }
                break;
            case 7:
              //  console.log('Данные AS' );
                if (this._set_hex(hex))
                {
                    if ( this.version == 0 )
                    {
                        switch(this.type_package) {
                            case 1: 
                               return this.smart_package_1();
                            break;
                            default:
                                return false;
                            break;
                        }
                    }
                    else if ( this.version == 1 )
                    {
                        switch(this.type_package) {
                            case 1: 
                               return this.smart_as0101Rev2_package_1();
                            break;
                            default:
                                return false;
                            break;
                        }
                    }
                }
                else
                {
                   return false;
                }
                break;
            case 8:
              //  console.log('Данные MS' );
                if (this._set_hex(hex))
                {
                    if ( this.version == 0 )
                    {
                        switch(this.type_package) 
                        {
                            case 1: 
                               return this.smart_package_1();
                            break;
                            default:
                                return false;
                            break;
                         }
                    }
                    else if ( this.version >= 1 )
                    {
                        switch(this.type_package) {
                            case 1: 
                               return this.smart_ms0101Rev2_package_1();
                            break;
                            default:
                                return false;
                            break;
                        }
                    }
                }
                else
                {
                   return false;
                }
                break;
            
            case 9:
             //   console.log('Данные sve1' );
                if (this._set_hex(hex))
                {
                    switch(this.type_package) {
                        case 1: 
                           return this.sve_1_package_1();
                        break;
                        default:
                            return false;
                        break;
                     }
                }
                else
                {
                   return false;
                }
                break;
            case 10:
              //  console.log('Данные SS' );
                if (this._set_hex(hex))
                {
                    switch(this.type_package) {
                        case 1: 
                           return this.smart_package_1();
                        break;
                        default:
                            return false;
                        break;
                     }
                }
                else
                {
                   return false;
                }
                break;
            case 12:
               // console.log('Данные УЭ' );
                if (this._set_hex(hex))
                {
                     switch(this.type_package) {
                        case 1: 
                           return this.ue_package_1();
                        break;
                        case 2:  
                           return this.ue_package_2();
                        break;
                        case 3:  
                           return this.ue_package_3();
                        case 4:  
                           return this.ue_package_4();
                        break;
                        case 5:  
                           return this.ue_package_5();
                        break;
                        case 6:  
                           return this.ue_package_6();
                        break;
                        default:
                            return false;
                        break;
                     }
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 13:
                if (this._set_hex(hex))
                {
                    switch(this.port) {
                        case 2: 
                           return this.ug_package_1();
                        break;
                        default:
                            return false;
                        break;
                     }
                }
                else
                {
                   return false;
                }
                break;
            case 14:
                if (this._set_hex(hex))
                {
                    switch(this.port) {
                        case 2: 
                           return this.lm_package_1();
                        break;
                        default:
                            return false;
                        break;
                     }
                }
                else
                {
                   return false;
                }
                break;
            case 15:
                if (this._set_hex(hex))
                {
                    switch(this.port) {
                        case 2: 
                            if ( this.version == 0 )
                            {
                                return this.td12_package();
                            }
                            else
                            {
                                return this.tl11_package();
                            }
                        break;
                        default:
                            return false;
                        break;
                     }
                }
                else
                {
                   return false;
                }
                break;
            case 17:
                    //   console.log('Данные gm-1' );
                    if (this._set_hex(hex))
                    {
                        if ( this.version == 0 )
                        {
                            return this.gm_1_package();
                        }
                        else
                        {
                            return this.gm_1_package_rev2();
                        }
                    }
                    else
                    {
                        return false;
                    }
                    break;
            case 18:
                if (this._set_hex(hex))
                {
                    let port = this.port;
                    if( port == 4 )
                    {
                        if(this.type_package == 255) return this.package_correction_time();
                    }
                    else if( port == 3 )
                    {
                        if(this.type_package == 0) return this.package_settings();
                    }
                    if ( this.version == 0 && port == 2)
                    {
                        switch(this.type_package) {
                            case 1: 
                                return this.si_22_package_1();
                            break;
                            case 2:  
                                return this.si_22_package_2();
                            break;
                            default:
                                return false;
                            break;
                        }
                    }
                    else if ( this.version == 1 )
                    {
                        if( port == 2 )
                        {
                            switch(this.type_package) {
                                case 1: 
                                    return this.si_21_or_22_package_1_rev2();
                                break;
                                case 2:  
                                    return this.si_21_or_22_package_2_rev2();
                                break;
                                default:
                                    return false;
                                break;
                            }
                        }
                        else if( port == 195 && this.type_package == 195 ) 
                        {
                            return this.si_21_or_22_package_195_rev2();
                        }
                        else if( port == 85 ) 
                        {
                            return this.si_21_or_22_package_85_rev2();
                        }
                    }
                 }
                 else
                 {
                    return false;
                 }
                break;
            case 20:
                //   console.log('Данные m-bus-1' );
                if (this._set_hex(hex))
                {
                    switch(this.type_package) {
                        case 1: 
                            return this.mbus_1_package_1();
                            break;
                        case 3:  
                            return this.mbus_1_package_3();
                            break;
                        case 4:  
                            return this.mbus_1_package_4();
                            break;
                        case 5:  
                            return this.mbus_1_package_5();
                            break;
                        case 6:  
                            return this.mbus_1_package_6();
                            break;
                        default:
                            return false;
                        break;
                    }
                }
                else
                {
                    return false;
                }
                break;
            case 21:
                //   console.log('Данные m-bus-2' );
                   if (this._set_hex(hex))
                   {
                        switch(this.type_package) {
                           case 1: 
                              return this.mbus_2_package_1();
                              break;
                           default:
                               return false;
                           break;
                        }
                    }
                    else
                    {
                       return false;
                    }
                   break;
                case 23:
                //   console.log('Данные hs0101' );
                    if (this._set_hex(hex))
                    {
                        return this.hs0101_package();
                    }
                    else
                    {
                        return false;
                    }
                    break;
                case 24:
                    // console.log('Данные spbzip' );
                    if (this._set_hex(hex))
                    {
                        let port = this.port;
                        let type_package = this.type_package;
                        if( port == 2 )
                        {
                            if(type_package == 1) return this.spbzip_package_1();
                            if(type_package == 2) return this.spbzip_package_2();
                            if(type_package == 3) return this.spbzip_package_3();
                            if(type_package == 4) return this.spbzip_package_4();
                            if(type_package == 5) return this.spbzip_package_5();
                            if(type_package == 6) return this.spbzip_package_6();
                            if(type_package == 7) return this.spbzip_package_7();
                            if(type_package == 32) return this.spbzip_package_32();
                        }
                        else if( port == 5 )
                        {
                            if(this.type_package == 8) return this.spbzip_package_8();
                            if(this.type_package == 9) return this.spbzip_package_9();
                            if(this.type_package == 10) return this.spbzip_package_10();
                            if(this.type_package == 11) return this.spbzip_package_11();
                            if(this.type_package == 12) return this.spbzip_package_12();
                            if(this.type_package == 13) return this.spbzip_package_13();
                            if(this.type_package == 14) return this.spbzip_package_14();
                            if(this.type_package == 15) return this.spbzip_package_15();
                            if(this.type_package == 33) return this.spbzip_package_33();
                        }
                        else if( port == 6 )
                        {
                            if(this.type_package == 16) return this.spbzip_package_16();
                            if(this.type_package == 17) return this.spbzip_package_17();
                            if(this.type_package == 18) return this.spbzip_package_18();
                        }
                        else if( port == 7 )
                        {
                            if(this.type_package == 19) return this.spbzip_package_19();
                            if(this.type_package == 20) return this.spbzip_package_20();
                            if(this.type_package == 21) return this.spbzip_package_21();
                            if(this.type_package == 22) return this.spbzip_package_22();
                            if(this.type_package == 23) return this.spbzip_package_23();
                            if(this.type_package == 24) return this.spbzip_package_24();
                            if(this.type_package == 25) return this.spbzip_package_25();
                            if(this.type_package == 26) return this.spbzip_package_26();
                            if(this.type_package == 27) return this.spbzip_package_27();
                            if(this.type_package == 28) return this.spbzip_package_28();
                            if(this.type_package == 29) return this.spbzip_package_29();
                            if(this.type_package == 30) return this.spbzip_package_30();
                            if(this.type_package == 31) return this.spbzip_package_31();
                        }
                        else if( port == 4 )
                        {
                            if(this.type_package == 255) return this.package_correction_time();
                        }
                        else if( port == 3 )
                        {
                            if(this.type_package == 0) return this.package_settings();
                        }
                    }
                return false;
                break;
            case 25:
                //   console.log('Данные um0101' );
                let port = this.port;
                if ( this._set_hex(hex) && port == 2 )
                {
                    return this.um0101_package();
                }
                else
                {
                    return false;
                }
                break;
            case 26:
                if (this._set_hex(hex))
                {
                    let port = this.port;
                    if( port == 2 )
                    {
                        return this.src_package();
                    }
                    else if( port == 4 )
                    {
                        if(this.type_package == 255) return this.package_correction_time();
                    }
                    else if( port == 3 )
                    {
                        if(this.type_package == 0) return this.package_settings();
                    }   
                }
                else
                {
                    return false;
                }
                break;
            default:
                console.log('Данные неизвестного для типа' );
                return false;
                break;
        }
    }
}
class device extends vega_converter
{
    constructor()
    {
        super();
        this.devEui=0;
        this.device_type=0;
        this.version = undefined;
        this.count_channels=0;
        this.trans_period=0;
        this.lat=0;
        this.lng=0;
        this.devName="";
        this.last_data_ts="";
        this.last_data_charge=false;
        this.other_info;
        this.channels=new Array();
        this.settings = 
        {
            ts:0,
            params:new Object()
        };
        this.tempSettings = new Object();
    }
    refreshTempSetting()
    {
        var jsonSettings = JSON.stringify(this.settings);
        this.tempSettings = JSON.parse(jsonSettings);        
    }
    getHexTempSettings(channel,settings)
    {
        var resultHex = '00';
        if ( isObject(this.tempSettings.params) )
        {
            for(var i = 0; i < settings.length; i++)
            {
                var id = settings[i];
                var param = this.tempSettings.params[id];
                if (param.id===undefined)
                {
                    param.id = parseInt( id );
                }
                var hexParam = this.encodeParamSettings(param,channel);
                resultHex += hexParam;
            }
        }
        var countBytes = Math.round(resultHex.length/2);
        if ((countBytes)>51)
        {
            return '';
        }
        return resultHex;
    }
    encodeParamSettings(param, channel)
    {
        var paramHex = '';
        var id = param.id;
        var value = param.value;
        var hexValue = '';
        switch ( id ) {
            case 50:
                    var hexValue0 = this.intToHex(value[0],1);
                    var hexValue1 = this.intToHex(value[1],1);
                    var hexValue2 = this.intToHex(value[2],1);
                    hexValue = hexValue0+hexValue1+hexValue2;
                break;
            case 52:
                    var hexValue0 = this.intToHex(value[0],1);
                    var hexValue1 = this.intToHex(value[1],1);
                    var hexValue2 = this.intToHex(value[2],1);
                    hexValue = hexValue0+hexValue1+hexValue2;
                break;
            case 54:
                    hexValue = this.uIntToHex(value,4);
                break;
            case 55:
                    hexValue = this.intToHex(value,2);
                break;
            case 80:
                    hexValue = this.intToHex(value,1);
                break;
            case 81:
                    hexValue = this.intToHex(value,1);
                break;
            case 84:
                    hexValue = this.uIntToHex(value,4);
                break;
            case 85:
                    hexValue = this.uIntToHex(value,2);
                break;
            case 86:
                    hexValue = this.uIntToHex(value,2);
                break;
            case 88:
                    hexValue = this.uIntToHex(value);
                break;
            case 89:
                    hexValue = this.uIntToHex(value);
                break;

            case 115:
                    hexValue = this.uIntToHex(value,1);
                break;
            case 116:
                    hexValue = this.uIntToHex(value,1);
                break;
            case 117:
                    hexValue = this.uIntToHex(value,2);
                break;
            case 118:
                    hexValue = this.uIntToHex(value,2);
                break;
            case 119:
                    hexValue = this.uIntToHex(value,2);
                break;
            case 120:
                    hexValue = this.uIntToHex(value,2);
                break;
                
            case 124:
                    hexValue = this.uIntToHex(value,1);
                break;
            default:
                    hexValue = this.uIntToHex(value);
                break;
        }
        var hexId = this.intToBytes(id,2);
        var size = Math.round(hexValue.length/2);
        var hexSize = this.intToBytes(size,1);
        var validHexSize = hexSize&&hexSize.length%2===0&&size<=255;
        var validHexId = hexId&&hexId.length%2===0&&id<=(256*256-1);
        var validHexValue = hexValue&&hexValue.length%2===0;
        if (validHexSize&&validHexValue&&validHexId)
        {
            paramHex=hexId+hexSize+hexValue;
        }
        return paramHex;
    }
    setSettings(data)
    {
        if (data.ts<=this.settings.ts) 
        {
            return false;
        }
        
        for(var key in data)
        {
            this.settings[key] = data[key];
        }
        
        var hex = data.data;
        this.settings.bytesList = this.hexToBytes(hex);
        var cloneBytesList = cloneArr(this.settings.bytesList);
        var paramsSettings = this.bytesToParamsSettings(cloneBytesList);
        for(var key in paramsSettings)
        {
            var paramSettings = paramsSettings[key];
            var value = this.parseParamSettings(paramSettings.id,paramSettings.rawData);
            paramSettings.value = value.value;
            paramSettings.name = value.name;
        }
        this.settings.params = paramsSettings;
        return true;
    }
    parseParamSettings(id,bytes)
    {
        var result = 
        {
            value : undefined,
            name : "unknown"
        }
        if (typeof id === 'number' && isObject(bytes) )
        {
            switch ( id ) 
            {
                case 1:
                    result.name='SETTING_FREQ_PLAN';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 2:
                    result.name='SETTING_CUSTOM_FREQ_PLAN';
                    break;
                case 3:
                    result.name='SETTING_ACTIVATION_TYPE';
                    break;
                case 4:
                    result.name='SETTING_CONFIRMATION';
                    result.value = this.bytesToUint(bytes);
                    //0 или 1
                    break;
                case 5:
                    result.name='SETTING_ADR';
                    result.value = this.bytesToUint(bytes);
                    //0 или 1
                    break;
                case 6:
                    result.name='SETTING_RX1_DELAY';
                    break;
                case 7:
                    result.name='SETTING_JOIN1_DELAY';
                    break;
                case 8:
                    result.name='SETTING_NB_TRANS';
                    result.value = this.bytesToUint(bytes);
                    //1-15
                    break;
                case 9:
                    result.name='SETTING_TX_POWER';
                    break;
                case 10:
                    result.name='SETTING_DATARATE';
                    break;
                case 11:
                    result.name='SETTING_CHANNEL_MASK';
                    break;
                case 12:
                    result.name='SETTING_INPUT1_MODE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 13:
                    result.name='SETTING_INPUT2_MODE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 14:
                    result.name='SETTING_INPUT3_MODE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 15:
                    result.name='SETTING_INPUT4_MODE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 16:
                    result.name='SETTING_TRANS_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    //1 - 1H
                    //2 - 6H
                    //3 - 12H
                    //4 - 24H
                    //5 - 5m
                    //6 - 15m
                    //7 - 30m
                    break;
                case 17:
                    result.name='SETTING_OUTPUT1_MODE';
                    break;
                case 18:
                    result.name='SETTING_OUTPUT2_MODE';
                    break;
                case 19:
                    result.name='SETTING_PLUGGED_DEVICE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 20:
                    result.name='SETTING_INTERFACE_SPEED';
                    break;
                case 21:
                    result.name='SETTING_INTERFACE_ANS_TIMEOUT';
                    break;
                case 22:
                    result.name='PLUGGED_DEVICE_ADDRESS';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 23:
                    result.name='PLUGGED_DEVICE2_ADDRESS';
                    break;
                case 24:
                    result.name='PLUGGED_DEVICE3_ADDRESS';
                    break;
                case 25:
                    result.name='PLUGGED_DEVICE4_ADDRESS';
                    break;
                case 26:
                    result.name='PLUGGED_DEVICE5_ADDRESS';
                    break;
                case 27:
                    result.name='PLUGGED_DEVICE6_ADDRESS';
                    break;
                case 28:
                    result.name='PLUGGED_DEVICE7_ADDRESS';
                    break;
                case 29:
                    result.name='PLUGGED_DEVICE8_ADDRESS';
                    break;
                case 30:
                    result.name='PLUGGED_DEVICE9_ADDRESS';
                    break;
                case 31:
                    result.name='PLUGGED_DEVICE10_ADDRESS';
                    break;
                case 32:
                    result.name='SETTING_MBUS_INTERFACE_SPEED';
                    break;
                case 33:
                    result.name='SETTING_MBUS_DEVICE_TYPE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 34:
                    result.name='SETTING_INTERFACE_DATABITS';
                    break;
                case 35:
                    result.name='SETTING_INTERFACE_STOPBITS';
                    break;
//                case 36:
//                    result.name='SETTING_INTERFACE_STOPBITS';
//                    break;
                case 37:
                    result.name='SETTING_INTERFACE_PARITY';
                    break;
                case 38:
                    result.name='SETTING_INPUT1_GUARD_MODE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 39:
                    result.name='SETTING_INPUT2_GUARD_MODE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 40:
                    result.name='SETTING_INPUT3_GUARD_MODE';
                    break;
                case 41:
                    result.name='SETTING_INPUT4_GUARD_MODE';
                    break;
                case 42:
                    result.name='SETTING_TEMP_SENSOR_TYPE';
                    break;
                case 43:
                    result.name='SETTING_ALARM_THRESHOLD';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 44:
                    result.name='SETTING_SENSOR_SENSIVITY';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 45:
                    result.name='SETTING_SVE_TRANS_PERIOD';
                    break;
                case 46:
                    result.name='SETTING_SVE_MONTH_DAY';
                    break;
                case 47:
                    result.name='SETTING_SVE_WEEK_DAY';
                    break;
                case 48:
                    result.name='SETTING_TP_HEAT_TIME';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 49:
                    result.name='SETTING_COLLECTION_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    //1 - 1H
                    //2 - 6H
                    //3 - 12H
                    //4 - 24H
                    //5 - 5m
                    //6 - 15m
                    //7 - 1m
                    break;
                case 50:
                    result.name='SETTING_ELECTRO_INFO_PACKET_PERIOD';
                    var b1 = this.bytesToUint([bytes[0]]);
                    var b2 = this.bytesToUint([bytes[1]]);
                    var b3 = this.bytesToUint([bytes[2]]);
                    result.value = [b1,b2,b3];
                    break;
                case 51:
                    result.name='SETTING_ELECTRO_INSTANT_PACKET_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 52:
                    result.name='SETTING_ELECTRO_ENERGY_PACKET_PERIOD';
                    var b1 = this.bytesToUint([bytes[0]]);
                    var b2 = this.bytesToUint([bytes[1]]);
                    var b3 = this.bytesToUint([bytes[2]]);
                    result.value = [b1,b2,b3];
                    break;
                case 53:
                    result.name='SETTING_ELECTRO_ENABLE_POWER_PROFILE';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 54:
                    result.name='SETTING_ELECTRO_METER_PASSWORD';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 55:
                    result.name='SETTING_TIMEZONE';
                    result.value = this.bytesToInt(bytes);
                    break;
                case 56:
                    result.name='SETTING_START_READINGS';
                    break;
                case 57:
                    result.name='SETTING_MODIFIED_APPEUI';
                    break;
                case 58:
                    result.name='SETTING_MODIFIED_APPKEY';
                    break;
                case 59:
                    result.name='SETTING_MODIFIED_NWKSKEY';
                    break;
                case 60:
                    result.name='SETTING_MODIFIED_APPSKEY';
                    break;
                case 61:
                    result.name='SETTING_MODIFIED_DEVADDR';
                    break;
                case 62:
                    result.name='SETTING_LM_MOVE_COLLECTION_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 63:
                    result.name='SETTING_LM_MOVE_TRANS_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    //1 - 1H
                    //2 - 6H
                    //3 - 12H
                    //4 - 24H
                    //5 - 5m
                    //6 - 15m
                    //7 - 30m
                    break;
                case 64:
                    result.name='SETTING_GSM_TX_TECHNOLOGY';
                    break;
                case 65:
                    result.name='SETTING_GSM_NETWORK_MODE';
                    break;
                case 66:
                    result.name='SETTING_GSM_HOST_IP';
                    break;
                case 67:
                    result.name='SETTING_GSM_HOST_PORT';
                    break;
                case 68:
                    result.name='SETTING_GSM_TOPIC_NAME';
                    break;
                case 69:
                    result.name='SETTING_GSM_CLIENT_ID';
                    break;
                case 70:
                    result.name='SETTING_GAS_DECIMAL_COUNT';
                    break;
                case 71:
                    result.name='SETTING_LM_MOVE_ALARM';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 72:
                    result.name='SETTING_ELECTRO_ENABLE_SEND_EVENTS';
                    break;
                case 73:
                    result.name='SETTING_GNSS_MODULE';
                    break;
                case 74:
                    result.name='SETTING_GSM_CONNECT_APN';
                    break;
                case 75:
                    result.name='SETTING_GSM_CONNECT_USERNAME';
                    break;
                case 76:
                    result.name='SETTING_GSM_CONNECT_PASSWORD';
                    break;
                case 77:
                    result.name='SETTING_GSM_CONNECT_PROTOCOL';
                    break;
                case 78:
                    result.name='SETTING_ALARM_COLLECTION_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 79:
                    result.name='SETTING_INSTANT_SEND';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 80:
                    result.name='SETTING_TEMPERATURE_LIMIT_LOW';
                    result.value = this.bytesToInt(bytes);
                    break;
                case 81:
                    result.name='SETTING_TEMPERATURE_LIMIT_HIGH';
                    result.value = this.bytesToInt(bytes);
                    break;
                case 82:
                    result.name='SETTING_SMOKE_THRESHOLD';
                    break;
                case 83:
                    result.name='SETTING_LIMIT_ENERGY';
                    break;
                case 84:
                    result.name='SETTING_LIMIT_POWER';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 85:
                    result.name='SETTING_LIMIT_LOW';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 86:
                    result.name='SETTING_LIMIT_HIGH';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 88:
                    result.name='SETTING_HUMID_LIMIT_LOW';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 89:
                    result.name='SETTING_HUMID_LIMIT_HIGH';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 114:
                    result.name='SETTING_ELECTRO_TRANS_PERIOD';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 115:
                    result.name='SETTING_NOISE_THRESHOLD_LOW';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 116:
                    result.name='SETTING_NOISE_THRESHOLD_HI';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 117:
                    result.name='SETTING_LIGHT_THRESHOLD_LOW';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 118:
                    result.name='SETTING_LIGHT_THRESHOLD_HI';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 119:
                    result.name='SETTING_CO2_THRESHOLD_LOW';
                    result.value = this.bytesToUint(bytes);
                    break;
                case 120:
                    result.name='SETTING_CO2_THRESHOLD_HI';
                    result.value = this.bytesToUint(bytes);
                    break;

                case 124:
                    result.name='SETTING_AUTOARMING_MESSAGE_ENABLE';
                    result.value = this.bytesToInt(bytes);
                    break;
                default:
                    break;
            }
        }
        
        return result;
    }
    valid(str)
    {
       return str&&str!=undefined&&str.toString().length>0?true:false;
    }
    set_device(obj,deviceType)
    {   
        var count_channels = undefined;
        if( isObject(deviceType) )
        {
            count_channels = deviceType.count_channels;
            this.device_type=obj.device_type;
            this.version=obj.version;
            
            if( this.version === undefined && isObject(deviceType.versions) )
            {
                this.version = deviceType.versions.first;
            }
        }
        if (this.valid(obj.devEui))
        {
            this.devEui=obj.devEui;
        }
        else
        {
            return false;
        }
        if (count_channels!=undefined&&count_channels>0)
        {
            if (count_channels>50)
            {
                count_channels=50;
            }
            for (var i =0;i<count_channels;i++)
            {
                if (obj['other_info_'+(i+1)]&&obj['other_info_'+(i+1)]!=undefined)
                {
                    var mychanneel_obj = {};
                    try
                    {
                        if(typeof obj['other_info_'+(i+1)] === 'object' )
                        {
                            mychanneel_obj = obj['other_info_'+(i+1)];
                        }
                        else if(typeof obj['other_info_'+(i+1)] === 'string')
                        {
                            // if(this.devEui=='3036323168374F14')
                            // {
                            //     console.log(1);
                            // }
                            //obj['other_info_'+(i+1)] = obj['other_info_'+(i+1)].replace(/\\/g,"\/");
                            // obj['other_info_'+(i+1)] = obj['other_info_'+(i+1)].replace('ООО "Продукты"','ООО <<Продукты>>');    
                            mychanneel_obj = JSON.parse(obj['other_info_'+(i+1)])
                        }
                        this.channels[i] = mychanneel_obj;
                        this.channels[i].history = new Array();
                        this.channels[i].statistics=new Statistic();
                    }
                    catch(err)
                    {
                         this.channels[i]=obj['other_info_'+(i+1)];
                         this.channels[i].history = new Array();
                         this.channels[i].statistics=new Statistic();
                    }
                }
                else
                {
                    this.channels[i]=new Object();
                    this.channels[i].history = new Array();
                    this.channels[i].statistics=new Statistic();
                    this.channels[i].num_channel=i+1;
                }
                this.channels[i]._self = this;
                this.channels[i].edit=new Channel().edit;
                this.channels[i].getInfo=new Channel().getInfo;
                this.channels[i].devEui = this.devEui;
                this.channels[i].last_data_charge = this.last_data_charge;
            }
        }
        if (this.valid(obj.trans_period))
        {
            this.trans_period=obj.trans_period;
        }
        if (obj.last_data_ts!=undefined&&obj.last_data_ts>0&&obj.last_data_ts.toString().length<=13)
        {
            this.last_data_ts=obj.last_data_ts;
        }
        if (this.valid(obj.devName))
        {
            this.devName=obj.devName;
        }
        if (this.valid(obj.lat))
        {
            this.lat=obj.lat;
        }
        if (this.valid(obj.lng))
        {
            this.lng=obj.lng;
        }
        if (this.valid(obj.other_info))
        {
            try{
                this.other_info=JSON.parse(obj.other_info);
            }
            catch(err)
            {
                
            }
        }
        return this;
    }
    get_json()
    {
        try
        {
            return JSON.stringify(this);
        }
        catch(err)
        {
            return false;
        }
    }
}
class audioControl 
{
    constructor(tag,status)
    {
        this.audio = document.getElementsByTagName(tag)[0];
        this.audio.status = status;
    }
    play()
    {
        if( !this.valid() ) return false;
        this.audio.play().catch(function() {
            console.log('play audio error');
        });
    }
    pause()
    {
        if( !this.valid() ) return false;
        this.audio.pause();
    }
    currentTime(time)
    {
        if( !this.valid() ) return false;
        if( time === undefined ) time = 0;
        this.audio.currentTime = time;
    }
    stop()
    {
        if( !this.valid() ) return false;
        this.pause();
        this.currentTime();
    }
    replayAudio()
    {
        if( !this.valid() ) return false;
        this.stop();
        this.play();
    }
    toggleMutedAudio()
    {
        if( !this.valid() ) return false;
        if( this.audio.muted ) this.audio.muted = false;
        this.audio.muted = true;
    }
    valid()
    {
        return isObject(this.audio) && this.audio.status ;
    }
}
class map_nav extends audioControl
{
    constructor(tagAudio,stateAudio)
    {
        super(tagAudio,stateAudio);
        this.level_1_list = new Array();
        this.level_2_list = new Array();
        this.level_2_filter_list = new Array();
        this.level_3_list = new Array();
        this.check_level_1=-1;//Выбранный объект
        this.check_level_2=-1;//Выбранное помещение
        this.history_danger = new Array();
        this.check_level_1_arr = [];
        this.check_level_2_arr = [];
        this.list = new Object();
        this.filterDevices = '';
        this.filter = new Object();
    }
    checkFilter()
    {
        for(var key in this.filter)
        {
            if(this.filter[key])
            {
                return true;
            }
        }
        return false;
    }
    //запись в массив истории тут,  и функции поиска по лев 1 лев2  и тд и збс
    set_history_danger(devEui,num,name_level_1,address_level_1,level_2)
    {
        var tempObj = new Object();
        tempObj.devEui = devEui;
        tempObj.num = num;
        tempObj.name_level_1 = name_level_1;
        tempObj.address_level_1 = address_level_1;
        tempObj.level_2 = level_2;
        this.history_danger.push(tempObj);
        this.replayAudio();
    }
    get_level_1_danger(name_level_1,address_level_1)
    {
        var countDanger = 0;
        for (var i=0;i< this.history_danger.length;i++)
        {
            if (this.history_danger[i].name_level_1==name_level_1&&this.history_danger[i].address_level_1==address_level_1)
            {
                countDanger++;
            }
        }
        return countDanger;
    }
    get_level_2_danger(level_2,name_level_1,address_level_1)
    {
        var countDanger = 0;
        for (var i=0;i< this.history_danger.length;i++)
        {
            if (this.history_danger[i].level_2==level_2&&this.history_danger[i].name_level_1==name_level_1&&this.history_danger[i].address_level_1==address_level_1)
            {
                countDanger++;
            }
        }
        return countDanger;
    }
    checkSelectedLevel_1(level_1)
    {
        var validLevel_1 = level_1!==undefined&&level_1.name!==undefined&&level_1.address!==undefined;
        if (validLevel_1)
        {
            var key = level_1.name.toString()+level_1.address.toString();
            var chackLevelSelected = this.check_level_1_arr[key]!==undefined;
            if (chackLevelSelected)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
     checkSelectedLevel_2(level_2)
    {
        var validLevel_2 = level_2!==undefined;
        if (validLevel_2)
        {
            var key = level_2.name.toString();
            var chackLevelSelected = this.check_level_2_arr[key]!==undefined;
            if (chackLevelSelected)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
    select_check_level_2(level_2)
    {
        var validLevel_2 = level_2!==undefined;
        if (validLevel_2)
        {
            var key = level_2.name.toString();
            var chackLevelSelected = this.check_level_2_arr[key]!==undefined;
            if (chackLevelSelected)
            {
                delete this.check_level_2_arr[key];
            }
            else
            {
                this.check_level_2_arr[key] = level_2;
            }
        }
    }
    set_danger_level_1()
    {
        
    }
    level_2_list_f(level_1)
    {
        var res = [];
        var validL1 = level_1&& isObject(level_1) &&level_1.name_level_1!==undefined&&level_1.address_level_1!==undefined;
        if (validL1)
        {
            var nameL1 = level_1.name_level_1;
            var addressL1 = level_1.address_level_1;
            var hashL1 = md5(nameL1+addressL1);
            var l1 = this.list[hashL1];
            if ( l1 && isObject(l1) && l1.list && isObject(l1.list) )
            {
                for (var key in l1.list)
                {
                    var l2 = l1.list[key];
                    if ( l2 && isObject(l2) && l2.name )
                    {
                        res.push(l2.name);
                    }
                }
                return res;
            }
        }
        return this.level_2_list;
    }
    add(channel)
    {
        if (channel)
        {
            var nameL1 = channel.name_level_1;
            var addressL1 = channel.address_level_1;
            var nameL2 = channel.level_2;
            var nameL3 = channel.name;
            if (!nameL1||!addressL1||!nameL2||!nameL3) return;
            var hashL1 = md5(nameL1+addressL1);
            var hashL2 = md5(nameL2);
            var hashL3 = md5(nameL3+channel.num_channel+channel.devEui);
            if (!this.list[hashL1])
            {
                this.set_level_1(nameL1,addressL1);
                var self = this;
                this.list[hashL1] = {
                    name:nameL1,
                    address:addressL1,
                    self:self,
                    check:function(){
                       this._check=!this._check;
                    },
                    list:new Object(),
                    getChannels: function(param,value){
                        var result = new Object();
                        var dublicates = [];
                        for(var key in this.list)
                        {
                            var l2 = this.list[key];
                            if (l2.list)
                            {
                               for(var key2 in l2.list)
                               { 
                                   
                                   var channel = l2.list[key2];
                                   var uniqKey = key+key2;
                                   channel.uniqKey = uniqKey;
                                   var hk = channel.$$hashKey.split(':' )[1];
                                   if (channel[param] == value && !dublicates[hk] )
                                   {
                                       dublicates[hk]=true
                                       result[uniqKey] = channel;
                                   }
                               }
                            }
                        }
                        return result;
                    },
                    checkPresenceTypeChannel: function(type){
                        for(var key in this.list)
                        {
                            var l2 = this.list[key];
                            if (l2.list)
                            {
                               for(var key in l2.list)
                               { 
                                   var channel = l2.list[key];
                                   if (channel.type_channel == type)
                                   {
                                       return true;
                                   }
                               }
                            }
                        }
                        return false;
                    },
                    valid: function(){
                        if (this.list)
                        {
                            for(var key in this.list)
                            {
                                var l2 = this.list[key];
                                if (l2.list)
                                {
                                   for(var key in l2.list)
                                   { 
                                       var channel = l2.list[key];
                                       if (channel.name&&this.name==channel.name_level_1&&this.address==channel.address_level_1&&channel.level_2==l2.name) 
                                       {
                                           var filter = this.self.filterDevices;
                                           var fullFilter = this.self.filter;
                                           var checkFullFilter = this.self.checkFilter();
                                           if (filter||checkFullFilter)
                                           {
                                               filter = filter.toLowerCase();
                                               var name_lavel_1 = this.name.toLowerCase();
                                               var address_lavel_1 = this.address.toLowerCase();
                                               var validlevel1 = name_lavel_1.indexOf(filter)>-1||address_lavel_1.indexOf(filter)>-1;
                                               if (!checkFullFilter&&validlevel1) return true;
                                               for(var keyLevel2 in this.list)
                                               {
                                                   var l2 = this.list[keyLevel2];
                                                   var validLevel2 = l2.name&&l2.name.toLowerCase().indexOf(filter)>-1;
                                                   if (!checkFullFilter&&validLevel2) return true;
                                                   for(var keyChannel in l2.list)
                                                   {
                                                       var thischannel = l2.list[keyChannel];
                                                       var nameChannel = thischannel.name;
                                                       var devEuiChannel = thischannel.devEui;
                                                       var validNameChannel = typeof nameChannel === 'string';
                                                       var validDeviceModel = thischannel._self.device_type == fullFilter.deviceModel || fullFilter.deviceModel == undefined || fullFilter.deviceModel == 0;
                                                       if (validNameChannel&&nameChannel) nameChannel=nameChannel.toLowerCase();
                                                       if (devEuiChannel) devEuiChannel=devEuiChannel.toLowerCase();
                                                       var validChannel = validNameChannel&&(nameChannel.indexOf(filter)>-1||devEuiChannel.indexOf(filter)>-1);
                                                       if(checkFullFilter&&validDeviceModel)
                                                       {
                                                            if(filter)
                                                            {
                                                                if(validlevel1||validLevel2||validChannel) return true;
                                                                return false;
                                                            }
                                                            return true;
                                                       }
                                                       else if(!checkFullFilter)
                                                       {
                                                        if (validNameChannel&&(nameChannel.indexOf(filter)>-1||devEuiChannel.indexOf(filter)>-1)) return true;
                                                       }
                                                   }
                                               }
                                               return false;
                                           }
                                           return true;
                                       }   
                                   }
                                }
                            }
                        }
                        return false;
                    }
                };
            }
            var level_1 = this.list[hashL1];
            if (!level_1.list[hashL2])
            {
                level_1.list[hashL2] = {
                    name:nameL2,
                    list:new Object(),
                    check:function(){
                       this._check=!this._check;
                    },
                    valid: function(){
                        if (this.list)
                        {
                            for(var key in this.list)
                            { 
                                var channel = this.list[key];
                                if (channel.name&&this.self.name==channel.name_level_1&&this.self.address==channel.address_level_1&&channel.level_2==this.name) 
                                {
                                    var filter = this.self.self.filterDevices;
                                    var fullFilter = this.self.self.filter;
                                    var checkFullFilter = this.self.self.checkFilter();
                                    if (filter||checkFullFilter)
                                    {
                                        filter = filter.toLowerCase();
                                        
                                        var name_lavel_1 = this.self.name.toLowerCase();
                                        var address_lavel_1 = this.self.address.toLowerCase();
                                        var validlevel1 = name_lavel_1.indexOf(filter)>-1||address_lavel_1.indexOf(filter)>-1;
                                        if (!checkFullFilter&&validlevel1) return true;
                                        var l2 = this;
                                        var validLevel2 = l2.name&&l2.name.toLowerCase().indexOf(filter)>-1;
                                        if (!checkFullFilter&&validLevel2) return true;
                                        for(var keyChannel in l2.list)
                                        {
                                            var thischannel = l2.list[keyChannel];
                                            var nameChannel = thischannel.name;
                                            var devEuiChannel = thischannel.devEui;
                                            var validDeviceModel = thischannel._self.device_type == fullFilter.deviceModel || fullFilter.deviceModel == undefined || fullFilter.deviceModel == 0;
                                            if (nameChannel&&devEuiChannel) 
                                            {
                                                nameChannel=nameChannel.toLowerCase();
                                                devEuiChannel=devEuiChannel.toLowerCase();
                                                var validChannel = nameChannel.indexOf(filter)>-1||devEuiChannel.indexOf(filter)>-1;
                                                if(checkFullFilter&&validDeviceModel)
                                                {
                                                    if(filter)
                                                    {
                                                        if(validlevel1||validLevel2||validChannel) return true;
                                                        return false;
                                                    }
                                                    return true;
                                                }
                                                else if(!checkFullFilter)
                                                {
                                                    if (nameChannel.indexOf(filter)>-1||devEuiChannel.indexOf(filter)>-1&&validDeviceModel) return true;
                                                }
                                                
                                            }
                                        }
                                        return false;
                                    }
                                    return true;
                                }
                            }
                        }
                        return false;
                    }
                };
                level_1.list[hashL2].self = level_1;
            }
            this.set_level_2(nameL2);
            var level_2 = level_1.list[hashL2];
            level_2.list[hashL3] = channel;           
            this.set_level_3(channel.name);
        }
    }
    getCountL1()
    {
        return Object.keys(this.list).length;
    }
    getCountL2()
    {
        var tmp = new Object();
        for(var key in this.list)
        {
            var l1 = this.list[key];
            if ( l1 && isObject(l1) )
            {
                for(var hash in l1.list)
                {
                    tmp[hash] = true;
                }
            }
        }
        return Object.keys(tmp).length;
    }
    set_level_1(name,address)
    {
        if (name&&name!=undefined&&address&&address!=undefined)
        {
            var temp = new Object();
            temp.name_level_1=name;
            temp.address_level_1=address;
            temp.danger=false;
            for (var i=0;i<this.level_1_list.length;i++)
            {
                if (this.level_1_list[i].name_level_1==name&&this.level_1_list[i].address_level_1==address)
                {
                    return false;
                }
            }
            this.level_1_list.push(temp);
            return true;
        }
    }
    set_level_2(name)
    {
        if (name!=undefined&&name&&this.level_2_list.indexOf(name.toString())==-1)
        {
            this.level_2_list.push(name.toString());
            this.level_2_filter_list.push({name:name.toString(),tick:true});
        }
    }
    set_level_3(name)
    {
        if (name!=undefined&&name&&this.level_3_list.indexOf(name.toString())==-1)
        {
            this.level_3_list.push(name.toString());
        }
    }
}


